# BARBARIAN_CIV_MOD
#
# by jdog5000
# Version 1.5
#


from CvPythonExtensions import *
import CvUtil
import PyHelpers		
import Popup as PyPopup
import RevUtils
import SdToolKitCustom as SDTK

# globals
gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
game = CyGame()

class BarbarianCiv :

    def __init__(self, customEM, config ) :

        self.config = config
        
        self.EventKeyDown=6

        self.newPlayerIdx = None

        self.maxCivs = config.getint("Revolution", "MaxCivs", gc.getMAX_CIV_PLAYERS())
        if( self.maxCivs <= 1 ) :
            self.maxCivs = gc.getMAX_CIV_PLAYERS()
        
        # For Python implementation only
        self.doGameStartKill = self.config.getboolean("BarbarianCiv", "DoGameStartKill", False)

        # Number of civs to kill at game start, to free space for new civs later
        # Python implementation only
        self.killFraction = self.config.getfloat("BarbarianCiv", "KillFraction", .3)
        
        # Radius around one city considered "close" when forming civ from barb cities
        # Reduce for smaller maps?
        self.cityRadius = self.config.getint("BarbarianCiv", "CityRadius", 10)

        # Minimum population needed for a group of barb cities to settle
        self.minPopulation = self.config.getint("BarbarianCiv", "MinPopulation", 1)

        # Allow barb cities to turn into minor civs
        self.allowMinor = self.config.getboolean("BarbarianCiv", "AllowMinor", False)

        # Affects number of units given to militaristic barbs        
        self.militaryStrength = self.config.getfloat("BarbarianCiv", "MilitaryStrength", 1.0)        

        # If city was founded by other than barbs, bring back a similar style civ (if possible)
        self.barbCivsByStyle = self.config.getboolean("BarbarianCiv", "BarbCivsByStyle", True)

        # Max number of turns after a militaristic barb civ starts alone will they declare war on first contact
        self.vikingWindow = self.config.getint("BarbarianCiv", "MaxVikingStyleWindow", 30)

        # Allow cities to flip to barbs by culture
        self.doCultureCheck = self.config.getboolean("BarbarianCiv", "DoCultureCheck", True)

        # How close other civ's capital needs to be for Barb civ to immeadiately declare war
        self.warCloseDist = self.config.getint("BarbarianCiv", "WarCloseDist", 21)

        # Offer control of new barb civs
        self.offerControl = self.config.getboolean("BarbarianCiv", "OfferControl", False)

        # popup alert when new civ is formed from barb cities (forced true if offering control)
        self.LOG_BARB_SETTLE = self.config.getboolean("BarbarianCiv", "BarbSettlePopup", True) or self.offerControl
        self.blockPopupInAuto = self.config.getboolean("BarbarianCiv", "BlockPopupInAuto", True)
        self.barbSettlePopup = 7008
        # enable debug log and functions
        self.LOG_DEBUG  = self.config.getboolean("BarbarianCiv", "DebugMode", False)    

        self.customEM = customEM

        if( self.doGameStartKill ) : self.customEM.addEventHandler( "GameStart", self.onGameStart )
        self.customEM.addEventHandler( "BeginGameTurn", self.onBeginGameTurn )
        self.customEM.addEventHandler( "firstContact", self.onFirstContact )
        self.customEM.setPopupHandler( self.barbSettlePopup, ["barbSettlePopup",self.barbSettleHandler,self.blankHandler] )
        if( self.LOG_DEBUG ) :
            self.customEM.addEventHandler( "kbdEvent", self.onKbdEvent )
            CvUtil.pyPrint("Initializing BarbarianCiv Mod")
            print "Settings: DoGSK: %d Frac: %f CityRad: %d AlMinor: %d ByStyle: %d VikWin: %d DoCul: %d BarbSet: %d dbg: %d" \
            %(self.doGameStartKill, self.killFraction, self.cityRadius, self.allowMinor, self.barbCivsByStyle, self.vikingWindow, self.doCultureCheck, self.LOG_BARB_SETTLE, self.LOG_DEBUG)


    def blankHandler( self, playerID, netUserData, popupReturn ) :

            # Dummy handler to take the second event for popup
            return                                    

    def onGameStart(self, argsList) :
        'kill off some civs'
        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Killing off some civs")

        # This function is designed for Python only implementations.
        # It kills off a few civs to free up some slots.
        # These slots may then be filled by barb cities turning into full civs
        numToKill = 0
        if( self.killFraction <= 1 ) :
            numToKill = int(round( self.killFraction*(game.countCivPlayersAlive() - game.getNumHumanPlayers()) ))

        if( self.LOG_DEBUG ) :
            popup = PyPopup.PyPopup()
            bodStr = 'Killing off %d civs to make space for barb civs'%(numToKill)
            popup.setBodyString( bodStr )
            popup.launch()

        loopCount = 0
        while( numToKill > 0 ) :
            loopCount += 1
            unluckyIdx = game.getSorenRandNum(gc.getMAX_CIV_PLAYERS(),'BarbarianCiv: Killing some player')
            unlucky = gc.getPlayer(unluckyIdx)
            if( unlucky.getNumUnits() > 0 and not unlucky.isHuman() ) :
                unlucky.killUnits()
                unlucky.killCities()
                numToKill -= 1
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Killed off the %s, idx: %d"%(unlucky.getCivilizationDescription(0),unluckyIdx))
            if( loopCount > 500 ) :
                break

    def barbSettleHandler( self, iPlayerID, netUserData, popupReturn ) :
           
            if( popupReturn.getButtonClicked() == 0 ) :
                if( self.LOG_DEBUG ) : print ("BarbCiv - Clicked option 0, no change")

            elif( popupReturn.getButtonClicked() == 1 ) :
                RevUtils.changeHuman( self.newPlayerIdx, iPlayerID )
                

    def onBeginGameTurn( self, argsList ) :
        'check for worthy barb cities'
        iGameTurn = argsList[0]

        self.checkBarbCities()
        

    def onKbdEvent(self, argsList ):
        'keypress handler'
        eventType,key,mx,my,px,py = argsList

        if ( eventType == self.EventKeyDown ):
	    theKey=int(key)
	    
            # For debug or trial only
	    if( theKey == int(InputTypes.KB_B) and self.customEM.bShift and self.customEM.bCtrl ) :
                cityList = self.checkBarbCities(force = True)

    def checkBarbCities( self, force = False, forceCul = False ) :
        'Look for barb cities with neighbors or culture'
        #if( self.LOG_DEBUG ) : CyInterface().addImmediateMessage("Checking Barbarian Cities","")
        
        barbPlayer = PyPlayer( gc.getBARBARIAN_PLAYER() )

        if( barbPlayer.getNumCities() >= 1 ) :
            
            barbCityList = barbPlayer.getCityList()
            # ith entry of Tree contains list of cities close to ith entry of city list
            barbCityTree = list()
            cultureList = list()

            for i in range(0,len(barbCityList)) :

                ix = barbCityList[i].getX()
                iy = barbCityList[i].getY()
                closeCityList = list()
                closeCityList.append(barbCityList[i])
                cultureList.append(barbCityList[i].getCulture())

                # Build a list of cities close to this city
                for j in range(0,len(barbCityList)) :
                    if( not barbCityList[j] in closeCityList ) :
                        jx = barbCityList[j].getX()
                        jy = barbCityList[j].getY()
                        # I think this will catch cities close over edge of map ...
                        if( plotDistance(ix,iy,jx,jy) < self.cityRadius ) :
                            closeCityList.append(barbCityList[j])

                barbCityTree.append(closeCityList)

            # Find city with most neighbors cities
##            maxListLen = 0
##            maxListIdx = 0
##            for i,cityList in enumerate(barbCityTree) :
##                if( len(cityList) > maxListLen ) :
##                    maxListLen = len(cityList)
##                    maxListIdx = i
            
            # Find city with highest neighboring population
            maxListPop = 0
            maxListIdx = 0
            for i,cityList in enumerate(barbCityTree) :
                pop = 0
                for j in range(0,len(cityList)) :
                    pop += cityList[j].getPopulation()
                if( pop > maxListPop ) :
                    maxListPop = pop
                    maxListIdx = i

            maxCultureIdx = 0
            maxCulture = 0
            for i,culture in enumerate(cultureList) :
                if( culture >= maxCulture ) :
                    maxCulture = culture
                    maxCultureIdx = i                

            if( False and self.LOG_DEBUG ) :
                popup = PyPopup.PyPopup()
                bodStr = localText.getText("TXT_KEY_BARBCIV_CHECK_BARB", ())%(len(barbCityList),maxListLen,self.cityRadius)
                popup.setBodyString( bodStr )
                popup.launch()

            
            pop = maxListPop
            cl = int( barbCityList[i].getCulture()/2.0 + barbCityList[i].GetCy().countTotalCultureTimes100()/1000.0 )
            # Odds modifier from config file
            mod = self.config.getfloat("BarbarianCiv", "Modifier", 1.00)
            # If lots of barb cities, increase odds
            numCitiesMod = int(pow(len(barbCityList),.5))
            # Change by gamespeed
            gsm = (661.0)/(game.getEstimateEndTurn() + 1.0)
            if( gsm == 661.0 ) : gsm = 1.0
            # Odds of barb city settling down go up by era
            sumEra = 0
            for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( gc.getPlayer(idx).isAlive() ) :
                    sumEra += gc.getPlayer(idx).getCurrentEra()
            eraMod = sumEra/(1.0*game.countCivPlayersAlive()) - game.getStartEra()/2.0
            if( eraMod < .5 ) :
                eraMod = .4

            popChance = int(mod*numCitiesMod*gsm*eraMod*1.5*(pop*pop))
            culChance = int(mod*gsm*eraMod*cl)

            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Barb odds: %d pop, %d cul %d"%(popChance,culChance,cl))

            
            if( pop > self.minPopulation and popChance > (game.getSorenRandNum(999,'BarbarianCiv: Check barbs')) or force ) :
                #if( self.LOG_DEBUG ) : CyInterface().addImmediateMessage("Attempting to reincarnate civ, giving them %d cities"%(len(barbCityTree[maxListIdx])),"")
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Population barb roll succeeded, with %d cities, %d pop, forced? %d"%(len(barbCityTree[maxListIdx]),maxListPop,force))
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Current modifiers are: mod %f, ncm %d, gsm %f"%(mod,numCitiesMod,gsm))
                self.createBarbCiv( barbCityTree[maxListIdx] )
                return
            elif( self.doCultureCheck and culChance > (game.getSorenRandNum(99,'BarbarianCiv: Check barbs')) or forceCul ) :
                # Check for cultured cities
                #if( self.LOG_DEBUG ) : CyInterface().addImmediateMessage("Attempting to reincarnate civ, giving them a cultured city","")
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Cult city barb roll succeeded, with %d adj cult, forced? %d"%(cl,forceCul))
                self.createBarbCiv( [barbCityList[maxCultureIdx]] )
                return
                
        else :
            if( force or forceCul ) :
                CyInterface().addImmediateMessage("No Barbarian cities found!","")

    def createBarbCiv( self, cityList ) :
        'Reincarnates a civ, giving them the cities in cityList'

        if( game.countCivPlayersAlive() < self.maxCivs ) :
            #if( self.LOG_DEBUG ) : CyInterface().addImmediateMessage("Creating new civ ...","")
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Creating new civ ...")

            if( len(cityList) < 1 ) :
                return
            
            newPlayerIdx = -1
            # Check for dead civ with culture in the city
            for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( not gc.getPlayer(i).isAlive() and cityList[0].GetCy().getCulture(i) > 30 ) :
                    newPlayerIdx = i
                    break

            if( newPlayerIdx < 0 ) :
                # Put in empty slot
                for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                    if( not gc.getPlayer(i).isEverAlive() and (gc.getPlayer(i).getNumUnits() == 0) ) :
                        newPlayerIdx = i
                        break

                # Overwrite a civ that never managed to found a city and hasn't lost any cities (maybe a rebel)
                if( newPlayerIdx < 0 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - No empty slots, looking for 'fake' civs")
                    for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                        if( (not gc.getPlayer(i).isAlive()) and (not gc.getPlayer(i).isFoundedFirstCity()) and gc.getPlayer(i).getCitiesLost() == 0 and (gc.getPlayer(i).getNumUnits() == 0) ) :
                            newPlayerIdx = i
                            break

                if( newPlayerIdx < 0 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - no 'fake' civs to replace, aborting barb civ creation")
                    return

                # Check if barb city has culture from a non-barb player
                cultPlayer = None
                cultPlayerID = cityList[0].GetCy().findHighestCulture()
                if( cultPlayerID >= 0 and cultPlayerID < gc.getBARBARIAN_PLAYER() ) :
                    cultPlayer = gc.getPlayer( cultPlayerID )
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civs new capital was culturally influenced by %s"%(cultPlayer.getCivilizationDescription(0)))

                # Can reincarnate as a minor civ if you want
                iMinor = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_MINOR')
                # Will cause problems when looking for capital later on, as barbs have no capital ... otherwise works
                iBarbarian = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_BARBARIAN')
                # Civs not currently in the game
                availableCivs = list()
                # Civs with similar style to cultPlayer, if they exist
                similarStyleCivs = list()
                for civType in range(0,gc.getNumCivilizationInfos()) :
                    if( not civType == iBarbarian ) :
                        if( not civType == iMinor or self.allowMinor ) :
                            taken = False
                            for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                                if( civType == gc.getPlayer(i).getCivilizationType() ) :
                                    if( gc.getPlayer(i).isAlive() or gc.getPlayer(i).isFoundedFirstCity() or gc.getPlayer(i).getCitiesLost() > 0 ) :
                                        taken = True
                                        break
                            if( not taken ) :
                                availableCivs.append(civType)
                                if( not cultPlayer == None ) :
                                    if( gc.getCivilizationInfo( cultPlayer.getCivilizationType() ).getArtStyleType() == gc.getCivilizationInfo(civType).getArtStyleType() ) :
                                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Potential similar style civ: %s"%(gc.getCivilizationInfo(civType).getShortDescription(0)))
                                        similarStyleCivs.append(civType)

                if( len(availableCivs) < 1 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - ERROR: Unexpected lack of unused civ types")
                    return

                if( len(similarStyleCivs) > 0 and self.barbCivsByStyle ) :
                    # Create a civ using this style
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Creating a similar style civ to %s"%(cultPlayer.getCivilizationDescription(0)))
                    availableCivs = similarStyleCivs
                        
                newCivIdx = availableCivs[game.getSorenRandNum(len(availableCivs),'BarbarianCiv: pick unused civ type')]

                # Choose a leader for the new civ
                leaderList = list()
                for leaderType in range(0,gc.getNumLeaderHeadInfos()) :
                    if( gc.getCivilizationInfo(newCivIdx).isLeaders(leaderType) ) :
                        leaderList.append(leaderType)

                if( len(leaderList) < 1 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - ERROR: Unexpected lack of possible leaders")
                    return
                
                newLeaderIdx = leaderList[game.getSorenRandNum(len(leaderList),'BarbarianCiv: pick leader')]
                    
                #if( self.LOG_DEBUG ) : CyInterface().addImmediateMessage("Attempting to create the %s at idx: %d, with leader %s"%( gc.getCivilizationInfo(newCivIdx).getShortDescription(0),newPlayerIdx,gc.getLeaderHeadInfo(newLeaderIdx).getLeaderHead()),"")
           
                ix = cityList[0].getX()
                iy = cityList[0].getY()
                cityPlot = cityList[0].plot()

                # If someone other than barbs owned this city previously, military buildup will declare war on them!
                prevCapitalOwner = None 
                if( cityList[0].GetCy().getPreviousOwner() >= 0 and cityList[0].GetCy().getPreviousOwner() < gc.getBARBARIAN_PLAYER() ) : 
                    prevCapitalOwner = cityList[0].GetCy().getPreviousOwner()
                    
                try :
                    # SDK version, -1 for put alone on new team
                    success = game.createNewPlayer( newPlayerIdx, newCivIdx, newLeaderIdx, -1, True, ix, iy )
                except :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  ***BC - DLL not found, using Python only")
                    availableCivs = list()
                    for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                        if( (not gc.getPlayer(i).isAlive()) and gc.getPlayer(i).isEverAlive() ) :
                            # Dead, but was previously alive
                            if( (not gc.getPlayer(i).isFoundedFirstCity()) and gc.getPlayer(i).getCitiesLost() == 0 and (gc.getPlayer(i).getNumUnits() == 0) ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Python imp: Player %d is available with civ type %d"%(i,gc.getPlayer(i).getCivilizationType()))
                                availableCivs.append(i)
                        elif( not gc.getPlayer(i).isEverAlive() ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Never alive player %d has civ type %d"%(i,gc.getPlayer(i).getCivilizationType()))

                    if( len(availableCivs) < 1 ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Python imp: No available players, aborting barb civ settle")
                        success = False
                        newCivIdx = -1
                        newPlayerIdx = -1
                        return
                    else :
                        success = True
                        newPlayerIdx = availableCivs[game.getSorenRandNum(len(availableCivs),'BarbarianCiv: Py imp pick dead civ')]
                        gc.getPlayer(newPlayerIdx).acquireCity( cityList[0].GetCy(), False, False )

                self.newPlayerIdx = newPlayerIdx

                if( not success ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - FAILED to reincarnate civ type %d in slot %d"%(newCivIdx,newPlayerIdx))
                    return

            if( gc.getPlayer(newPlayerIdx).getNumCities() > 0 ) :

                newPlayer = gc.getPlayer(newPlayerIdx)
                pyNewPlayer = PyPlayer(newPlayerIdx)
                civName = newPlayer.getCivilizationDescription(0)
                leadName = newPlayer.getName()

                totalEra = 0
                count = 0
                for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                    if( gc.getPlayer(idx).isAlive() ) :
                       totalEra += gc.getPlayer(idx).getCurrentEra()
                       count += 1

                avgEra = totalEra/(1.0*count)

                # Announce the barb civ settling
                if( self.LOG_BARB_SETTLE ) :
                    bodStr = bodStr = localText.getText("TXT_KEY_BARBCIV_SPREAD", ())%(leadName,civName)
                    if( not self.blockPopupInAuto or game.getAIAutoPlay() == 0 ) :
                        popup = PyPopup.PyPopup(self.barbSettlePopup,contextType = EventContextTypes.EVENTCONTEXT_ALL)
                        if( self.offerControl ) : bodStr += localText.getText("TXT_KEY_BARBCIV_SPREAD_OPT", ())%(leadName)
                        popup.setBodyString( bodStr )
                        popup.addButton( localText.getText("TXT_KEY_BARBCIV_OK", ()) )
                        if( self.offerControl ) :
                            bodStr += popup.addButton(localText.getText("TXT_KEY_BARBCIV_OPT_AM", ())%(leadName))
                        popup.launch(bCreateOkButton = False)
                    else :
                        CyInterface().addImmediateMessage(bodStr,"")
                    
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - %s has organized the Barbarians and formed the %s!  Turn: %d, Year: %d"%(leadName,civName,game.getElapsedGameTurns(),game.getTurnYear(game.getElapsedGameTurns())))
                                                      
                if( len(cityList) > 1 ) :
                    #if( self.LOG_DEBUG ) : CyInterface().addImmediateMessage("Giving the %s %d additional cities"%(civName,len(cityList)-1),"")
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving the %s %d additional cities"%(civName,len(cityList)-1))
                    for i,giftCity in enumerate(cityList[1:]) :
                        giftCity.changePopulation(1)
                        # Using following method to acquire city produces 'revolted and joined' replay messages
                        giftCity.plot().setOwner( newPlayer.getID() )

                # Note: city acquisition may invalidate previous city pointer, so have to create new list of cities                       
                cityList = pyNewPlayer.getCityList()
                capital = pyNewPlayer.getCapitalCity()
                # Find nearby civs
                newPlayerArea = capital.GetCy().area()
                closePlayers = list()
                closeTeams = list()
                for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                    if( not idx == newPlayerIdx ) :
                        if( newPlayerArea.getCitiesPerPlayer(idx) > 0 ) :
                            
                            playerI = gc.getPlayer(idx)
                            capitalI = playerI.getCapitalCity()
                            if( plotDistance( capital.getX(), capital.getY(), capitalI.getX(), capital.getY() ) < self.warCloseDist/2 ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - %s are very close to new barb civ"%(playerI.getCivilizationDescription(0)))
                                closePlayers.append(idx)
                            if( plotDistance( capital.getX(), capital.getY(), capitalI.getX(), capital.getY() ) < self.warCloseDist ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - %s are close to new barb civ"%(playerI.getCivilizationDescription(0)))
                                closePlayers.append(idx)

                            if( plotDistance( capital.getX(), capital.getY(), capitalI.getX(), capital.getY() ) < 25 ) :
                                if( not playerI.getTeam() in closeTeams ) :
                                    closeTeams.append(playerI.getTeam())
                                

                # Pick a style for barbarian organization
                militaryOdds = int( 100*self.config.getfloat("BarbarianCiv", "MilitaryBuildupBaseOdds", 0.50) )
                iAgg = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGGRESSIVE')
                iExp = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_EXPANSIVE')
                if( gc.getLeaderHeadInfo( newPlayer.getLeaderType() ).hasTrait( iAgg ) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - %s is aggressive"%(leadName))
                    militaryOdds += 25
                elif( gc.getLeaderHeadInfo( newPlayer.getLeaderType() ).hasTrait( iExp ) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - %s is expansionist"%(leadName))
                    militaryOdds += 15
                if( len(closePlayers) == 0 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - New civ is all alone")
                    militaryOdds -= 15
                if( not cityList[0].GetCy().getOriginalOwner() == gc.getBARBARIAN_PLAYER() ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - New civ's capital not built by barbs")
                    militaryOdds += 20

                barbSettleType = 0
                playerDict = {}
                playerDict['SpawnTurn'] = game.getElapsedGameTurns()
                iRand = game.getSorenRandNum(100,'BarbarianCiv: Check barbs')
                if( militaryOdds > iRand ) :
                    # Military consolidation
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Military buildup, roll: %d > %d"%(militaryOdds,iRand))
                    barbSettleType = 1
                    playerDict['BarbStyle'] = 'Military'
                    SDTK.sdObjectSetDict( 'BarbarianCiv', newPlayer, playerDict )
                else :
                    # Builder consolidation
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Builder buildup, roll: %d <= %d"%(militaryOdds,iRand))
                    barbSettleType = 2
                    playerDict['BarbStyle'] = 'Builder'
                    SDTK.sdObjectSetDict( 'BarbarianCiv', newPlayer, playerDict )

                # Giving the new civ free stuff!
                # Base stuff
                newPlayer.changeGold( 150 + game.getSorenRandNum(150,'BarbarianCiv: give money') )
                
                barbTechFrac = self.config.getfloat("BarbarianCiv", "BarbTechFrac", 0.60)
                barbTeam = gc.getTeam( gc.getBARBARIAN_TEAM() )
                for techID in range(0,gc.getNumTechInfos()) :
                    numTeams = game.countCivTeamsAlive() + 1
                    numTeamsWhoKnow = game.countKnownTechNumTeams(techID)
                    fracKnow = numTeamsWhoKnow/(1.0*numTeams)
                    if( fracKnow >= barbTechFrac and newPlayer.canEverResearch(techID) ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - OLD: Giving %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
                        #gc.getTeam(newPlayer.getTeam()).setHasTech(techID,True,newPlayerIdx,False,False)

                    
                    if( barbTeam.isHasTech(techID) ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - NEW: Giving %s   from barb"%(PyInfo.TechnologyInfo(techID).getDescription()))
                        gc.getTeam(newPlayer.getTeam()).setHasTech(techID,True,newPlayerIdx,False,False)
                        continue

                    localNumTeamsWhoKnow = 0
                    for idx in closeTeams :
                        teamI = gc.getTeam(idx)
                        if( teamI.isHasTech(techID) ) :
                            localNumTeamsWhoKnow += 1

                    if( len(closeTeams) > 0 ) :
                        modFracKnow = fracKnow/2.0 + localNumTeamsWhoKnow/(2.0*(len(closeTeams) + .5))
                        if( modFracKnow >= barbTechFrac and newPlayer.canEverResearch(techID) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - NEW: Giving %s   %f with %f loc"%(PyInfo.TechnologyInfo(techID).getDescription(),modFracKnow,localNumTeamsWhoKnow/(1.0*len(closeTeams)+.5)))
                            gc.getTeam(newPlayer.getTeam()).setHasTech(techID,True,newPlayerIdx,False,False)
                    else :
                        fracKnow -= self.config.getfloat("BarbarianCiv", "NewWorldReduction", 0.15)
                        if( fracKnow >= barbTechFrac and newPlayer.canEverResearch(techID) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - NEW: Giving %s   %f no loc"%(PyInfo.TechnologyInfo(techID).getDescription(),fracKnow))
                            gc.getTeam(newPlayer.getTeam()).setHasTech(techID,True,newPlayerIdx,False,False)
                        
                        

                # One great person: merhcant, artist, prophet, scientific, engineer, or general
                iProphet = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_PROPHET')
                iArtist = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ARTIST')     # Will seemingly always be used for culture
                iMerchant = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_MERCHANT')
                iScientist = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_SCIENTIST')
                iEngineer = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_ENGINEER')
                iGeneral = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_GREAT_GENERAL')

                if( barbSettleType == 1 ) : gpList = [iProphet,iProphet,iScientist,iGeneral,iGeneral,iEngineer]
                else : gpList = [iProphet,iProphet,iArtist,iMerchant,iScientist,iScientist,iEngineer,iEngineer]
                gpType = gpList[game.getSorenRandNum(len(gpList),'BarbarianCiv: pick leader')]
                newPlayer.initUnit(gpType,ix,iy,UnitAITypes.NO_UNITAI)
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - GP type %s"%(PyInfo.UnitInfo(gpType).getDescription()))
                
                # Special stuff depending on type
                if( barbSettleType == 1 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Military buildup!")

                    iHorseback = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_HORSEBACK_RIDING')
                    iSailing = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_SAILING')
                    iWheel = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_THE_WHEEL')
                    iHusbandry = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_ANIMAL_HUSBANDRY')
                    iBronze = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_BRONZE_WORKING')
                    iIron = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_IRON_WORKING')
                    iGuilds = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_GUILDS')

                    # Pick militaristic flavor: viking, horse, chariot, balance etc
                    milStyle = None
                    if( newPlayer.countNumCoastalCities() > newPlayer.getNumCities()/2 ) :
                        if( len(closePlayers) == 0 ) :
                            vikingOdds = 75
                        else :
                            vikingOdds = 30

                        if( game.getSorenRandNum(100,'BC: Viking style') < vikingOdds ) :
                            milStyle = 'viking'
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - VIKING style!")
                            if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iSailing) ) :
                                gc.getTeam(newPlayer.getTeam()).setHasTech(iSailing,True,newPlayerIdx,False,False)
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iSailing).getDescription()))
                            if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iBronze) ) :
                                gc.getTeam(newPlayer.getTeam()).setHasTech(iBronze,True,newPlayerIdx,False,False)
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iBronze).getDescription()))
                            else :
                                if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iIron) ) :
                                    gc.getTeam(newPlayer.getTeam()).setHasTech(iIron,True,newPlayerIdx,False,False)
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iIron).getDescription()))


                    if( milStyle == None ) :
                        if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iHorseback) ) :
                            if( game.getSorenRandNum(100,'BC: Chariot style') < 30 ) :
                                milStyle = 'chariot'
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - CHARIOT style!")
                                if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iWheel) ) :
                                    gc.getTeam(newPlayer.getTeam()).setHasTech(iWheel,True,newPlayerIdx,False,False)
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iWheel).getDescription()))
                                if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iHusbandry) ) :
                                    gc.getTeam(newPlayer.getTeam()).setHasTech(iHusbandry,True,newPlayerIdx,False,False)
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iHusbandry).getDescription()))
                                if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iBronze) ) :
                                    gc.getTeam(newPlayer.getTeam()).setHasTech(iBronze,True,newPlayerIdx,False,False)
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iBronze).getDescription()))
                                else :
                                    if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iIron) ) :
                                        gc.getTeam(newPlayer.getTeam()).setHasTech(iIron,True,newPlayerIdx,False,False)
                                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iIron).getDescription()))

                    if( milStyle == None ) :
                        if( game.getSorenRandNum(100,'BC: Horse style') < 35 ) :
                            milStyle = 'horse'
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - HORSE style!")
                            if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iHorseback) ) :
                                gc.getTeam(newPlayer.getTeam()).setHasTech(iHorseback,True,newPlayerIdx,False,False)
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iHorseback).getDescription()))
                            else :
                                if( game.countKnownTechNumTeams(iGuilds) > 1 ) :
                                    if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iGuilds) ) :
                                        gc.getTeam(newPlayer.getTeam()).setHasTech(iGuilds,True,newPlayerIdx,False,False)
                                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iGuilds).getDescription()))
      
                    if( milStyle == None ) :
                        milStyle = 'balance'
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - BALANCED style!")
                        if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iBronze) ) :
                            gc.getTeam(newPlayer.getTeam()).setHasTech(iBronze,True,newPlayerIdx,False,False)
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iBronze).getDescription()))
                        elif( not gc.getTeam(newPlayer.getTeam()).isHasTech(iIron) ) :
                                gc.getTeam(newPlayer.getTeam()).setHasTech(iIron,True,newPlayerIdx,False,False)
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iIron).getDescription()))
                        elif( not gc.getTeam(newPlayer.getTeam()).isHasTech(iHorseback) ) :
                            gc.getTeam(newPlayer.getTeam()).setHasTech(iHorseback,True,newPlayerIdx,False,False)
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iHorseback).getDescription()))
                        else :
                            if( game.countKnownTechNumTeams(iGuilds) > 1 ) :
                                if( not gc.getTeam(newPlayer.getTeam()).isHasTech(iGuilds) ) :
                                    gc.getTeam(newPlayer.getTeam()).setHasTech(iGuilds,True,newPlayerIdx,False,False)
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving military %s"%(PyInfo.TechnologyInfo(iGuilds).getDescription()))

                    SDTK.sdObjectSetVal( 'BarbarianCiv', newPlayer, 'MilStyle', milStyle )
             
                elif( barbSettleType == 2 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Builder buildup!")

                    # Extra techs
                    if( len(closeTeams) > 0 ) : 
                        maxCount = self.config.getint("BarbarianCiv", "BuilderBonusTechs", 3)
                    else :
                        maxCount = self.config.getint("BarbarianCiv", "BuilderBonusTechs", 3) - 1
                        
                    newTechList = list()
                    for techID in range(0,gc.getNumTechInfos()) :
                        if( not gc.getTeam(newPlayer.getTeam()).isHasTech(techID) and newPlayer.canResearch(techID,True) ) :
                            newTechList.append(techID)

                    for k in range(0,maxCount) :
                        if( len(newTechList) > 0 ) :
                            techID = newTechList[game.getSorenRandNum(len(newTechList),'BC: Builder tech')]
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving builder %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
                            gc.getTeam(newPlayer.getTeam()).setHasTech(techID,True,newPlayerIdx,False,False)
                    
##                        numTeams = game.countCivTeamsAlive() + 1
##                        numTeamsWhoKnow = game.countKnownTechNumTeams(techID)
##                        fracKnow = numTeamsWhoKnow/(1.0*numTeams)
##                        techMod = self.config.getfloat("BarbarianCiv", "BuilderTechModifier", 1.00)
##                        if( not gc.getTeam(newPlayer.getTeam()).isHasTech(techID) and newPlayer.canEverResearch(techID) and techMod*100*fracKnow > game.getSorenRandNum(80,'BarbarianCiv: extra tech') ) :
##                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving builder %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
##                            gc.getTeam(newPlayer.getTeam()).setHasTech(techID,True,newPlayerIdx,False,False)

                # Basic units, find the (potentially unique) unit type for the class for this civ
                workerClass = CvUtil.findInfoTypeNum(gc.getUnitClassInfo,gc.getNumUnitClassInfos(),'UNITCLASS_WORKER')
                iWorker = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(workerClass)
                settlerClass = CvUtil.findInfoTypeNum(gc.getUnitClassInfo,gc.getNumUnitClassInfos(),'UNITCLASS_SETTLER')
                iSettler = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(settlerClass)
                scoutClass = CvUtil.findInfoTypeNum(gc.getUnitClassInfo,gc.getNumUnitClassInfos(),'UNITCLASS_SCOUT')
                iScout = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(scoutClass)

                # Military units, have to vary depending on available techs
                iBestDefender = UnitTypes.NO_UNIT
                iCounter = UnitTypes.NO_UNIT
                iMobile = UnitTypes.NO_UNIT
                iAttack = UnitTypes.NO_UNIT
                iAssaultShip = UnitTypes.NO_UNIT
                for unitID in range(0,gc.getNumUnitInfos()) :
                    if( newPlayer.canTrain(unitID,False,False) and gc.getUnitInfo(unitID).getDomainType() == DomainTypes.DOMAIN_LAND ): # canTrain only does technology check
                        unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                        # Defender (Archer,Longbow)
                        if( gc.getUnitInfo(unitID).getDefaultUnitAIType() == UnitAITypes.UNITAI_CITY_DEFENSE ):
                            if( (iBestDefender == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() >= gc.getUnitInfo(iBestDefender).getCombat() ) :
                                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ def units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                                if( unitID == gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                    iBestDefender = unitID
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Best defender set to %s"%(PyInfo.UnitInfo(iBestDefender).getDescription()))
                        # Counter (Axemen,Phalanx)
                        if( gc.getUnitInfo(unitID).getDefaultUnitAIType() == UnitAITypes.UNITAI_COUNTER ):
                            if( (iCounter == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() >= gc.getUnitInfo(iCounter).getCombat() ) :
                                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ counter units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                                if( unitID == gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                    iCounter = unitID
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Best counter unit set to %s"%(PyInfo.UnitInfo(iCounter).getDescription()))
                        # Assault units
                        if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ATTACK ) ):
                            if( (iAttack == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() > gc.getUnitInfo(iAttack).getCombat() ) :
                                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ attack units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                                if( unitID == gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                    iAttack = unitID
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Best attack set to %s"%(PyInfo.UnitInfo(iAttack).getDescription()))
                        # Fast Attack (Horse archer)
                        if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ATTACK ) and gc.getUnitInfo(unitID).getMoves() > 1 ):
                            if( (iMobile == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() > gc.getUnitInfo(iMobile).getCombat() ) :
                                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ mob units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                                if( unitID == gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                    iMobile = unitID
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Best mobile set to %s"%(PyInfo.UnitInfo(iMobile).getDescription()))

                    elif( newPlayer.canTrain(unitID,False,False) and gc.getUnitInfo(unitID).getDomainType() == DomainTypes.DOMAIN_SEA ):
                        unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                        # Military unit transport ship
                        if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ASSAULT_SEA ) ):
                            if( unitID == gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                iAssaultShip = unitID
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Best assault ship set to %s"%(PyInfo.UnitInfo(iAssaultShip).getDescription()))

                        
                # Use classes to give unique buidlings
                libraryClass = CvUtil.findInfoTypeNum(gc.getBuildingClassInfo,gc.getNumBuildingClassInfos(),'BUILDINGCLASS_LIBRARY')
                iLibrary = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationBuildings( libraryClass )
                #CvUtil.findInfoTypeNum(gc.getBuildingInfo,gc.getNumBuildingInfos(),'BUILDING_LIBRARY')
                granaryClass = CvUtil.findInfoTypeNum(gc.getBuildingClassInfo,gc.getNumBuildingClassInfos(),'BUILDINGCLASS_GRANARY')
                iGranary = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationBuildings( granaryClass )
                barracksClass = CvUtil.findInfoTypeNum(gc.getBuildingClassInfo,gc.getNumBuildingClassInfos(),'BUILDINGCLASS_BARRACKS')
                iBarracks = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationBuildings( barracksClass )
                marketClass = CvUtil.findInfoTypeNum(gc.getBuildingClassInfo,gc.getNumBuildingClassInfos(),'BUILDINGCLASS_MARKET')
                iMarket = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationBuildings( marketClass )
                wallsClass = CvUtil.findInfoTypeNum(gc.getBuildingClassInfo,gc.getNumBuildingClassInfos(),'BUILDINGCLASS_WALLS')
                iWalls = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationBuildings( wallsClass )
                lighthouseClass = CvUtil.findInfoTypeNum(gc.getBuildingClassInfo,gc.getNumBuildingClassInfos(),'BUILDINGCLASS_LIGHTHOUSE')
                iLighthouse = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationBuildings( lighthouseClass )
                forgeClass = CvUtil.findInfoTypeNum(gc.getBuildingClassInfo,gc.getNumBuildingClassInfos(),'BUILDINGCLASS_FORGE')
                iForge = gc.getCivilizationInfo( newPlayer.getCivilizationType() ).getCivilizationBuildings( forgeClass )

                # Put stuff in cities
                for i,city in enumerate(cityList) :
                    cityX = city.getX()
                    cityY = city.getY()
                    if( not iBestDefender == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iBestDefender,cityX,cityY,3)
                    if( not iWorker == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iWorker,cityX,cityY,1)
                    if( city.getPopulation() < 3 ) :
                        city.setPopulation(3)
                    if( city.getPopulation() < 6 ) :
                        city.changePopulation( game.getSorenRandNum(7-city.getPopulation(),'BarbarianCiv: change city population') )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - New population of %s is %d"%(city.getName(),city.getPopulation()))
                    if( barbSettleType == 1 ) : # Military
                        baseUnits = max([3-i,0])
                        extraUnits = int( avgEra )
                        if( milStyle == 'viking' ) :
                            # Viking style, fewer horsies (but still allows elephants ... )
                            if( city.GetCy().isCoastal(10) and not iAssaultShip == UnitTypes.NO_UNIT ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Placing boat in %s"%(city.getName()))
                                pyNewPlayer.initUnit(iAssaultShip,city.getX(),city.getY(),1)
                            iNumCounter = 1 + (extraUnits+1)/2 + game.getSorenRandNum( 1 + baseUnits + extraUnits, 'BC give counter')
                            iNumMobile  = 0 +                game.getSorenRandNum( max([baseUnits + extraUnits-3,2]), 'BC give mobile')
                            iNumAttack  = 1 + extraUnits/2 + game.getSorenRandNum( 1 + baseUnits + extraUnits, 'BC give attack')
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Viking style, in %s placing %d count, %d mob, %d att"%(city.getName(),iNumCounter,iNumMobile,iNumAttack))
                        elif( milStyle == 'chariot' ) :
                            iNumCounter = 1 + extraUnits/2 + game.getSorenRandNum( max([baseUnits + extraUnits-2,2]), 'BC give counter')
                            iNumMobile  = 1 + extraUnits/2 + game.getSorenRandNum( baseUnits + extraUnits, 'BC give mobile')
                            iNumAttack  = 0 +                game.getSorenRandNum( max([baseUnits + extraUnits-4,1]), 'BC give attack')
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Chariot style, in %s placing %d count, %d mob, %d att"%(city.getName(),iNumCounter,iNumMobile,iNumAttack))
                        elif( milStyle == 'horse' ) :
                            iNumCounter = 0 +                game.getSorenRandNum( max([baseUnits + extraUnits-3,2]), 'BC give counter')
                            iNumMobile  = 2 + (extraUnits+1)/2 + game.getSorenRandNum( 1 + baseUnits + extraUnits, 'BC give mobile')
                            iNumAttack  = 0 + extraUnits/2 + game.getSorenRandNum( max([baseUnits + extraUnits-3,2]), 'BC give attack')
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Horse style, in %s placing %d count, %d mob, %d att"%(city.getName(),iNumCounter,iNumMobile,iNumAttack))
                        else :
                            iNumCounter = 1 + extraUnits/2 + game.getSorenRandNum( baseUnits + extraUnits, 'BC give counter')
                            iNumMobile  = 0 +                game.getSorenRandNum( max([baseUnits + extraUnits-1,0]), 'BC give mobile')
                            iNumAttack  = 1 + (extraUnits+1)/2 + game.getSorenRandNum( baseUnits + extraUnits, 'BC give attack')
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Balanced style, in %s placing %d count, %d mob, %d att"%(city.getName(),iNumCounter,iNumMobile,iNumAttack))
 
                        if( not iCounter == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iCounter,cityX,cityY,iNumCounter)
                        if( not iMobile == UnitTypes.NO_UNIT )  : pyNewPlayer.initUnit(iMobile, cityX,cityY,iNumMobile)
                        if( not iAttack == UnitTypes.NO_UNIT )  : pyNewPlayer.initUnit(iAttack, cityX,cityY,iNumAttack)

                        if( city.canConstruct( iBarracks ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Constructing %s in %s"%(PyInfo.BuildingInfo(iBarracks).getDescription(),city.getName()))
                            city.setHasRealBuildingIdx(iBarracks,True)
                    elif( barbSettleType == 2 ) : # Builder
                        if( city.canConstruct( iLibrary ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Constructing %s in %s"%(PyInfo.BuildingInfo(iLibrary).getDescription(),city.getName()))
                            city.setHasRealBuildingIdx(iLibrary,True)
                        if( city.canConstruct( iLighthouse ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Constructing %s in %s"%(PyInfo.BuildingInfo(iLighthouse).getDescription(),city.getName()))
                            city.setHasRealBuildingIdx(iLighthouse,True)
                        if( city.canConstruct( iGranary ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Constructing %s in %s"%(PyInfo.BuildingInfo(iGranary).getDescription(),city.getName()))
                            city.setHasRealBuildingIdx(iGranary,True)
                    iTotCul = city.GetCy().countTotalCultureTimes100()/100
                    if( iTotCul > 0 ) :
                        city.setCulture( int(iTotCul/2.0) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Culture of %s set to %d"%(city.getName(),city.getCulture()))
                        
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - %d units placed in %s"%(city.plot().getNumUnits(),city.getName()))
                        
                # More stuff in capital
                if( not iScout == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iScout,capital.getX(),capital.getY(),1)
                if( not iWorker == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iWorker,capital.getX(),capital.getY(),2)
                if( capital.canConstruct( iWalls ) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Constructing %s in %s"%(PyInfo.BuildingInfo(iWalls).getDescription(),capital.getName()))
                    capital.setHasRealBuildingIdx(iWalls,True)
                if( capital.canConstruct( iForge ) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Constructing %s in %s"%(PyInfo.BuildingInfo(iForge).getDescription(),capital.getName()))
                    capital.setHasRealBuildingIdx(iForge,True)
                if( city.canConstruct( iGranary ) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Constructing %s in %s"%(PyInfo.BuildingInfo(iGranary).getDescription(),city.getName()))
                    city.setHasRealBuildingIdx(iGranary,True)
                if( len(cityList) < 3 ) :
                    if( not iSettler == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iSettler,capital.getX(),capital.getY(),1)
                    if( not iBestDefender == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iBestDefender,capital.getX(),capital.getY(),1)
                if( barbSettleType == 2 ) : # Builder
                    if( not iWorker == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iWorker,capital.getX(),capital.getY(),1)
                    if( not iSettler == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iSettler,capital.getX(),capital.getY(),1)
                    if( not iBestDefender == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iBestDefender,capital.getX(),capital.getY(),1)
                    if( not iCounter == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iCounter,capital.getX(),capital.getY(),2)
                    if( capital.canConstruct( iMarket ) ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Constructing %s in %s"%(PyInfo.BuildingInfo(iMarket).getDescription(),capital.getName()))
                        capital.setHasRealBuildingIdx(iMarket,True)
                    if( newPlayer.getCurrentEra() > 1 ) :
                        if( not iWorker == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iWorker,capital.getX(),capital.getY(),1)
                        if( not iBestDefender == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iBestDefender,capital.getX(),capital.getY(),1)
                elif( barbSettleType == 1 and (len(cityList) == 1 or newPlayer.getCurrentEra() > 2) ) : # Military extra stuff if small or late
                    if( not iCounter == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iCounter,cityX,cityY,1)
                    if( not iMobile == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iMobile,cityX,cityY,1)
                    if( not iAttack == UnitTypes.NO_UNIT ) : pyNewPlayer.initUnit(iAttack,cityX,cityY,1)


                # Give units some experience
                defenseUnits = pyNewPlayer.getUnitsOfType( iBestDefender )
                for unit in defenseUnits :
                    if( 60 > game.getSorenRandNum(100,'BarbarianCiv: unit experience') ) :
                        unit.setExperience( game.getSorenRandNum(7,'BarbarianCiv: unit experience'), 10 )
                if( barbSettleType == 1 ) : # Military
                    mobileUnits = pyNewPlayer.getUnitsOfType( iMobile )
                    for unit in mobileUnits :
                        unit.setExperience( game.getSorenRandNum(7,'BarbarianCiv: unit experience'), 10 )
                    counterUnits = pyNewPlayer.getUnitsOfType( iCounter )
                    for unit in counterUnits :
                        unit.setExperience( game.getSorenRandNum(7,'BarbarianCiv: unit experience'), 10 )
                    attackUnits = pyNewPlayer.getUnitsOfType( iAttack )
                    for unit in attackUnits :
                        unit.setExperience( game.getSorenRandNum(7,'BarbarianCiv: unit experience'), 10 )
                        
                # Diplomacy
                contactList = list()
                for i in range(0, gc.getMAX_CIV_PLAYERS()) :
                    if( newPlayer.canContact( i ) ) :
                        contactList.append( i )

                if( not newPlayer.isAlive() ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Setting new player alive")
                    newPlayer.setNewPlayerAlive(True)

                if( barbSettleType == 1 ) :
                    # Military buildup will declare war on some nearby civ
                    victimIdx = -1
                    closeContactList = list()
                    for closeIdx in closePlayers :
                        if( closeIdx in contactList ) :
                            closeContactList.append(closeIdx)
                    
                    if( not prevCapitalOwner == None and gc.getPlayer( prevCapitalOwner ).isAlive() ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - New civ declaring war on previous city owner, %d"%(prevCapitalOwner))
                        victimIdx = prevCapitalOwner                    
                    elif( len(closeContactList) > 0 ) :
                        vr = game.getSorenRandNum(len(closeContactList),'BarbarianCiv: pick who to declare war on')
                        victimIdx = closeContactList[vr]
                    elif( len(closePlayers) > 0 ) :
                        vr = game.getSorenRandNum(len(closePlayers),'BarbarianCiv: pick who to declare war on')
                        victimIdx = closePlayers[vr]
                    elif( len( contactList ) > 0 ) :
                        vr = game.getSorenRandNum(len(contactList),'BarbarianCiv: pick who to declare war on')
                        victimIdx = contactList[vr]

                    if( victimIdx >= 0 ) :
                        pyNewPlayer.getTeam().declareWar(victimIdx,True)
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - New civ declared war on %s"%(PyPlayer(victimIdx).getCivilizationName()))
                    
                    else :
                        # Give them some boats to go raiding!
                        for i,city in enumerate(cityList) :
                            if( city.GetCy().isCoastal(10) ) :
                                if( not iAssaultShip == UnitTypes.NO_UNIT ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving new civ some boats for raiding")
                                    pyNewPlayer.initUnit(iAssaultShip,city.getX(),city.getY(),2)
                                    break

                # Reveal some surrounding territory
                iGoodyMap = CvUtil.findInfoTypeNum(gc.getGoodyInfo,gc.getNumGoodyInfos(),'GOODY_MAP')
                newCityList = pyNewPlayer.getCityList()
                unitList = pyNewPlayer.getUnitList()
                for city in newCityList :
                    for unit in unitList :
                        if( unit.atPlot(city.plot()) ):
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Giving map around %s"%(city.getName()))
                            newPlayer.receiveGoody( city.GetCy().plot(), iGoodyMap, unit )
                            break
            
        else :
            #if( self.LOG_DEBUG ) : CyInterface().addImmediateMessage("Game is full ...","")
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Game is full ...")


    def onFirstContact( self, argsList ) :
        'Contact'
	iTeamX,iHasMetTeamY = argsList

        #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - %d has just met %d"%(iTeamX,iHasMetTeamY))

        teamX = gc.getTeam(iTeamX)
        if( teamX.getNumMembers() > 1 ) :
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Team %d has too many members: %d"%(iTeamX,teamX.getNumMembers()))          
            return
        
        playerX = None
        playerY = None
        for i in range(0,gc.getMAX_CIV_PLAYERS()) :
            player = gc.getPlayer(i)
            if( player.isAlive() ):
                if( player.getTeam() == iTeamX ) :
                    playerX = player
                elif( player.getTeam() == iHasMetTeamY ) :
                    playerY = player

        if( playerX == None or playerY == None ) :
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Hmmm ... didn't find any players")
            return

        if( playerX.isHuman() ) :
            return

        if( SDTK.sdObjectExists( 'BarbarianCiv', playerX ) ) :
            if( SDTK.sdObjectGetVal( 'BarbarianCiv', playerX, 'BarbStyle' ) == 'Military' ) :
                for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                    if( playerX.canContact( idx ) and not idx == playerY.getID() ) :
                        #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Barb civ %d can also contact %d, no Viking action"%(iTeamX,idx))
                        return

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - The %s (militaristic barb) has just met its first civ, %d"%(playerX.getCivilizationDescription(0),iHasMetTeamY))
	
                if( game.getGameTurn() < (SDTK.sdObjectGetVal( 'BarbarianCiv', playerX, 'SpawnTurn' ) + self.vikingWindow) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - The %s is going Viking on the %s!!!"%(playerX.getCivilizationDescription(0),playerY.getCivilizationDescription(0)))
                    teamX.declareWar(iHasMetTeamY,True)
                    unitList = PyPlayer(playerX.getID()).getUnitList()
                    for unit in unitList :
                        if( not unit.isNone() ) :
                            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - checking out unit %s, id %d"%(unit.getName(),unit.getID()))
                            if( gc.getUnitInfo(unit.getUnitType()).getUnitAIType(UnitAITypes.UNITAI_ASSAULT_SEA) ) :
                                if( unit.isCargo() and not unit.hasCargo() ) :
                                    unit.setUnitAIType( UnitAITypes.UNITAI_ASSAULT_SEA )
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  BC - Setting %s to assault sea"%(unit.getName()))
                
