# RebelTypes.py
#
# by jdog5000
# Version 1.0

# This file sets up the most likely rebel civ types to appear when a revolution occurs in a particular civ.

from CvPythonExtensions import *
import CvUtil

gc = CyGlobalContext()

# Initialize list to empty
RebelTypeList = list()

# This function actually sets up the lists of rebel types for each civ
# All rebel types in this list are equally likely
# If none of these are available, defaults to a similar art style civ
def setup( ) :

    #print "Setting up rebel type list"

    global RebelTypeList

    for idx in range(0,gc.getNumCivilizationInfos()) :
        RebelTypeList.append( list() )

    try :
        iAmerica    = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_AMERICA')
        iArabia     = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_ARABIA')
        iAztec      = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_AZTEC')
        iCarthage   = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_CARTHAGE')
        iCelt       = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_CELT')
        iChina      = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_CHINA')
        iEgypt      = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_EGYPT')
        iEngland    = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_ENGLAND')
        iFrance     = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_FRANCE')
        iGermany    = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_GERMANY')
        iGreece     = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_GREECE')
        iInca       = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_INCA')
        iIndia      = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_INDIA')
        iJapan      = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_JAPAN')
        iKorea      = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_KOREA')
        iMali       = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_MALI')
        iMongol     = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_MONGOL')
        iOttoman    = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_OTTOMAN')
        iPersia     = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_PERSIA')
        iRome       = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_ROME')
        iRussia     = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_RUSSIA')
        iSpain      = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_SPAIN')
        iViking     = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_VIKING')
        iZulu       = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_ZULU')

        # Format is:
        # RebelTypeList[iHomeland] = [iRebel1, iRebel2, iRebel3]
        # No limit on length of rebel list, can be zero

        RebelTypeList[iAmerica]     = [iEngland, iAztec]
        RebelTypeList[iArabia]      = [iEgypt,iPersia,iOttoman,iCarthage]
        RebelTypeList[iAztec]       = [iInca,iSpain,iAmerica]
        RebelTypeList[iCarthage]    = [iRome,iGreece,iMali,iSpain]
        RebelTypeList[iCelt]        = [iFrance,iEngland,iGermany,iSpain]
        RebelTypeList[iChina]       = [iKorea,iMongol,iIndia,iJapan]
        RebelTypeList[iEgypt]       = [iOttoman,iArabia,iPersia,iGreece]
        RebelTypeList[iEngland]     = [iAmerica,iIndia,iZulu]
        RebelTypeList[iFrance]      = [iGermany,iCelt,iEngland,iMali]
        RebelTypeList[iGermany]     = [iFrance,iRussia,iViking]
        RebelTypeList[iGreece]      = [iRome,iPersia,iCarthage,iOttoman]
        RebelTypeList[iInca]        = [iAztec,iSpain]
        RebelTypeList[iIndia]       = [iPersia,iChina,iEngland]
        RebelTypeList[iJapan]       = [iKorea,iChina,iMongol]
        RebelTypeList[iKorea]       = [iJapan,iChina,iMongol]
        RebelTypeList[iMali]        = [iCarthage,iEgypt,iFrance,iZulu]
        RebelTypeList[iMongol]      = [iChina,iRussia,iPersia,iKorea]
        RebelTypeList[iOttoman]     = [iPersia,iGreece,iGermany,iArabia]
        RebelTypeList[iPersia]      = [iOttoman,iIndia,iMongol,iGreece]
        RebelTypeList[iRome]        = [iGreece,iCarthage,iCelt,iEgypt]
        RebelTypeList[iRussia]      = [iViking,iGermany,iMongol,iPersia]
        RebelTypeList[iSpain]       = [iCarthage,iArabia,iAztec,iInca]
        RebelTypeList[iViking]      = [iRussia,iGermany,iEngland,iAmerica]
        RebelTypeList[iZulu]        = [iMali,iArabia,iEgypt]
        
        #print "Completed rebel type list"

    except:
        print "Error!  Rebel types not found, no short lists available"
