# Utility functions for Revolution Mod
#
# by jdog5000
# Version 0.8

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import pickle
import SdToolKitCustom
try :
    import ChangePlayer
except :
    pass

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
game = CyGame()

# The City Script array IDs when loading/saving data
iCityWarningCounterID = 0
iCityRevolutionCivID = 1
iCityRevolutionIndexID = 2
iCityPrevRevIndexID = 3
iCityLocalRevIndexID = 4
iCityRevolutionCounterID = 5
iCityReinforcementCounterID = 6

# civicsList[0] is a list of all civics of option type 0
civicsList = list()

LOG_DEBUG = True
revCultureModifier = 1.0
endWarsOnDeath = True


########################## Event Handling ##########################################################

def registerEvents( customEM, config ) :

        global LOG_DEBUG, revCultureModifier, endWarsOnDeath

        customEM.addEventHandler( 'cityBuilt', onCityBuilt )
        customEM.addEventHandler( 'cityAcquired', onCityAcquired )
        customEM.addEventHandler( 'EndGameTurn', onEndGameTurn )
        customEM.addEventHandler( "setPlayerAlive", onSetPlayerAlive )

        LOG_DEBUG = config.getboolean("Revolution", "DebugMode", False)

        revCultureModifier = config.getfloat("Revolution", "RevCultureModifier", 1.0)
        endWarsOnDeath = config.getboolean("Revolution", "EndWarsOnDeath", True)

        initCivicsList()

def initCivicsList( ) :

    CvUtil.pyPrint("  Rev - Initializing Civics List")

    global civicsList

    for i in range(0,gc.getNumCivicOptionInfos()) :
        civicsList.append(list())

    for i in range(0,gc.getNumCivicInfos()) :
        civicInfo = gc.getCivicInfo(i)
        civicsList[civicInfo.getCivicOptionType()].append(i)
    

def onCityBuilt( argsList ):
        'City Built'
        city = argsList[0]

	initCity( city )

	pPlayer = gc.getPlayer( city.getOwner() )

	if( pPlayer.isBarbarian() ) :
            updateCityVal(city, 'RevolutionIndex', 400 )
            return

	if( not city.area().getID() == pPlayer.getCapitalCity().area().getID() ) :
            updateCityVal(city, 'RevolutionIndex', 350 )
        else :
            updateCityVal(city, 'RevolutionIndex', 250 )

def onCityAcquired( argsList ):
	'City Acquired'

	owner,playerType,city,bConquest,bTrade = argsList

	newOwnerID = city.getOwner()
        newOwner = gc.getPlayer(newOwnerID)
        newOwnerCiv = newOwner.getCivilizationType()
        oldOwnerID = city.getPreviousOwner()
        orgOwnerID = city.getOriginalOwner()		

        if( newOwnerID == gc.getBARBARIAN_PLAYER() and getCityVal(city, 'RevolutionCounter') > 0 ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - City %s captured by barb rebels!"%(city.getName()))
            oldOwner = gc.getPlayer(oldOwnerID)
            
            if( not oldOwnerID == orgOwnerID ) :
                orgOwner = gc.getPlayer(orgOwnerID)
                #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - City %s originally owned by %s!"%(city.getName(),orgOwner.getCivilizationDescription(0)))

                #if( something ) :
                    # Flip back to original owner

                    # Kill nearby barbs                    

            if( city.countTotalCultureTimes100() > 100*100 ) :
                if( not (oldOwnerID == city.findHighestCulture()) ) :
                    cultOwner = gc.getPlayer( city.findHighestCulture() )
                    #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - City %s culturally dominated by %s!"%(city.getName(),cultOwner.getCivilizationDescription(0)))

                    #if( something ) :
                        # Flip to cultural owner

                        # Gift nearby barbs

            initCity(city)

        elif( newOwnerCiv == getCityVal(city, 'RevolutionCiv') ) :

            # Did city recently rebel in their favor or currently unhappy?
            if( getCityVal(city, 'RevolutionCounter') > 0 or (city.unhappyLevel(0) - city.happyLevel()) > 0 ) :
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Rebellious city %s is captured by rebel civ %s (%d)!!!"%(city.getName(),newOwner.getCivilizationDescription(0),newOwnerCiv))

                cultPlayer = None
                if( city.countTotalCultureTimes100() > 100*100 ) :
                    if( not city.getPreviousOwner() == city.findHighestCulture() ) :
                        cultPlayer = gc.getPlayer( city.findHighestCulture() )
                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - New rebel city %s culturally dominated by %s"%(city.getName(),cultPlayer.getCivilizationDescription(0)))

                if( city.plot().countTotalCulture() > 100 ) :
                    if( (not city.plot().calculateCulturalOwner() == city.getPreviousOwner()) and cultPlayer == None ) :
                        cultPlayer = gc.getPlayer( city.plot().calculateCulturalOwner() )
                        if( not cultPlayer == None ) :
                            if( cultPlayer.isBarbarian() ) :
                                cultPlayer = None
                            else :
                                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - New rebel city %s is culturally influenced by %s"%(city.getName(),cultPlayer.getCivilizationDescription(0)))

                newCulVal = int( revCultureModifier*max([city.getCulture(oldOwnerID),city.countTotalCultureTimes100()/200]) )
                newPlotVal = int( revCultureModifier*max([city.plot().getCulture(oldOwnerID),city.plot().countTotalCulture()/2]) )
                giveCityCulture( city, newOwnerID, newCulVal, newPlotVal, overwriteHigher = False )

                ix = city.getX()
                iy = city.getY()

                if( newOwner.isHuman() ) :
                    CyInterface().addImmediateMessage("With your capture of %s, support for the rebellion in the city swells ..."%(city.getName()),"")
                elif( city.isVisible( game.getActiveTeam(), False ) ) :
                    CyInterface().addImmediateMessage("The rebel %s have taken control of %s!  Support for the rebellion swells ..."%(newOwner.getCivilizationDescription(0),city.getName()),"")

                [iWorker,iBestDefender,iCounter,iAttack] = getHandoverUnitTypes( city, newOwner, newOwner )

                newOwner.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                newOwner.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                newOwner.initUnit( iCounter, ix, iy, UnitAITypes.NO_UNITAI )
                newOwner.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )
                if( gc.getPlayer( newOwnerID ).getNumCities() <= 1 ) :
                    # Extra units for first city captured
                    newOwner.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                    newOwner.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )

                # Give a boat to island rebels
                if( city.isCoastal(10) and city.area().getNumCities() < 3 and city.area().getNumTiles() < 25 ) :
                    iAssaultShip = UnitTypes.NO_UNIT
                    for unitID in range(0,gc.getNumUnitInfos()) :
                        if( newOwner.canTrain(unitID,False,False) and gc.getUnitInfo(unitID).getDomainType() == DomainTypes.DOMAIN_SEA ):
                            unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                            # Military unit transport ship
                            if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ASSAULT_SEA ) ):
                                if( unitID == gc.getCivilizationInfo( newOwner.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                    if( iAssaultShip == UnitTypes.NO_UNIT or gc.getUnitInfo(unitID).getCombat() >= gc.getUnitInfo(iAssaultShip).getCombat() ) :
                                        iAssaultShip = unitID
                    if( not iAssaultShip == UnitTypes.NO_UNIT ) :
                        newOwner.initUnit( iAssaultShip, ix, iy, UnitAITypes.UNITAI_ASSAULT_SEA )
                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Rebels get a %s to raid motherland"%(PyInfo.UnitInfo(iAssaultShip).getDescription()))

            # City once rebelled as this civ type, but not currently rebellious
            else :
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - %s, captured by former rebel identity: %s (%d)!"%(city.getName(),newOwner.getCivilizationDescription(0),newOwnerCiv))
                newCulVal = int( revCultureModifier*max([city.getCulture(oldOwnerID)/2,city.countTotalCultureTimes100()/400]) )
                newPlotVal = int( revCultureModifier*max([city.plot().getCulture(oldOwnerID)/2,city.plot().countTotalCulture()/4]) )
                giveCityCulture( city, newOwnerID, newCulVal, newPlotVal, overwriteHigher = False )
        
            # Init city script data (unit spawn counter, rebel player)
            initCity(city)

        iTurns = city.getOccupationTimer()
        #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Occupations timer is currently %d"%(iTurns))
        updateCityVal( city, 'RevolutionCounter', max([int(1.5*iTurns),3]) )
        updateCityVal( city, 'WarningCounter', 0 )

def onEndGameTurn( argsList ) :

    if( game.getGameTurn()%10 == 0 ) :
        updateAttitudeExtras()

    removeFloatingRebellions()

def onSetPlayerAlive( argsList ) :

    iPlayerID = argsList[0]
    bNewValue = argsList[1]
    if( bNewValue == False ) :
        pPlayer = gc.getPlayer( iPlayerID )
        print 'Rev - %s are dead, %d cities lost, %d founded a city'%(pPlayer.getCivilizationDescription(0),pPlayer.getCitiesLost(),pPlayer.isFoundedFirstCity())
        if( not pPlayer.isFoundedFirstCity() ) :
            # Add +1 for this turn?
            for turnID in range(0,game.getGameTurn()) :
                if( pPlayer.getAgricultureHistory(turnID) > 0 ) :
                    print 'Rev - Setting founded city to true for failed reincarnation of rebel player %d'%(iPlayerID)
                    pPlayer.setFoundedFirstCity( True )
                    break

        if( endWarsOnDeath ) :
            pTeam = gc.getTeam(pPlayer.getTeam())
            if( pTeam.getNumMembers() == 1 or not pTeam.isAlive() ) :
                for idx in range(0,gc.getMAX_CIV_TEAMS()) :
                    if( not idx == pTeam.getID() ) :
                        if( pTeam.isAtWar(idx) ) :
                            pTeam.makePeace(idx)

        if( iPlayerID == game.getActivePlayer() and game.getAIAutoPlay() == 0 ) :
            try :
                ChangePlayer.changeHumanPopup( True )
            except :
                pass

def updateAttitudeExtras( ) :

    for i in range(0,gc.getMAX_CIV_PLAYERS()) :
        playerI = gc.getPlayer( i )
        
        for j in range(0,gc.getMAX_CIV_PLAYERS()) :
            playerJ = gc.getPlayer( j )
            attEx = playerI.AI_getAttitudeExtra(j)
            # Odds and rate partially determined by current attitude value
            if( attEx > 0 and game.getSorenRandNum(100,'Rev: Attitude') < attEx*15 ) :
                playerI.AI_changeAttitudeExtra(j, -(1+attEx/10))
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Extra Attitude for %s of %s now %d"%(playerI.getCivilizationDescription(0),playerJ.getCivilizationDescription(0),playerI.AI_getAttitudeExtra(j)))
            elif( attEx < 0 and game.getSorenRandNum(100,'Rev: Attitude') < -attEx*20 ) :
                teamI = gc.getTeam( playerI.getTeam() )
                if( not teamI.isAtWar( playerJ.getTeam() ) ) :
                    playerI.AI_changeAttitudeExtra(j, -(attEx/10))
                    if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Extra Attitude for %s of %s now %d"%(playerI.getCivilizationDescription(0),playerJ.getCivilizationDescription(0),playerI.AI_getAttitudeExtra(j)))

def removeFloatingRebellions( ) :
    # Destroy all units for a rebel who is at peace, yet has no cities

    for i in range(0,gc.getMAX_CIV_PLAYERS()) :
        player = gc.getPlayer(i)
        if( not player.isFoundedFirstCity() and player.getNumUnits() > 0 ) :
            if( player.getNumUnits() == player.getNumMilitaryUnits() ) :
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Player %d (%s) is a homeless rebel"%(player.getID(),player.getCivilizationDescription(0)))
                if( gc.getTeam( player.getTeam() ).getAtWarCount( True ) == 0 ) :
                    if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Player %d has lost their cause, terminating rebel"%(player.getID()))
                    player.killUnits()

########################## Revolution helper functions ###############################


def plotGenerator( startPlot, maxRadius ) :
    # To be used as: for [radius,plot] in RevUtils.plotGenerator(plot,5) :
    # Returns plots starting at radius 1 and up to max Radius

    # Start with center plot
    yield [0,startPlot]

    radius = 1
    gameMap = gc.getMap()
    # Expand radius slowly, searching concentric squares
    while( radius <= maxRadius ) :
        # Top and bottom rows
        for ix in range(startPlot.getX()-radius,startPlot.getX()+radius+1) :
            for iy in [startPlot.getY() - radius, startPlot.getY() + radius] :

                if( ix < 0 ) :
                    if( gameMap.isWrapX() ) :
                        ix = CyMap().getGridWidth() + ix
                    else :
                        continue
                elif( ix >= CyMap().getGridWidth() ) :
                    if( gameMap.isWrapX() ) :
                        ix = ix - CyMap().getGridWidth()
                    else :
                        continue

                if( iy < 0 ) :
                    if( gameMap.isWrapY() ) :
                        iy = CyMap().getGridHeight() + iy
                    else :
                        continue
                elif( iy >= CyMap().getGridHeight() ) :
                    if( gameMap.isWrapY() ) :
                        iy = iy - CyMap().getGridHeight()
                    else :
                        continue

                yield [radius,gameMap.plot(ix,iy)]

        # Left and right columns (leave out corners)
        for ix in [startPlot.getX()-radius,startPlot.getX()+radius] :
            for iy in range(startPlot.getY() - radius + 1, startPlot.getY() + radius) :

                if( ix < 0 ) :
                    if( gameMap.isWrapX() ) :
                        ix = CyMap().getGridWidth() + ix
                    else :
                        continue
                elif( ix >= CyMap().getGridWidth() ) :
                    if( gameMap.isWrapX() ) :
                        ix = ix - CyMap().getGridWidth()
                    else :
                        continue

                if( iy < 0 ) :
                    if( gameMap.isWrapY() ) :
                        iy = CyMap().getGridHeight() + iy
                    else :
                        continue
                elif( iy >= CyMap().getGridHeight() ) :
                    if( gameMap.isWrapY() ) :
                        iy = iy - CyMap().getGridHeight()
                    else :
                        continue

                yield [radius,gameMap.plot(ix,iy)]

        radius += 1

def computeWarOdds( attacker, victim, area, allowAttackerVassal = True, allowVictimVassal = True, allowBreakVassal = True ) :

        if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Calculating war odds")

        attackerTeam = gc.getTeam( attacker.getTeam() )
        victimTeam = gc.getTeam( victim.getTeam() )

        if( attackerTeam.getID() == victimTeam.getID() ) :
            return [-50,attackerTeam,victimTeam]

        warOdds = 0

        if( attackerTeam.isAVassal() ) :
            if( not allowAttackerVassal ) :
                return [-50,attackerTeam,victimTeam]
            if( attackerTeam.isVassal(victimTeam.getID()) ) :
                if( not allowBreakVassal ) :
                    return [-50,attackerTeam,victimTeam]
                else :
                    # Allow vassal to rebel!!!
                    warOdds -= 25
                
            else :
                warOdds -= 10
                for teamID in range(0,gc.getMAX_CIV_TEAMS()) :
                    if( attackerTeam.isVassal(teamID) ) :
                        attackerTeam = gc.getTeam( teamID )
                        attacker = gc.getPlayer( attackerTeam.getLeaderID() )
                        break

        if( victimTeam.isAVassal() ) :
            if( not allowVictimVassal ) :
                return [-50,attackerTeam,victimTeam]
            if( victimTeam.isVassal(attackerTeam.getID()) ) :
                if( not allowBreakVassal ) :
                    return [-50,attackerTeam,victimTeam]
                else :
                    # Allow master to attack
                    warOdds -= 10
            else :
                for teamID in range(0,gc.getMAX_CIV_TEAMS()) :
                    if( victimTeam.isVassal(teamID) ) :
                        victimTeam = gc.getTeam( teamID )
                        victim = gc.getPlayer( victimTeam.getLeaderID() )
                        break

        if( attacker.AI_getAttitude(victim.getID()) == AttitudeTypes.ATTITUDE_FURIOUS ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Attacker furious with potential victim")
            warOdds += 50
        elif( attacker.AI_getAttitude(victim.getID()) == AttitudeTypes.ATTITUDE_ANNOYED ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Attacker annoyed with potential victim")
            warOdds += 25
        elif( attacker.AI_getAttitude(victim.getID()) == AttitudeTypes.ATTITUDE_PLEASED ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Attacker pleased with potential victim")
            warOdds -= 25
        elif ( attacker.AI_getAttitude(victim.getID()) == AttitudeTypes.ATTITUDE_FRIENDLY ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Attacker friendly with potential victim")
            warOdds -= 50
        else :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Attacker cautious with potential victim")
            
        if( area.getPower(victim.getID()) == 0 ) :
            if( victim.getPower() == 0 ) :
                powerFrac = 2
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Victim is powerless, %f for attacker"%(powerFrac))
            else :
                powerFrac = attacker.getPower()/(1.0*victim.getPower()) + .2
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Victim has no power presence in area, %f for attacker"%(powerFrac))
        else :
            powerFrac = area.getPower(attacker.getID())/(1.0*area.getPower(victim.getID()))
        if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Power fraction is %f for attacker"%(powerFrac))
        if( area.getCitiesPerPlayer(attacker.getID()) > 1 and powerFrac > 1.5 ) :
            warOdds += 50
        elif( powerFrac > 1.2 ) :
            warOdds += 35
        elif( powerFrac > 1.0 ) :
            warOdds += 10
        elif( powerFrac < .9 ) :
            warOdds -= 25
        elif( area.getCitiesPerPlayer(attacker.getID()) < 1 or powerFrac < .7 ) :
            warOdds -= 40

        if( area.getCitiesPerPlayer(attacker.getID()) > area.getCitiesPerPlayer(victim.getID()) ) :
            warOdds += 10

        iAgg = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGGRESSIVE')
        if( attacker.hasTrait(iAgg) ) :
            warOdds += 10

        return [warOdds,attackerTeam,victimTeam]

def giveTechs( toPlayer, fromPlayer, expensiveVars = [1,3], doTakeAway = True ) :

        # Give all techs known by fromPlayer, except a few of the most expensive
        knownTechs = list()
        mostExpensive = list()
        minMostExpensive = 0
        numMostExpensive = expensiveVars[0] + game.getSorenRandNum(expensiveVars[1],'Rev: Pick num techs')

        fromPlayerTeam = gc.getTeam( fromPlayer.getID() )
        toPlayerTeam = gc.getTeam( toPlayer.getID() )
        
        for techID in range(0,gc.getNumTechInfos()) :
            if( fromPlayerTeam.isHasTech( techID ) ) :
                knownTechs.append( techID )
                if( gc.getTechInfo( techID ).getResearchCost() > minMostExpensive ) :
                    if( len(mostExpensive) < numMostExpensive ) :
                        mostExpensive.append( techID )
                    else :
                        for j in range(0,len(mostExpensive)) :
                            if( gc.getTechInfo( mostExpensive[j] ).getResearchCost() == minMostExpensive ) :
                                mostExpensive.remove( mostExpensive[j] )
                                break
                        mostExpensive.append( techID )
                        minMostExpensive = gc.getTechInfo( mostExpensive[0] ).getResearchCost()
                        for j in range(0,len(mostExpensive)) :
                            if( gc.getTechInfo( mostExpensive[j] ).getResearchCost() < minMostExpensive ) :
                                minMostExpensive = gc.getTechInfo( mostExpensive[j] ).getResearchCost()

            if( doTakeAway and toPlayerTeam.isHasTech(techID) ) :
                # take away all techs
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Taking away %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
                toPlayerTeam.setHasTech(techID,False,toPlayer.getID(),False,False)

        if( doTakeAway ) :
            if( not toPlayer.getTechScore() == 0 ) :
                # Shouldn't have to do this, but tech scores never came out to zero for reincarnated civs ...
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Resetting tech score ...")
                toPlayer.changeTechScore( -toPlayer.getTechScore() )

        for techID in knownTechs :
            if( not techID in mostExpensive ) :
                toPlayerTeam.setHasTech(techID,True,toPlayer.getID(),False,False)
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Giving rev %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
            else :
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - NOT giving rev %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
                

def giveCityCulture( pCity, iPlayer, newCityVal, newPlotVal, overwriteHigher = False ) :
        # Places this culture value in city and city plot
        # Places half this value in neighboring plots

        if( iPlayer < 0 or iPlayer >= gc.getBARBARIAN_PLAYER() ) :
            return

        if( overwriteHigher or newCityVal > pCity.getCulture(iPlayer) ) :
            pCity.setCulture( iPlayer, newCityVal, True )
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Culture set to %d"%(pCity.getCulture(iPlayer)))
            
            if( overwriteHigher or newPlotVal > pCity.plot().getCulture(iPlayer) ) :
                pCity.plot().setCulture( iPlayer, newPlotVal, True )

            gameMap = gc.getMap()

            for [radius,pPlot] in plotGenerator( pCity.plot(), 1 ) :
                if( pPlot.countTotalCulture() > 0 ) :
                    if( overwriteHigher or newPlotVal/2 > pPlot.getCulture(iPlayer) ) :
                        pPlot.setCulture( iPlayer, newPlotVal/2, True )
                    
        else :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %s already has a lot of culture here"%(gc.getPlayer(iPlayer).getCivilizationDescription(0)))

                                             
def getHandoverUnitTypes( city, pPlayer, compPlayer = None ) :

        warriorClass = CvUtil.findInfoTypeNum(gc.getUnitClassInfo,gc.getNumUnitClassInfos(),'UNITCLASS_WARRIOR')
        iWarrior = gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(warriorClass)
        workerClass = CvUtil.findInfoTypeNum(gc.getUnitClassInfo,gc.getNumUnitClassInfos(),'UNITCLASS_WORKER')
        iWorker = gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(workerClass)
        iBestDefender = UnitTypes.NO_UNIT
        iCounter = UnitTypes.NO_UNIT
        iAttack = UnitTypes.NO_UNIT
        if( not compPlayer == None ) :
            compPy = PyPlayer( compPlayer.getID() )

        for unitID in range(0,gc.getNumUnitInfos()) :
            if( city.canTrain(unitID,False,False) and gc.getUnitInfo(unitID).getDomainType() == DomainTypes.DOMAIN_LAND ): # canTrain only does technology check
                unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                unitInfo = gc.getUnitInfo(unitID)
                if( not unitInfo.getPrereqAndTech() == TechTypes.NO_TECH ) :
                    unitTechInfo = gc.getTechInfo( unitInfo.getPrereqAndTech() )

                    # Defender (Archer,Longbow)
                    if( gc.getUnitInfo(unitID).getDefaultUnitAIType() == UnitAITypes.UNITAI_CITY_DEFENSE ):
                        if( (iBestDefender == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() >= gc.getUnitInfo(iBestDefender).getCombat() ) :
                            #if( LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ def units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                            if( unitID == gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                if( compPlayer == None ) :
                                    iBestDefender = unitID
                                else :
                                    compUnitID = gc.getCivilizationInfo( compPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                    if( len(compPy.getUnitsOfType(unitID)) > 0 ) :
                                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rev -  Comp has %d %s (def)"%(len(compPy.getUnitsOfType(compUnitID)),PyInfo.UnitInfo(compUnitID).getDescription()))
                                        iBestDefender = unitID
                                    elif( unitTechInfo.getEra() < compPlayer.getCurrentEra() ) :
                                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rev -  Outdated unit %s (def)"%(PyInfo.UnitInfo(unitID).getDescription()))
                                        iBestDefender = unitID
                                    else :
                                        pass
                                    
                                #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Best defender set to %s"%(PyInfo.UnitInfo(iBestDefender).getDescription()))
                    # Counter (Axemen,Phalanx)
                    if( gc.getUnitInfo(unitID).getUnitAIType(UnitAITypes.UNITAI_COUNTER) ):
                        if( (iCounter == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() >= gc.getUnitInfo(iCounter).getCombat() ) :
                            #if( LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ counter units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                            if( unitID == gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                if( compPlayer == None ) :
                                    iCounter = unitID
                                else :
                                    compUnitID = gc.getCivilizationInfo( compPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                    if( len(compPy.getUnitsOfType(unitID)) > 1 ) :
                                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rev -  Comp has %d %s (count)"%(len(compPy.getUnitsOfType(compUnitID)),PyInfo.UnitInfo(compUnitID).getDescription()))
                                        iCounter = unitID
                                    else :
                                        pass
                                #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev  - Best counter unit set to %s"%(PyInfo.UnitInfo(iCounter).getDescription()))
                    # Assault units
                    if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ATTACK ) ):
                        if( (iAttack == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() > gc.getUnitInfo(iAttack).getCombat() ) :
                            #if( LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ attack units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                            if( unitID == gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                if( compPlayer == None ) :
                                    iAttack = unitID
                                else :
                                    compUnitID = gc.getCivilizationInfo( compPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                    if( len(compPy.getUnitsOfType(unitID)) > 1 ) :
                                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rev -  Comp has %d %s (att)"%(len(compPy.getUnitsOfType(compUnitID)),PyInfo.UnitInfo(compUnitID).getDescription()))
                                        iAttack = unitID
                                    else :
                                        pass
                                #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev  - Best attack set to %s"%(PyInfo.UnitInfo(iAttack).getDescription()))

        if( iBestDefender == UnitTypes.NO_UNIT ) :
            if( not iCounter == UnitTypes.NO_UNIT ) :
                iBestDefender = iCounter
            else :
                iBestDefender = iWarrior
        if( iCounter == UnitTypes.NO_UNIT ) : iCounter = iBestDefender
        if( iAttack == UnitTypes.NO_UNIT ) : iAttack = iCounter
        
        if( LOG_DEBUG ) :
                CvUtil.pyPrint("Rev - Best defender set to %s"%(PyInfo.UnitInfo(iBestDefender).getDescription()))
                CvUtil.pyPrint("Rev - Best counter unit set to %s"%(PyInfo.UnitInfo(iCounter).getDescription()))
                CvUtil.pyPrint("Rev - Best attack set to %s"%(PyInfo.UnitInfo(iAttack).getDescription()))

        return [iWorker,iBestDefender,iCounter,iAttack]

def getUprisingUnitTypes( pCity, pRevPlayer, isJoinWar ) :

        spawnableUnits = list()
        trainableUnits = list()

        owner = gc.getPlayer( pCity.getOwner() )
        ownerPy = PyPlayer( pCity.getOwner() )
        enemyPy = None
        if( isJoinWar ) :
            enemyPy = PyPlayer( pRevPlayer.getID() )

        bIsBarb = (pRevPlayer.getID() == gc.getBARBARIAN_PLAYER())

        for unitID in range(0,gc.getNumUnitInfos()) :
            ownerUnits = ownerPy.getUnitsOfType( unitID )
            iOwnerEra = owner.getCurrentEra()
            unitInfo = gc.getUnitInfo(unitID)
            if( not unitInfo.getPrereqAndTech() == TechTypes.NO_TECH ) :
                unitTechInfo = gc.getTechInfo( unitInfo.getPrereqAndTech() )

                if( unitTechInfo.getEra() > iOwnerEra - 3 ) :
                    #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel: %s requires knowledge of %s"%(unitInfo.getDescription(),unitTechInfo.getDescription()))
                    if( len( ownerUnits ) > 0 ) :
                        if( ownerUnits[0].canAttack() ) :
                            #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Owner has %d %s"%(len(ownerUnits),PyInfo.UnitInfo(unitID).getDescription()))
                            if( unitInfo.getDomainType() == DomainTypes.DOMAIN_LAND and ( unitInfo.getUnitAIType(UnitAITypes.UNITAI_ATTACK) or unitInfo.getUnitAIType(UnitAITypes.UNITAI_COUNTER) )):
                                unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                                for unit in ownerUnits :
                                    if( plotDistance( unit.getX(), unit.getY(), pCity.getX(), pCity.getY() ) < 7 ) :
                                        if( bIsBarb ) :
                                            spawnUnitID = gc.getCivilizationInfo( owner.getCivilizationType() ).getCivilizationUnits(unitClass)
                                        else :
                                            spawnUnitID = gc.getCivilizationInfo( pRevPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                        spawnableUnits.append( spawnUnitID )
                                        #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can spawn from owner %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))
                                        if( gc.getUnitInfo(unitID).getDefaultUnitAIType() == UnitAITypes.UNITAI_CITY_DEFENSE ) :
                                            if( unitTechInfo.getEra() == iOwnerEra ) :
                                                if( spawnableUnits.count( spawnUnitID ) > 3 ) :
                                                    break
                                            else :
                                                if( spawnableUnits.count( spawnUnitID ) > 8 ) :
                                                    break
                                        else :
                                            if( unitTechInfo.getEra() == iOwnerEra ) :
                                                if( spawnableUnits.count( spawnUnitID ) > 1 ) :
                                                    break
                                            else :
                                                if( spawnableUnits.count( spawnUnitID ) > 3 ) :
                                                    break

                                if( not unitTechInfo.getEra() == iOwnerEra ) :
                                    # Can spawn old units from further away
                                    for unit in ownerUnits :
                                        if( unit.area().getID() == pCity.area().getID() ):
                                            if( bIsBarb ) :
                                                spawnUnitID = gc.getCivilizationInfo( owner.getCivilizationType() ).getCivilizationUnits(unitClass)
                                            else :
                                                spawnUnitID = gc.getCivilizationInfo( pRevPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                            spawnableUnits.append( spawnUnitID )
                                            #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can spawn from Area %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))
                                            if( unitTechInfo.getEra() < iOwnerEra ) :
                                                spawnableUnits.append( spawnUnitID )
                                                #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can spawn from Area %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))
                                            break

                    if( not enemyPy == None ) :
                        enemyUnits = enemyPy.getUnitsOfType( unitID )
                        if( len( enemyUnits ) > 0 ) :
                            if( enemyUnits[0].canAttack() ) :
                                if( unitInfo.getDomainType() == DomainTypes.DOMAIN_LAND and unitInfo.getUnitAIType( UnitAITypes.UNITAI_ATTACK )  ):
                                    unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                                    for unit in enemyUnits :
                                        if( plotDistance( unit.getX(), unit.getY(), pCity.getX(), pCity.getY() ) < 7 ) :
                                            if( bIsBarb ) :
                                                # Should never get here
                                                spawnUnitID = unitID
                                            else :
                                                spawnUnitID = gc.getCivilizationInfo( pRevPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                            spawnableUnits.append( spawnUnitID )
                                            #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can spawn from enemy %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))

            
            if( pCity.canTrain(unitID,False,False) and gc.getUnitInfo(unitID).getDomainType() == DomainTypes.DOMAIN_LAND ):
                if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ATTACK ) ):
                    unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                    if( bIsBarb ) :
                        spawnUnitID = gc.getCivilizationInfo( owner.getCivilizationType() ).getCivilizationUnits(unitClass)
                    else :
                        spawnUnitID = gc.getCivilizationInfo( pRevPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                        
                    if( not spawnUnitID in trainableUnits ) :
                        trainableUnits.append( spawnUnitID )
                        if( gc.getUnitInfo(unitID).getCombat() > 4 ) :
                            trainableUnits.append( spawnUnitID )
                            if( gc.getUnitInfo(unitID).getCombat() > 15 ) :
                                trainableUnits.append( spawnUnitID )
                        #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can build %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))

        if( len(spawnableUnits) < 1 ) :
            spawnableUnits = trainableUnits

        return spawnableUnits

def getNumDefendersNearPlot( iPlotX, iPlotY, iPlayer, iRange = 2, bIncludePlot = True, bIncludeCities = False ) :
    # bIncludePlot takes precedence over bIncludeCities
    iNumUnits = 0

    gameMap = gc.getMap()
    basePlot = gameMap.plot(iPlotX,iPlotY)

    for [radius,pPlot] in plotGenerator( basePlot, iRange ) :

        if( pPlot.getX() == iPlotX and pPlot.getY() == iPlotY ) :
            if( not bIncludePlot ) :
                continue
        elif( pPlot.isCity() and not bIncludeCities ) :
            continue

        iNumUnits += pPlot.getNumDefenders( iPlayer )

    return iNumUnits            

def getClosestCityXY( iPlotX, iPlotY, iPlayer, maxRange = 10, bIncludeBase = True ) :

    gameMap = gc.getMap()
    basePlot = gameMap.plot(iPlotX,iPlotY)

    for [radius,pPlot] in plotGenerator( basePlot, maxRange ) :
        if( radius == 0 and not bIncludeBase ) :
            continue
        if( pPlot.isCity() ) :
            if( pPlot.getOwner() == iPlayer ) :
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev -  Found city %s (owner %d) at radius %d"%(pPlot.getPlotCity().getName(),pPlot.getPlotCity().getOwner(),radius))
                return [pPlot.getX(),pPlot.getY()]

    return None

def getSpawnablePlots( iPlotX, iPlotY, pSpawnPlayer, bLand = True, bIncludePlot = True, bIncludeCities = False, bSameArea = True, iRange = 2, iSpawnPlotOwner = -1, bCheckForEnemy = True ) :

        spawnablePlots = list()

        gameMap = gc.getMap()
        basePlot = gameMap.plot(iPlotX,iPlotY)

        try :
            iBaseArea = basePlot.area().getID()
        except AttributeError :
            if( bSameArea ) : print "WARNING: Passed an arealess plot!"
            iBaseArea = -1
            bSameArea = False
        iBasePlotOwner = basePlot.getOwner()
        iNumPlotsChecked = 0

        for [radius,pPlot] in plotGenerator( basePlot, iRange ) :

                if( not bIncludePlot and pPlot.getX() == iPlotX and pPlot.getY() == iPlotY ) :
                    continue

                if( pPlot.isImpassable() ):
                    continue

                iNumPlotsChecked += 1

                if( bLand and pPlot.isWater() ) :
                    continue

                if( not bLand and not pPlot.isWater() ) :
                    continue

                if( not bIncludeCities and pPlot.isCity() ) :
                    continue

                if( bSameArea and not iBaseArea == pPlot.area().getID() ) :
                    continue

                if( bCheckForEnemy ) :
                    if( len( getEnemyUnits(pPlot.getX(),pPlot.getY(),pSpawnPlayer.getID()) ) > 0 ) :
                        continue

                # When iSpawnPlotOwner >= 0, plot owner must be either iSpawnPlotOwner, iBasePlotOwner, or no one
                if( iSpawnPlotOwner < 0 or pPlot.getOwner() == iSpawnPlotOwner or pPlot.getOwner() == iBasePlotOwner or pPlot.getOwner() == PlayerTypes.NO_PLAYER ) :
                    spawnablePlots.append( [pPlot.getX(),pPlot.getY()] )

        if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Found %d plots out of %d checked"%(len(spawnablePlots),iNumPlotsChecked))

##        for plot in spawnablePlots :
##            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - SP: %d, %d"%(plot[0],plot[1]))

        return spawnablePlots

def getEnemyUnits( iPlotX, iPlotY, iEnemyOfPlayer, domain = -1 ) :

        pEnemyOfTeam = gc.getTeam( gc.getPlayer(iEnemyOfPlayer).getTeam() )
        gameMap = gc.getMap()
        pPlot = gameMap.plot(iPlotX,iPlotY)

        enemyUnits = list()

        for i in range(0,pPlot.getNumUnits()) :
            pUnit = pPlot.getUnit(i)
            pUnitTeam = gc.getTeam( pUnit.getTeam() )
            if( pEnemyOfTeam.isAtWar(pUnit.getTeam()) ) :
                if( domain < 0 or pUnit.getDomainType() == domain ) :
                    enemyUnits.append( pUnit )
                    #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Unit %s, id %d, player %d, at %d,%d"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner(),pUnit.plot().getX(),pUnit.plot().getY()))

        return enemyUnits

def moveEnemyUnits( iPlotX, iPlotY, iEnemyOfPlayer, iMoveToX, iMoveToY, iInjureMax = 0, bDestroyNonLand = True, bLeaveSiege = False ) :

        unitList = getEnemyUnits( iPlotX, iPlotY, iEnemyOfPlayer )

##        for pUnit in unitList :               
##                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Moving %s (id: %d, player: %d)"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner()))
##                unitPlot = pUnit.plot()
##                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - from %d, %d"%(unitPlot.getX(),unitPlot.getY()))

        if( iInjureMax > 0 ) :
            for pUnit in unitList :
                iInjure = iInjureMax/2 + game.getSorenRandNum(iInjureMax/2,'Rev: Wound retreating units')
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %d starts at %d"%(pUnit.getID(),pUnit.getDamage()))
                if( pUnit.canFight() and pUnit.getDamage() < iInjure ) :
                    pUnit.setDamage( iInjure, iEnemyOfPlayer )
                    #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %d injured to %d"%(pUnit.getID(),pUnit.getDamage()))

        pPlot = gc.getMap().plot(iMoveToX,iMoveToY)

        toKillList = list()
        for pUnit in unitList :
            if( not pUnit.getDomainType() == DomainTypes.DOMAIN_LAND or not pUnit.canMoveInto(pPlot,False,False,True) ) :
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Pulling out %s for subsequent kill"%(pUnit.getName()))
                if( bDestroyNonLand ) :
                    toKillList.append(pUnit)
                else :
                    if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Leaving %s (%s)"%(pUnit.getName(),gc.getPlayer(pUnit.getOwner()).getCivilizationDescription(0)))
            elif( bLeaveSiege and pUnit.bombardRate() > 0 ) :
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Leaving siege %s (%s)"%(pUnit.getName(),gc.getPlayer(pUnit.getOwner()).getCivilizationDescription(0))) 
                pass
            else :
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Moving %s (id: %d, player: %d)"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner()))
                #unitPlot = pUnit.plot()
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - from %d, %d"%(unitPlot.getX(),unitPlot.getY()))
                pUnit.setXY( iMoveToX, iMoveToY )
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - to %d, %d"%(pUnit.plot().getX(),pUnit.plot().getY()))

        #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Units moved")

        for pUnit in toKillList :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Killing %s at %d, %d"%(pUnit.getName(),pUnit.getX(),pUnit.getY()))
            if( not pUnit.isNone() and not pUnit.plot().isNone() ) :
                pUnit.kill(False,iEnemyOfPlayer)


def moveEnemyUnits2( iPlotX, iPlotY, iEnemyOfPlayer, iMoveToX, iMoveToY, iInjureMax = 0, bMoveAir = True, bLeaveSiege = False ) :

        unitList = getEnemyUnits( iPlotX, iPlotY, iEnemyOfPlayer )

##        for pUnit in unitList :               
##                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Moving %s (id: %d, player: %d)"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner()))
##                unitPlot = pUnit.plot()
##                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - from %d, %d"%(unitPlot.getX(),unitPlot.getY()))

        if( iInjureMax > 0 ) :
            for pUnit in unitList :
                iInjure = iInjureMax/2 + game.getSorenRandNum(iInjureMax/2,'Rev: Wound retreating units')
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %d starts at %d"%(pUnit.getID(),pUnit.getDamage()))
                if( pUnit.canFight() and pUnit.getDamage() < iInjure ) :
                    pUnit.setDamage( iInjure, iEnemyOfPlayer )
                    #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %d injured to %d"%(pUnit.getID(),pUnit.getDamage()))

        pPlot = gc.getMap().plot(iMoveToX,iMoveToY)

        for pUnit in unitList :
            if( pUnit.getDomainType() == DomainTypes.DOMAIN_LAND or (bMoveAir and pUnit.getDomainType() == DomainTypes.DOMAIN_AIR) ) :
                if( bLeaveSiege and pUnit.getDomainType() == DomainTypes.DOMAIN_LAND and pUnit.bombardRate() > 0 ) :
                    if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Leaving siege %s (%s)"%(pUnit.getName(),gc.getPlayer(pUnit.getOwner()).getCivilizationDescription(0)))
                else :
                    if( pUnit.getDomainType() == DomainTypes.DOMAIN_AIR ) :
                        if( pPlot.isCity() or pUnit.canMoveInto(pPlot,False,False,True) ) :
                            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Moving %s (id: %d, player: %d)"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner()))
                            pUnit.setXY( iMoveToX, iMoveToY )
                        else :
                            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Can't move air unit %s (%s)"%(pUnit.getName(),gc.getPlayer(pUnit.getOwner()).getCivilizationDescription(0)))
                    else :
                        if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Moving %s (id: %d, player: %d)"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner()))
                        pUnit.setXY( iMoveToX, iMoveToY )
            else :
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Leaving %s (%s)"%(pUnit.getName(),gc.getPlayer(pUnit.getOwner()).getCivilizationDescription(0)))


def clearOutCity( pCity, pPlayer, pEnemyPlayer ) :

    ix = pCity.getX()
    iy = pCity.getY()
    
    moveXY = getClosestCityXY( ix, iy, pPlayer.getID(), 25, bIncludeBase = False )
    if( moveXY == None ) :
        if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No nearby cities, just placing not too far away")
        retreatPlots = getSpawnablePlots( ix, iy, pPlayer, bLand = True, bIncludePlot = False, bIncludeCities = True, bSameArea = True, iRange = 3, iSpawnPlotOwner = pPlayer.getID(), bCheckForEnemy = True )
        if( len(retreatPlots) == 0 ) :
            retreatPlots = getSpawnablePlots( ix, iy, pPlayer, bLand = True, bIncludePlot = False, bIncludeCities = True, bSameArea = False, iRange = 5, iSpawnPlotOwner = -1, bCheckForEnemy = True )

        if( len(retreatPlots) > 0 ) :
            moveXY = retreatPlots[game.getSorenRandNum(len(retreatPlots),'Rev')]

    if( not moveXY == None ) :
        moveEnemyUnits2( ix, iy, pEnemyPlayer.getID(), moveXY[0], moveXY[1], bMoveAir = True )

        # Handle water units
        waterUnits = getEnemyUnits( ix, iy, pEnemyPlayer.getID(), domain = DomainTypes.DOMAIN_SEA )

        if( len(waterUnits) > 0 ) :
            retreatPlots = getSpawnablePlots( ix, iy, pPlayer, bLand = False, bIncludePlot = False, bIncludeCities = False, bSameArea = False, iRange = 1, iSpawnPlotOwner = pPlayer.getID(), bCheckForEnemy = True )
            if( len(retreatPlots) == 0 ) :
                retreatPlots = getSpawnablePlots( ix, iy, pPlayer, bLand = False, bIncludePlot = False, bIncludeCities = False, bSameArea = False, iRange = 5, iSpawnPlotOwner = -1, bCheckForEnemy = True )
            if( len(retreatPlots) > 0 ) :
                moveXY = retreatPlots[game.getSorenRandNum(len(retreatPlots),'Rev')]
                for unit in waterUnits :
                    if( unit.canMoveInto(gc.getMap().plot(moveXY[0],moveXY[1]),False,False,True) ) :
                        if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Moving water unit %s (id: %d, player: %d)"%(unit.getName(),unit.getID(),unit.getOwner()))
                        unit.setXY( moveXY[0], moveXY[1] )


def getRevolutionCityLists( pPlayer ) :

    playerPy = PyPlayer( pPlayer.getID() )
    cityList = playerPy.getCityList()

    pTeam = gc.getTeam( pPlayer.getTeam() )

    revReadyCities = list()
    revInstigatorCities = list()
    warnCities = list()
    unhappyCities = list()

    capRevIdx = 0

    for city in cityList :
        pCity = city.GetCy()

        revIdx = getCityRevolutionIndex( pCity )
        prevRevIdx = getCityPrevRevIndex( pCity )
        localRevIdx = getCityLocalRevIndex( pCity )

        numUnhappy = pCity.angryPopulation(0)
        modifier = pCity.getPopulation()*pCity.getWarWearinessPercentAnger()/100
        modNumUnhappy = pCity.angryPopluation( -modifier )

        #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %s has numUnhappy %d, mod %d, modNumUn %d"%(pCity.getName(),numUnhappy,modifier,modNumUnhappy))
        
        #if( modNumUnhappy > 0 ) :
            

def getModNumUnhappy( pCity, wwMod = 2.0, silent = False ) :

    modifier = int( wwMod*pCity.getPopulation()*pCity.getWarWearinessPercentAnger()/1000 )

    modNumUnhappy = pCity.angryPopulation( -modifier )

    if( not silent and LOG_DEBUG and pCity.angryPopulation(0) > 0 and game.getGameTurn()%25 == 0 ) :
        CvUtil.pyPrint("  Rev - %s has numUnhappy %d, mod %d, modNumUn %d"%(pCity.getName(),pCity.angryPopulation(0),modifier,modNumUnhappy))

    if( modNumUnhappy <= 0 ) :
        numHappy = max( [0, pCity.happyLevel() - pCity.unhappyLevel(0)] )
        numHappy = min( [numHappy, pCity.getPopulation()] )
        return -numHappy
    else :
        return modNumUnhappy

########################## Civics effect helper functions #####################

def getCivicsRevIdxLocal( iPlayer ) :

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() ) :
        return [0,"",""]

    if( pPlayer.getNumCities() == 0 ) :
        return [0,"",""]

    if( len(civicsList) < gc.getNumCivicOptionInfos() ) :
        initCivicsList()

    localRevIdx = 0
    goodStrings = ""
    badStrings = ""

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        civicEffect = civicInfo.getRevIdxLocal()

        # Effect doubles for some when a much better alternative exists
        if( civicEffect > 0 and civicInfo.getRevLaborFreedom() < -1 ) :
            for j in civicsList[civicInfo.getCivicOptionType()] :
                if( pPlayer.canDoCivics(j) ) :
                    jInfo = gc.getCivicInfo(j)
                    if( jInfo.getRevLaborFreedom() > 1 ) :
                        civicEffect = 2*civicEffect
                        #CvUtil.pyPrint("  Rev - Effect of %s doubled to %d because can do %s"%(civicInfo.getDescription(),civicEffect,jInfo.getDescription()))
                        break

        if( civicEffect > 0 and civicInfo.getRevDemocracyLevel() < -1 ) :
            for j in civicsList[civicInfo.getCivicOptionType()] :
                if( pPlayer.canDoCivics(j) ) :
                    jInfo = gc.getCivicInfo(j)
                    if( jInfo.getRevDemocracyLevel() > 1 ) :
                        civicEffect = 2*civicEffect
                        #CvUtil.pyPrint("  Rev - Effect of %s doubled to %d because can do %s"%(civicInfo.getDescription(),civicEffect,jInfo.getDescription()))
                        break
        
        if( civicEffect > 0 ) :
            badStrings += civicInfo.getDescription() + ', '
        elif( civicEffect < 0 ) :
            goodStrings += civicInfo.getDescription() + ', '

        #CvUtil.pyPrint("  Rev - %s local effect: %d"%(civicInfo.getDescription(),civicEffect))

        localRevIdx += civicEffect

    #goodStrings.strip(', ')
    #badString.strip(', ')
    return [localRevIdx,goodStrings,badStrings]

def getCivicsRevIdxNational( iPlayer ) :

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() ) :
        return [0,"",""]

    if( pPlayer.getNumCities() == 0 ) :
        return [0,"",""]

    if( len(civicsList) < gc.getNumCivicOptionInfos() ) :
        initCivicsList()

    revIdx = 0
    goodStrings = ""
    badStrings = ""

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        civicEffect = civicInfo.getRevIdxLocal()

        # Effect doubles for some when a much better alternative exists
        if( civicEffect > 0 and civicInfo.getRevLaborFreedom() < -1 ) :
            for j in civicsList[civicInfo.getCivicOptionType()] :
                if( pPlayer.canDoCivics(j) ) :
                    jInfo = gc.getCivicInfo(j)
                    if( jInfo.getRevLaborFreedom() > 1 ) :
                        civicEffect = 2*civicEffect
                        #CvUtil.pyPrint("  Rev - Effect of %d doubled to %d because can do %s"%(civicInfo.getDescription(),civicEffect,jInfo.getDescription()))
                        break

        if( civicEffect > 0 and civicInfo.getRevDemocracyLevel() < -1 ) :
            for j in civicsList[civicInfo.getCivicOptionType()] :
                if( pPlayer.canDoCivics(j) ) :
                    jInfo = gc.getCivicInfo(j)
                    if( jInfo.getRevDemocracyLevel() > 1 ) :
                        civicEffect = 2*civicEffect
                        #CvUtil.pyPrint("  Rev - Effect of %d doubled to %d because can do %s"%(civicInfo.getDescription(),civicEffect,jInfo.getDescription()))
                        break
        
        if( civicEffect > 0 ) :
            badStrings += civicInfo.getDescription() + ', '
        elif( civicEffect < 0 ) :
            goodStrings += civicInfo.getDescription() + ', '

        #CvUtil.pyPrint("  Rev - %s local effect: %d"%(civicInfo.getDescription(),civicEffect))

        revIdx += civicEffect

    #goodStrings.strip(', ')
    #badString.strip(', ')
    return [revIdx,goodStrings,badStrings]

def getCivicsHolyCityEffects( iPlayer ) :

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() ) :
        return [0,0]

    if( pPlayer.getNumCities() == 0 ) :
        return [0,0]

    goodEffect = 0
    badEffect = 0

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        goodEffect += civicInfo.getRevIdxHolyCityGood()
        badEffect += civicInfo.getRevIdxHolyCityBad()

    return [goodEffect,badEffect]

def getCivicsReligionMods( iPlayer ) :
    
    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() ) :
        return [0,0]

    if( pPlayer.getNumCities() == 0 ) :
        return [0,0]

    goodMod = 0
    badMod = 0

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        goodMod += civicInfo.getRevIdxGoodReligionMod()
        badMod += civicInfo.getRevIdxBadReligionMod()

    return [goodMod,badMod]

def getCivicsDistanceMod( iPlayer ) :
    
    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() ) :
        return 0

    if( pPlayer.getNumCities() == 0 ) :
        return 0

    distMod = 0

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        distMod += civicInfo.getRevIdxDistanceMod()

    return distMod

def getCivicsNationalityMod( iPlayer ) :
    
    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() ) :
        return 0

    if( pPlayer.getNumCities() == 0 ) :
        return 0

    natMod = 0

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        natMod += civicInfo.getRevIdxNationalityMod()

    return natMod

def getCivicsViolentRevMod( iPlayer ) :
    
    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() ) :
        return 0

    if( pPlayer.getNumCities() == 0 ) :
        return 0

    vioMod = 0

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        vioMod += civicInfo.getRevIdxNationalityMod()

    return vioMod

def canDoCommunism( iPlayer ) :
    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() ) :
        return [False,None]

    for i in range(0,gc.getNumCivicInfos()) :
        civicInfo = gc.getCivicInfo(i)
        if( civicInfo.IsCommunism() and pPlayer.canDoCivics(i) ) :
            if( not i == pPlayer.getCivics(civicInfo.getCivicOptionType()) ) :
                return [True,i]

    return [False,None]

def canDoFreeSpeech( iPlayer ) :
    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() ) :
        return [False,None]

    for i in range(0,gc.getNumCivicInfos()) :
        civicInfo = gc.getCivicInfo(i)
        if( civicInfo.IsFreeSpeech() and pPlayer.canDoCivics(i) ) :
            if( not i == pPlayer.getCivics(civicInfo.getCivicOptionType()) ) :
                return [True,i]

    return [False,None]

def canDoElection( iPlayer ) :
    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() ) :
        return False

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        if( civicInfo.IsCanDoElection() ) :
            return True

    return False

def getReligiousFreedom( iPlayer ) :
    # Returns [freedom level, option type]

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() ) :
        return [0,None]

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        if( not civicInfo.getRevReligiousFreedom() == 0 ) :
            return [civicInfo.getRevReligiousFreedom(),i]

    return [0,None]


def getBestReligiousFreedom( iPlayer, relOptionType ) :
    # Returns [best level, civic type]

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() or relOptionType == None ) :
        return [0,None]

    bestFreedom = -11
    bestCivic = None

    for i in civicsList[relOptionType] :
        civicInfo = gc.getCivicInfo(i)
        civicFreedom = civicInfo.getRevReligiousFreedom()
        if( pPlayer.canDoCivics(i) and not civicFreedom == 0 ) :
            if( civicInfo.getRevReligiousFreedom() > bestFreedom ) :
                bestFreedom = civicFreedom
                bestCivic = i

    return [bestFreedom, bestCivic]

def getDemocracyLevel( iPlayer ) :
    # Returns [level, option type]

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() ) :
        return [0,None]

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        if( not civicInfo.getRevDemocracyLevel() == 0 ) :
            return [civicInfo.getRevDemocracyLevel(),i]

    return [0,None]


def getBestDemocracyLevel( iPlayer, optionType ) :
    # Returns [best level, civic type]

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() or optionType == None ) :
        return [0,None]

    bestLevel = -11
    bestCivic = None

    for i in civicsList[optionType] :
        civicInfo = gc.getCivicInfo(i)
        civicLevel = civicInfo.getRevDemocracyLevel()
        if( pPlayer.canDoCivics(i) and not civicLevel == 0 ) :
            if( civicLevel > bestLevel ) :
                bestLevel = civicLevel
                bestCivic = i

    return [bestLevel, bestCivic]

def getLaborFreedom( iPlayer ) :
    # Returns [level, option type]

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() ) :
        return [0,None]

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        if( not civicInfo.getRevLaborFreedom() == 0 ) :
            return [civicInfo.getRevLaborFreedom(),i]

    return [0,None]


def getBestLaborFreedom( iPlayer, optionType ) :
    # Returns [best level, civic type]

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() or optionType == None ) :
        return [0,None]

    bestLevel = -11
    bestCivic = None

    for i in civicsList[optionType] :
        civicInfo = gc.getCivicInfo(i)
        civicLevel = civicInfo.getRevLaborFreedom()
        if( pPlayer.canDoCivics(i) and not civicLevel == 0 ) :
            if( civicLevel > bestLevel ) :
                bestLevel = civicLevel
                bestCivic = i

    return [bestLevel, bestCivic]

def getEnvironmentalProtection( iPlayer ) :
    # Returns [level, option type]

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() ) :
        return [0,None]

    for i in range(0,gc.getNumCivicOptionInfos()) :
        iCivic = pPlayer.getCivics(i)
        civicInfo = gc.getCivicInfo(iCivic)
        if( not civicInfo.getRevEnvironmentalProtection() == 0 ) :
            return [civicInfo.getRevEnvironmentalProtection(),i]

    return [0,None]


def getBestEnvironmentalProtection( iPlayer, optionType ) :
    # Returns [best level, civic type]

    pPlayer = gc.getPlayer(iPlayer)

    if( pPlayer.isNone() or not pPlayer.isAlive() or optionType == None ) :
        return [0,None]

    bestLevel = -11
    bestCivic = None

    for i in civicsList[optionType] :
        civicInfo = gc.getCivicInfo(i)
        civicLevel = civicInfo.getRevEnvironmentalProtection()
        if( pPlayer.canDoCivics(i) and not civicLevel == 0 ) :
            if( civicLevel > bestLevel ) :
                bestLevel = civicLevel
                bestCivic = i

    return [bestLevel, bestCivic]

    
########################## Interface with SdToolkit ###########################

#   Loads previously initialized data for Revolution mod from the central reservoir. If no data is found, init it.
def revLoad( object ):
	cyTable = SdToolKitCustom.sdLoad( object )

	if( 'Revolution' in cyTable ) :
            return cyTable['Revolution']
        else :
            return {}

#----------------- OBJECT FUNCTIONS -------------------#

#   revObjectInit ( object, Template_dictionary )
#   Initializes objects revolution member to the template dictionary.
def revObjectInit (object, VarDictionary):
	return SdToolKitCustom.sdObjectInit( 'Revolution', object, VarDictionary )

#   revObjectWipe( object )
#   Removes an entity that has been previously initialized by sdObjectInit.
#   Returns False on failure, True on success.
def revObjectWipe( object ):
	return SdToolKitCustom.sdObjectWipe( 'Revolution', object )

#   revObjectExists( object )
#   Checks whether or not an object has been initialized by sdObjectInit.
#   Returns bool False on failure, bool True on success.
def revObjectExists( object ):
	return SdToolKitCustom.sdObjectExists( 'Revolution', object )
	
#   revObjectGetVal( object, 'VariableName' )
#   Fetches a specific variable's value from the object's data set.
def revObjectGetVal( object, var ):
	return SdToolKitCustom.sdObjectGetVal( 'Revolution', object, var )

#   revObjectSetVal( object, 'VariableName', any_value )
#   Stores a specific variable's value within the object's data set.
#   Returns bool False on failure, bool True on success.
def revObjectSetVal( object, var, val ):
	return SdToolKitCustom.sdObjectSetVal( 'Revolution', object, var, val )

#   revObjectChangeVal( object, 'VariableName', change_in_value )
#   Updates an existing variable's value within the object's data set.
#   Returns bool False on failure, bool True on success.
def revObjectChangeVal( object, var, delta ):
	return SdToolKitCustom.sdObjectChangeVal( 'Revolution', object, var, delta )

#   revObjectUpdateVal( object, 'VariableName', any_value )
#   Updates an existing variable's value within the object's data set.
#   Returns bool False on failure, bool True on success.
def revObjectUpdateVal( object, var, val ):
	return SdToolKitCustom.sdObjectUpdateVal( 'Revolution', object, var, val )


def initCity(pCity):

    if( revObjectExists(pCity) ) :
        revObjectWipe(pCity)

    cityData = {}
    cityData['WarningCounter'] = 0
    cityData['RevolutionCiv'] = -1
    cityData['RevolutionIndex'] = 0
    cityData['PrevRevIndex'] = 0
    cityData['LocalRevIndex'] = 0
    cityData['RevolutionCounter'] = 0
    cityData['ReinforcementCounter'] = 0
    cityData['RevolutionTurn'] = -1

    return revObjectInit( pCity, cityData )

# ----------- Functions for cities ------------------

def getCityVal(pCity, var):

    val = revObjectGetVal( pCity, var )
    if( val == None ) :
        initCity( pCity )
        return revObjectGetVal( pCity, 'WarningCounter' )

    return val

def setCityVal(pCity, var, val ) :

    success = revObjectSetVal( pCity, var, val )
    if( not success ) :
        initCity( pCity )
        return revObjectSetVal( pCity, var, val )

    return success

def updateCityVal(pCity, var, val ) :

    success = revObjectUpdateVal( pCity, var, val )
    if( not success ) :
        initCity( pCity )
        return revObjectUpdateVal( pCity, var, val )

    return success

def changeCityVal(pCity, var, delta):

    success = revObjectChangeVal( pCity, var, delta )
    if( not success ) :
        initCity( pCity )
        return revObjectChangeVal( pCity, var, delta )

    return success
        

########################## Player modification functions ###########################################

def changeCiv( playerIdx, newCivType = -1, newLeaderType = -1, teamIdx = -1 ) :
    # Changes specified players civ, leader and/or team
    # Does not change isHuman setting

    player = gc.getPlayer(playerIdx)
    oldCivType = player.getCivilizationType()
    oldLeaderType = player.getLeaderType()
    if( newCivType < 0 ) :
        newCivType = oldCivType
    if( newLeaderType < 0 ) :
        newLeaderType = oldLeaderType
    #print 'Changing player ... is human? %d'%(player.isHuman() or (gc.getActivePlayer().getID() == player.getID()))
    success = game.changePlayer( playerIdx, newCivType, newLeaderType, teamIdx, (player.isHuman() or (gc.getActivePlayer().getID() == playerIdx)), True )
    #print 'After change ... is human? %d'%(player.isHuman() or (gc.getActivePlayer().getID() == player.getID()))

    return success

def changeHuman( newHumanIdx, oldHumanIdx ) :

    newPlayer = gc.getPlayer(newHumanIdx)

    game.setActivePlayer( newHumanIdx, False )
    success = game.changePlayer( newHumanIdx, newPlayer.getCivilizationType(), newPlayer.getLeaderType(), -1, True, False )
    if( not success ) :
         CyInterface().addImmediateMessage(localText.getText("TXT_KEY_MOD_REVUTILS_ERROR1", ()),"")

    if( oldHumanIdx >= 0 ) :
        oldPlayer = gc.getPlayer(oldHumanIdx)
        success = game.changePlayer( oldHumanIdx, oldPlayer.getCivilizationType(), oldPlayer.getLeaderType(), -1, False, False )
        if( not success ) :
            CyInterface().addImmediateMessage(localText.getText("TXT_KEY_MOD_REVUTILS_ERROR2", ()),"")

    return success
