# REVOLUTION_MOD
#
# by jdog5000
# Version 0.91


from CvPythonExtensions import *
import CvUtil
import PyHelpers		
import Popup as PyPopup
import pickle
import RevUtils
try:
    import RebelTypes
except:
    pass

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
game = CyGame()
localText = CyTranslator()

class Revolution :

    def __init__(self, customEM, config):

        CvUtil.pyPrint("Initializing Revolution Mod")

        ####### System Variables #######
        self.EventKeyDown=6

        self.pickCivToSplitPopup = 7004
        self.revolutionPopup = 7007
        self.joinHumanPopup = 7009
        self.controlLostPopup = 7011
        self.revWatchPopup = 7012
        self.assimilationPopup = 7013

        ####### Revolution Variables ##########

        self.config = config
        self.customEM = customEM

        self.LOG_DEBUG = config.getboolean("Revolution", "DebugMode", False)
        self.DEBUG_MESS = config.getboolean("Revolution", "ShowDebugMessages", False)
        self.maxCivs = config.getint("Revolution", "MaxCivs", gc.getMAX_CIV_PLAYERS())
        if( self.maxCivs <= 1 ) :
            self.maxCivs = gc.getMAX_CIV_PLAYERS()
        self.enableRevoltStatusPopup = config.getboolean("Revolution", "EnableRevoltStatusPopup", True)
        self.allowAssimilation = config.getboolean("Revolution", "AllowAssimilation", True)
        self.revsGetFirstMove = config.getboolean("Revolution", "RevGetFirstMove", False)
        self.allowSmallBarbRevs = config.getboolean("Revolution", "AllowSmallBarbRevs", True)
        self.rebelEquipFrac = config.getint("Revolution", "RebelEquipFrac", 35)
        self.rebelCaptureSiege = config.getboolean("Revolution", "RebelCaptureSiege", True)
        self.bRebelTypes = config.getboolean("Revolution", "RebelTypes", True)
        self.bArtStyleTypes = config.getboolean("Revolution", "ArtStyleTypes", True)
        self.bAllowBreakVassal = config.getboolean("Revolution", "AllowBreakVassal", True)

        self.centerPopups = config.getboolean("Revolution", "CenterPopups", False)

        # Fewer than this number of cities won't try for independence, but change leader
        #self.minCitiesForIndependence = config.getint("Revolution","MinCitiesForIndependence",4)
        # Cities considered close to revolution instigator inside this radius
        self.closeRadius = config.getint("Revolution","CloseRadius",10)

        #### Cities may attempt to leave and join dominant culture
        self.culturalRevolution = config.getboolean("Revolution", "CulturalRevolution", True)
        # Cities with at least this level of your nationality won't start cultural revolutions
        self.maxNationalityThreshold = config.getint("Revolution", "MaxNationality", 40)

        #### Cities may ask for religious change or independence for religious reasons
        self.religiousRevolution = config.getboolean("Revolution", "ReligiousRevolution", True)
        # Cities with your state religion can join a religious revolution
        self.allowStateReligionToJoin = config.getboolean("Revolution", "AllowStateReligionToJoin", True)

        #### Cities may ask for civics changes
        self.civicRevolution = config.getboolean("Revolution", "CivicRevolution", True)

        #### Cities may ask/demand changes in leadership
        self.leaderRevolution = config.getboolean("Revolution", "LeaderRevolution", True)
        self.humanLeaderRevolution = config.getboolean("Revolution", "HumanLeaderRevolution", True)

        #### Cities may ask/demand independence
        # This is the base kind of revolution, not really negotiable if you have this component on
        self.independenceRevolution = True

        #self.warningThreshold = config.getint("Revolution", "HumanWarningThreshold", 900)
        self.warnFrac = config.getfloat("Revolution", "HumanWarnFrac", .9)
        self.warnTurns = config.getint("Revolution", "WarnTurns", 15)
        #self.revReadyThreshold = config.getint("Revolution", "JoinRevolutionThreshold", 600)
        self.revReadyFrac = config.getfloat("Revolution", "JoinRevolutionFrac", .6)
        self.revInstigatorThreshold = config.getint("Revolution", "InstigateRevolutionThreshold", 1000)
        self.alwaysViolentThreshold = config.getint("Revolution", "AlwaysViolentThreshold", 1700)
        self.badLocalThreshold = config.getint("Revolution", "BadLocalThreshold", 8)
        self.warWearinessMod = config.getfloat("Revolution", "WarWearinessMod", 1.0)

        self.turnsBetweenRevs = config.getint("Revolution", "TurnsBetweenRevs", 8)
        self.deniedTurns = config.getint("Revolution", "DeniedTurns", 5)
        self.acceptedTurns = config.getint("Revolution", "AcceptedTurns", 10)
        self.acquiredTurns = config.getint("Revolution", "AcquiredTurns", 10)
        self.buyoffTurns = config.getint("Revolution", "BuyoffTurns", 10)
        self.baseReinforcementTurns = config.getint("Revolution", "BaseReinforcementTurns", 3)
        self.minReinforcementTurns = config.getint("Revolution", "MinReinforcementTurns", 1)
        if( self.minReinforcementTurns < 1 ) :
            self.minReinforcementTurns = 1

        # Increase rate of accumulation of revolution index
        self.revIdxModifier = config.getfloat("Revolution", "IndexModifier", 1.0)
        self.revIdxOffset = config.getint("Revolution", "IndexOffset", 0)
        self.humanIdxModifier = config.getfloat("Revolution", "HumanIndexModifier", 1.0)
        self.humanIdxOffset = config.getint("Revolution", "HumanIndexOffset", 0)
        # Change chances a revolution occurs given conditions
        self.chanceModifier = config.getfloat("Revolution", "ChanceModifier", 1.0)
        # Change strength of revolutions that resort to violence
        self.strengthModifier = config.getfloat("Revolution", "StrengthModifier", 1.0)
        # Change number of rebel reinforcement units
        self.reinforcementModifier = config.getfloat("Revolution", "ReinforcementModifier", 1.0)
        # Change strength of city area revolution factor
        self.colonyModifier = config.getfloat("Revolution", "ColonyModifier", 1.0)
        # Change strength of civ size revolution factor
        self.civSizeModifier = config.getfloat("Revolution", "CivSizeModifier", 1.0)
        # Change strength of religion revolution factor
        self.religionModifier = config.getfloat("Revolution", "ReligionModifier", 1.0)
        # Change strength of losing a city on revolution indices
        self.cityLostModifier = config.getfloat("Revolution", "CityLostModifier", 1.0)
        # Change strength of gaining a city on revolution indices
        self.cityAcquiredModifier = config.getfloat("Revolution", "CityAcquiredModifier", 1.0)
        # Change the initial strength of rev culture in a captured city (1.0 = 50% nationality)
        self.revCultureModifier = config.getfloat("Revolution", "RevCultureModifier", 1.0)

        self.iRevRoll = 20000

        # Stores human leader type when human loses control of civ
        self.humanLeaderType = None

        # Stores player id's human has reject assimilation overtures from
        self.noAssimilateList = list()

        ######################
        # These are initialized on first end of game turn
        self.iNationalismTech = None
        self.iLiberalismTech = None
        self.iSciMethodTech = None

        ############ Holders for popup variables ##############

        self.cityList = None
        self.pPlayer = None
        self.revType = None
        self.peaceful = None
        self.pRevPlayer = None
        self.isJoinWar = None
        self.joinPlayer = None
        self.newLeaderType = None
        self.newLeaderName = None
        self.isElection = None
        self.newCivic = None
        self.newReligion = None
        self.iHappiness = None
        self.odds = None
        self.buyOffCost = None
        self.vassalStyle = None
        
        ############# Register events and popups ##############
        # City and civ events
        customEM.addEventHandler( "cityLost", self.onCityLost )
        #customEM.addEventHandler( "cityBuilt", self.onCityBuilt )
        customEM.addEventHandler( "cityAcquired", self.onCityAcquired )
        customEM.addEventHandler( "religionFounded", self.onReligionFounded )
        customEM.addEventHandler( "buildingBuilt", self.onBuildingBuilt )

        # All revolution index update and processing
        customEM.addEventHandler( "EndGameTurn", self.onEndGameTurn )
        customEM.addEventHandler( "BeginPlayerTurn", self.onBeginPlayerTurn )
            
        # Popup launching and handling events
        if( self.enableRevoltStatusPopup ) :
            customEM.addEventHandler( "kbdEvent", self.onKbdEvent )
        self.customEM.setPopupHandler( self.revolutionPopup, ["revolutionPopup",self.revolutionPopupHandler,self.blankHandler] )
        self.customEM.setPopupHandler( self.joinHumanPopup, ["joinHumanPopup",self.joinHumanHandler,self.blankHandler] )
        self.customEM.setPopupHandler( self.controlLostPopup, ["controlLostPopup",self.controlLostHandler,self.blankHandler] )
        self.customEM.setPopupHandler( self.revWatchPopup, ["revWatchPopup", self.revWatchHandler, self.blankHandler] )
        self.customEM.setPopupHandler( self.assimilationPopup, ["assimilationPopup", self.assimilateHandler, self.blankHandler] )

    def blankHandler( self, playerID, netUserData, popupReturn ) :
        # Dummy handler to take the second event for popup
        return

    def showHumanPopup( self, playerID ) :
        # Determines whether to show popup to active player

        if( playerID == game.getActivePlayer() ) :
            if( self.revsGetFirstMove ) :
                if( game.getAIAutoPlay() <= 1 ) :
                    return True
            else :
                if( game.getAIAutoPlay() <= 1 ) :
                    return True

        return False

######################################################################


    def onCityLost(self, argsList):
        'City Lost'
        city = argsList[0]
        player = gc.getPlayer(city.getOwner())

        self.playerCityLost( player, city, bConquest = (city.plot().getNumDefenders(player.getID()) == 0) )

    def onReligionFounded(self, argsList):
        'Religion Founded'
        iReligion, iFounder = argsList
        pPlayer = gc.getPlayer(iFounder)

        if( pPlayer.getStateReligion() >= 0 ) :
            if( not (pPlayer.getStateReligion() == iReligion) and not pPlayer.isAnarchy() ) :
                pCity = gc.getGame().getHolyCity(iReligion)
                curRevIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                RevUtils.updateCityVal(pCity, 'RevolutionIndex', max([350,curRevIdx+100]) )
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s founded non-state religion, index now %d ... state %d, new %d"%(pCity.getName(),RevUtils.getCityVal( pCity, 'RevolutionIndex' ),pPlayer.getStateReligion(),iReligion))

    def onBuildingBuilt( self, argsList):
        'Building Completed'
        pCity, iBuildingType = argsList

        buildingInfo = gc.getBuildingInfo(iBuildingType)
        buildingClassInfo = gc.getBuildingClassInfo(buildingInfo.getBuildingClassType())

        if( buildingClassInfo.getMaxGlobalInstances() == 1 and buildingInfo.getPrereqReligion() < 0 and buildingInfo.getProductionCost() > 10 ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - World wonder %s build in %s"%(buildingInfo.getDescription(),pCity.getName()))
            curRevIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
            RevUtils.changeCityVal(pCity, 'RevolutionIndex', -max([150,int(0.25*curRevIdx)]) )
            for city in PyPlayer(pCity.getOwner()).getCityList() :
                listCity = city.GetCy()
                curRevIdx = RevUtils.getCityVal( listCity, 'RevolutionIndex' )
                RevUtils.changeCityVal( listCity, 'RevolutionIndex', -max([75,int(0.12*curRevIdx)]) )

        elif( buildingClassInfo.getMaxPlayerInstances() == 1 and buildingInfo.getPrereqReligion() < 0 and buildingInfo.getProductionCost() > 10 ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - National wonder %s build in %s"%(buildingInfo.getDescription(),pCity.getName()))
            curRevIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
            RevUtils.changeCityVal(pCity, 'RevolutionIndex', -max([80,int(0.12*curRevIdx)]) )
            for city in PyPlayer(pCity.getOwner()).getCityList() :
                listCity = city.GetCy()
                curRevIdx = RevUtils.getCityVal( listCity, 'RevolutionIndex' )
                RevUtils.changeCityVal( listCity, 'RevolutionIndex', -max([50,int(0.07*curRevIdx)]) )


    def onEndGameTurn( self, argsList ) :

        if( self.iNationalismTech == None ) :
            self.loadInfo()

        self.topCivAdjustments( )

        if( self.allowAssimilation ) :
            self.checkForAssimilation( )

    def onBeginPlayerTurn( self, argsList ) :

        iGameTurn, iPlayer = argsList

        # Stuff at end of previous players turn
        iPrevPlayer = iPlayer - 1
        while( iPrevPlayer >= 0 and not gc.getPlayer(iPrevPlayer).isAlive() ) :
                iPrevPlayer -= 1

        if( iPrevPlayer < 0 ) :
            iPrevPlayer = gc.getBARBARIAN_PLAYER()

        if( iPrevPlayer >= 0 and iPrevPlayer < gc.getBARBARIAN_PLAYER() ) :
            self.checkForRevReinforcement( iPrevPlayer )

        # Stuff at beginning of this players turn
        self.updatePlayerRevolution( argsList )

    def onKbdEvent(self, argsList ):
        'keypress handler'
        eventType,key,mx,my,px,py = argsList

        if ( eventType == self.EventKeyDown ):
	    theKey=int(key)
	    
	    if( theKey == int(InputTypes.KB_W) and self.customEM.bShift and self.customEM.bCtrl ) :
                playerPy = PyPlayer( game.getActivePlayer() )
                cityList = playerPy.getCityList()
                danger = localText.getText("TXT_KEY_REV_WATCH_DANGER", ()) + ':\n'
                warning = '\n' + localText.getText("TXT_KEY_REV_WATCH_WARNING", ()) + ':\n'
                safe = '\n' + localText.getText("TXT_KEY_REV_WATCH_SAFE", ()) + ':\n'
                for city in cityList :
                    pCity = city.GetCy()
                    revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                    if( revIdx >= self.revInstigatorThreshold ) :
                        danger += "  %s\n"%(pCity.getName())
                    elif( revIdx >= int(self.revReadyFrac*self.revInstigatorThreshold) ) :
                        warning += "  %s\n"%(pCity.getName())
                    else :
                        safe += "  %s\n"%(pCity.getName())

                # Additions by Caesium et al
                caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                caesiumtextResolution = caesiumtR.split('x')
                caesiumpasx = int(caesiumtextResolution[0])/10
                caesiumpasy = int(caesiumtextResolution[1])/10
                popup = PyPopup.PyPopup(self.revWatchPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                popup.setHeaderString( localText.getText("TXT_KEY_REV_WATCH_TITLE", ()) )
                popup.setBodyString( danger + warning + safe )
                popup.addSeparator()
                popup.addButton( localText.getText("TXT_KEY_REV_WATCH_DETAIL", ()) )
                popup.addButton( 'OK' )
                popup.launch(bCreateOkButton = False)
                # End additions by Caesium et al

##            if( theKey == int(InputTypes.KB_O) and self.customEM.bShift and self.customEM.bCtrl ) :
##                self.doAssimilation( 3, game.getActivePlayer() )
                

    def revWatchHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( popupReturn.getButtonClicked() == 0 ):
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Showing detailed rev watch for player %d"%(iPlayerID))

            if( self.iNationalismTech == None ) :
                self.loadInfo()

            civString = self.updateCivRevIndices( iPlayerID, bIsRevWatch = True )

            cityString = self.updateLocalRevIndices( iPlayerID, bIsRevWatch = True )

            # Additions by Caesium et al
            caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
            caesiumtextResolution = caesiumtR.split('x')
            caesiumpasx = int(caesiumtextResolution[0])/10
            caesiumpasy = int(caesiumtextResolution[1])/10
            popup = PyPopup.PyPopup()
            if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
            popup.setHeaderString( gc.getPlayer(iPlayerID).getCivilizationDescription(0) )
            popup.setBodyString( civString + '\n\n' + cityString )
            popup.launch()
            # End additions by Caesium et al
        

######################################################################

    def loadInfo( self ) :

        #try:
        if( self.bRebelTypes ) :
            RebelTypes.setup()
##        except:
##            pass
        
        self.iNationalismTech = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_NATIONALISM')
        self.iLiberalismTech = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_LIBERALISM')
        self.iSciMethodTech = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_SCIENTIFIC_METHOD')

    def playerCityLost( self, player, city, bConquest = True ) :

        if( player.getNumCities() < 1 or player.isBarbarian() ) :
            return

        iPlayer = player.getID()
        capital = player.getCapitalCity()
        capitalArea = capital.area().getID()

        iRevIdxChange = 0

        if( game.getGameTurn() - city.getGameTurnAcquired() < 2 ) :
            iRevIdxChange = 0
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: very recently acquired, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( game.getGameTurn() - city.getGameTurnAcquired() < 20 ) :
            iRevIdxChange = 50
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: recently acquired, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( city.isCapital() ) :
            iRevIdxChange = 400
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: capital, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( city.getHighestPopulation() < 4 ) :
            iRevIdxChange = 50
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: small city, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( city.plot().calculateCulturePercent(iPlayer) > 60 and city.countTotalCultureTimes100() > 100*100 ) :
            iRevIdxChange = 150
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: majority nationality, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( city.getOriginalOwner() == iPlayer ) :
            iRevIdxChange = 125
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: original founder, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        else :
            iRevIdxChange = 100
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: default, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
            
        if( not city.area().getID() == capitalArea and iRevIdxChange > 25 ) :
            iRevIdxChange -= 25
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is a colony, reducing effect to %d"%(city.getName(),iRevIdxChange))

        if( not bConquest ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Not conquest, reducing rev index increase from %d"%(iRevIdxChange))
            iRevIdxChange = iRevIdxChange/3.0

        iRevIdxChange = int( self.cityLostModifier*iRevIdxChange )

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s (%d bConq): %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),bConquest,iRevIdxChange))

        for city in PyPlayer(iPlayer).getCityList() :
            pCity = city.GetCy()
            RevUtils.changeCityVal(pCity, 'RevolutionIndex', iRevIdxChange )


    def onCityAcquired( self, argsList):
	'City Acquired'
		
	owner,playerType,city,bConquest,bTrade = argsList

	owner = gc.getPlayer( city.getOwner() )
	prevOwner = gc.getPlayer( city.getPreviousOwner() )

        # Register loss with players other cities
	#self.playerCityLost( prevOwner, city, bConquest )

	if( owner.isBarbarian() ) :
            return
	
        newRevIdx = 400
        changeRevIdx = -40
        iPlayer = owner.getID()
        
        if( iPlayer == city.getOriginalOwner() ) :
            changeRevIdx -= 25
            newRevIdx -= 100

        if( city.getHighestPopulation() < 6 ) :
            changeRevIdx += 20
            newRevIdx -= 100

	if( bConquest ) :
            # Occupied cities also rack up rev points each turn
            newRevIdx += RevUtils.getCityVal( city, 'RevolutionIndex' )/4
            newRevIdx = min( [newRevIdx, 600] )

            if( city.plot().calculateCulturePercent( iPlayer ) > 90 ) :
                changeRevIdx -= 75
                newRevIdx -= 100
            elif( city.plot().calculateCulturePercent( iPlayer ) > 40 ) :
                changeRevIdx -= 35
                newRevIdx -= 60
            elif( city.plot().calculateCulturePercent( iPlayer ) > 20 ) :
                changeRevIdx -= 30
            
        elif( bTrade ) :
            newRevIdx += RevUtils.getCityVal( city, 'RevolutionIndex' )/4
            newRevIdx = min( [newRevIdx, 650] )

            if( city.plot().calculateCulturePercent( iPlayer ) > 90 ) :
                newRevIdx -= 50
            
        else :
            # Probably cultural conversion
            newRevIdx -= 100

            if( city.plot().calculateCulturePercent( iPlayer ) > 50 ) :
                changeRevIdx -= 25

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Acquisition of %s by %s reduces rev indices by %d"%(city.getName(),owner.getCivilizationDescription(0),changeRevIdx))

        changeRevIdx = int( self.cityAcquiredModifier*changeRevIdx )

        for listCity in PyPlayer( iPlayer ).getCityList() :
            pCity = listCity.GetCy()
            RevUtils.changeCityVal(pCity, 'RevolutionIndex', changeRevIdx )

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - New rev idx for %s is %d"%(city.getName(),newRevIdx))

        RevUtils.updateCityVal( city, 'RevolutionIndex', newRevIdx )
        RevUtils.updateCityVal( city, 'RevolutionCounter', self.acquiredTurns )
        RevUtils.updateCityVal( city, 'ReinforcementCounter', 0 )

############################  Player turn revolution functions  ###################################

    def checkForRevReinforcement( self, iPlayer ) :

        # Checks iPlayer's cities for any rebel reinforcement units that should be spawned
        # Should be called at end of player's turn

        playerPy = PyPlayer( iPlayer )
        cityList = playerPy.getCityList()

        #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Checking player %d's cities for rebel reinforcement"%(iPlayer))

        for city in cityList :
            pCity = city.GetCy()
            if( RevUtils.getCityVal(pCity, 'ReinforcementCounter') == 1 ) :
                # Do something awesome
                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Checking player %d's city %s for rebel reinforcement spawning"%(iPlayer,pCity.getName()))
                self.doRevReinforcement( pCity )

        return

    def doRevReinforcement( self, pCity ) :

        revCivType = RevUtils.getCityVal(pCity, 'RevolutionCiv')
        ownerID = pCity.getOwner()
        owner = gc.getPlayer(ownerID)

        # City must have valid rev player
        if( revCivType < 0 ) :
            return
        if( revCivType == owner.getCivilizationType() ) :
            return

        pRevPlayer = None
        for i in range(0,gc.getMAX_CIV_PLAYERS()) :
            playerI = gc.getPlayer(i)
            if( playerI.isAlive() and playerI.getCivilizationType() == revCivType ) :
                pRevPlayer = playerI
                break

        if( pRevPlayer == None ) :
            return

        pRevTeam = gc.getTeam(pRevPlayer.getTeam())

        if( not pRevTeam.isAtWar(owner.getTeam()) ) :
            return

        # City must still be rebellious
        revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
        localRevIdx = RevUtils.getCityVal(pCity, 'LocalRevIndex')

        if( revIdx < self.revInstigatorThreshold ) :
            return
        if( localRevIdx < -(self.badLocalThreshold/2) ) :
            return

        rebPower = pCity.area().getPower( pRevPlayer.getID() )
        ownerPower = pCity.area().getPower( ownerID )

        if( rebPower > 2*ownerPower ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Reiforcing: Rebel power %d much higher than owner %d in area, No Reinforce for 3 turns"%(rebPower,ownerPower))
            RevUtils.updateCityVal(pCity, 'ReinforcementCounter', 3 )
            return

        # Do reinforcement
        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Reinforcing rebel %s outside %s (owned by %s)"%(pRevPlayer.getCivilizationDescription(0),pCity.getName(),owner.getCivilizationDescription(0)))
        spawnableUnits = RevUtils.getUprisingUnitTypes( pCity, pRevPlayer, True )
        #[iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, pRevPlayer )

        if( len(spawnableUnits) < 1 ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - ERROR!!! No rev units possible in %s"%(pCity.getName()))
            return

        ix = pCity.getX()
        iy = pCity.getY()

        if( RevUtils.getNumDefendersNearPlot( ix, iy, pRevPlayer.getID(), iRange = 3 ) == 0 ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No rebel troops to reinforce, try again later")
            RevUtils.updateCityVal(pCity, 'ReinforcementCounter', 2 + 1 )
            return            

        spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 2, iSpawnPlotOwner = pRevPlayer.getID(), bCheckForEnemy = True )
        if( len(spawnablePlots) == 0 ) :
            spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 3, iSpawnPlotOwner = pRevPlayer.getID(), bCheckForEnemy = True )
        if( len(spawnablePlots) == 0 ) :
            spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 3, iSpawnPlotOwner = -1, bCheckForEnemy = True )

        if( len(spawnablePlots) == 0 ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - ERROR!!! No rev spawn location possible in %s"%(pCity.getName()))
            return
        revSpawnLoc = spawnablePlots[game.getSorenRandNum(len(spawnablePlots),'Revolution: Pick rev plot')]

        iNumUnits = int( self.reinforcementModifier*(revIdx/(1.0*self.revInstigatorThreshold) + 1) )
        if( game.getGameTurn() - RevUtils.getCityVal(pCity, 'RevolutionTurn') < 3 ) :  
            iNumUnits = min([iNumUnits,(pCity.getPopulation())/4,localRevIdx/2])
        else :
            # Been a few turns since revolution, turn down intensity
            iNumUnits = min([iNumUnits,(pCity.getPopulation())/6,localRevIdx/4])

        if( RevUtils.getCityVal(pCity, 'RevolutionCounter') == 0 ) :
            iNumUnits -= 1
        if( RevUtils.getNumDefendersNearPlot( ix, iy, pRevPlayer.getID(), iRange = 2 ) > RevUtils.getNumDefendersNearPlot( ix, iy, ownerID, iRange = 2 ) ) :
            iNumUnits -= 1
            
        iNumUnits = max([iNumUnits,1])

        
        if( ownerID == game.getActivePlayer() or pRevTeam.isAtWar(gc.getActivePlayer().getTeam()) ) :
            mess = localText.getText("TXT_KEY_REV_MESS_REINFORCEMENTS",()) + " %s!"%(pCity.getName())
            CyInterface().addImmediateMessage(mess,"")
        elif( pRevPlayer.getID() == game.getActivePlayer() ) :
            mess = localText.getText("TXT_KEY_REV_MESS_YOUR_REINFORCEMENTS",()) + " %s!"%(pCity.getName())
            CyInterface().addImmediateMessage(mess,"")
            
        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spawning %d reinforcements for city of size %d"%(iNumUnits,pCity.getPopulation()))

        if( len(RevUtils.getEnemyUnits( revSpawnLoc[0], revSpawnLoc[1], pRevPlayer.getID())) > 0 ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - ERROR!  Spawning on plot with enemy units!!!")

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City at %d,%d spawning at %d,%d"%(ix,iy,revSpawnLoc[0],revSpawnLoc[1]))

        # Spawn rev units outside city
        newUnitList = list()
        for i in range(0,iNumUnits) :
            newUnitID = spawnableUnits[game.getSorenRandNum( len(spawnableUnits), 'Revolution: pick unit' )]
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Outside of %s, spawning %s"%(pCity.getName(),PyInfo.UnitInfo(newUnitID).getDescription()))
            newUnit = pRevPlayer.initUnit( newUnitID, revSpawnLoc[0], revSpawnLoc[1], UnitAITypes.NO_UNITAI )
            newUnitList.append( newUnit )

        for newUnit in newUnitList :
            # Injure units to simulate the lack of training in rebel troops
            iDamage = 10 + game.getSorenRandNum(15,'Rev - Injure unit')
            newUnit.setDamage( iDamage, ownerID )

            # Check AI settings
            iniAI = newUnit.getUnitAIType()
            if( iniAI == UnitAITypes.UNITAI_CITY_DEFENSE ) :
                newUnit.setUnitAIType( UnitAITypes.UNITAI_ATTACK )
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s starting with AI type: %d (ini %d)"%(newUnit.getName(),newUnit.getUnitAIType(),iniAI))

        # Set reinforcement timer again
        iReinforceTurns = self.baseReinforcementTurns - revIdx/self.revInstigatorThreshold
        if( localRevIdx > 0 ) :
            localFactor = min([localRevIdx,2*self.badLocalThreshold])
            iReinforceTurns -= localFactor/4
        else :
            iReinforceTurns += localRevIdx/2 + 1
        if( pCity.getPopulation() < 7 ) :
            iReinforceTurns += 7 - pCity.getPopulation()
            if( pCity.getPopulation() < 5 ) :
                iReinforceTurns = max([iReinforceTurns,4-pCity.getPopulation()/2])

        minReinfTurns = self.minReinforcementTurns
        if( owner.getCurrentEra() < 5 ) :
            minReinfTurns += 3 - (owner.getCurrentEra()+1)/2
            iReinforceTurns += 2 - (owner.getCurrentEra())/2
        iReinforceTurns = max([iReinforceTurns,minReinfTurns])
        if( RevUtils.getCityVal(pCity, 'RevolutionCounter') == 0 ) :
            iReinforceTurns += 2
        elif( game.getGameTurn() - RevUtils.getCityVal(pCity, 'RevolutionTurn') > 3 ) :
            iReinforceTurns += 1
        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s reinforcement counter set to %d"%(pCity.getName(),iReinforceTurns))
        RevUtils.updateCityVal(pCity, 'ReinforcementCounter', iReinforceTurns + 1 )
        

    def updatePlayerRevolution( self, argsList ):

        iGameTurn, iPlayer = argsList

        if( iPlayer == gc.getBARBARIAN_PLAYER() ) :
            return

        if( self.iNationalismTech == None ) :
            self.loadInfo()
            
        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 and iPlayer == 0 ) : CvUtil.pyPrint("  Revolt - Rev index report for year %d"%(game.getGameTurnYear()))

        self.updateRevolutionCounters( iPlayer )
        self.checkCivics( iPlayer )
        self.updateLocalRevIndices( iPlayer )
        self.updateCivRevIndices( iPlayer )
        self.checkForRevolution( iPlayer )

        return

    def updateRevolutionCounters( self, iPlayer ) :
            
        playerPy = PyPlayer( iPlayer )
        cityList = playerPy.getCityList()

        for city in cityList :
            pCity = city.GetCy()

            if( RevUtils.getCityVal(pCity, 'RevolutionCounter') > 0 ) :
                RevUtils.changeCityVal(pCity, 'RevolutionCounter', -1 )

            if( RevUtils.getCityVal( pCity, 'WarningCounter' ) > 0 ) :
                RevUtils.changeCityVal(pCity, 'WarningCounter', -1 )

            if( RevUtils.getCityVal(pCity, 'ReinforcementCounter') > 0 ) :
                RevUtils.changeCityVal(pCity, 'ReinforcementCounter', -1 )

    def checkCivics( self, iPlayer ) :

        pPlayer = gc.getPlayer( iPlayer )

        if( pPlayer.getNumCities() == 0 or pPlayer.isBarbarian() ) :
            return

        curCivics = list()

        for i in range(0,gc.getNumCivicOptionInfos()):
            curCivics.append( pPlayer.getCivics(i) )

        if( not RevUtils.revObjectExists(pPlayer) ) :
            RevUtils.revObjectInit( pPlayer, dict() )

        prevCivics = RevUtils.revObjectGetVal( pPlayer, "CivicList" )

        if( prevCivics == None or not len(prevCivics) == len(curCivics) ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Setting civics for %s"%(pPlayer.getCivilizationDescription(0)))
            RevUtils.revObjectSetVal( pPlayer, "CivicList", curCivics )
            return
        
        else :
            sumRevIdx = 0
            bChanged = False
            for [i,curCivic] in enumerate(curCivics) :
                if( not curCivic == prevCivics[i] ) :
                    curInfo  = gc.getCivicInfo(curCivic)
                    prevInfo = gc.getCivicInfo(prevCivics[i])
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s changing civic option %d from %s to %s"%(pPlayer.getCivilizationDescription(0),i,prevInfo.getDescription(),curInfo.getDescription()))
                    sumRevIdx += curInfo.getRevIdxSwitchTo() - prevInfo.getRevIdxSwitchTo()
                    bChanged = True

            if( not sumRevIdx == 0 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Net effect for %s: %d"%(pPlayer.getCivilizationDescription(0),sumRevIdx))
                for city in PyPlayer( iPlayer ).getCityList() :
                    pCity = city.GetCy()
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', sumRevIdx )

            if( bChanged ) :
                RevUtils.revObjectSetVal( pPlayer, "CivicList", curCivics )
                    

    def updateLocalRevIndices( self, iPlayer, subCityList = None, bIsRevWatch = False ) :
        # Updates the revolution effects local to each city

        # Adjust index accumulation for varying game speeds
        gameSpeedMod = (661.0)/(game.getEstimateEndTurn() + 1.0)
        if( gameSpeedMod == 661.0 ) : gameSpeedMod = 1.0
        #print "Revolt - Game speed mod is ", gameSpeedMod

        defNumPlayers = gc.getWorldInfo(gc.getMap().getWorldSize()).getDefaultPlayers()
	expCivPlots = CyMap().getLandPlots() / ( defNumPlayers * 1.0)

        pPlayer = gc.getPlayer(iPlayer)
        if( pPlayer.getNumCities() == 0 ) :
            return localText.getText("TXT_KEY_REV_WATCH_NO_CITIES",())

        pTeam = gc.getTeam( pPlayer.getTeam() )
        playerPy = PyPlayer( iPlayer )
        if( subCityList == None ) :
            cityList = playerPy.getCityList()
        else :
            cityList = subCityList

        hasNationalism = gc.getTeam(pPlayer.getTeam()).isHasTech(self.iNationalismTech)
        hasLiberalism = gc.getTeam(pPlayer.getTeam()).isHasTech(self.iLiberalismTech)
        hasSciMethod = gc.getTeam(pPlayer.getTeam()).isHasTech(self.iSciMethodTech)
        iCivNumLandPlots = pPlayer.getTotalLand()
        civSizeValue = self.civSizeModifier*iCivNumLandPlots / (1.0*expCivPlots)
        civSizeEraMod = max( [0, 0.70-0.20*pow(pPlayer.getCurrentEra(),.7)] )
        iCivEffRadius = pow((.5*iCivNumLandPlots + .5*expCivPlots)/3.4, .5)
##        if( pPlayer.getCurrentEra() < 3 ) :
##            civSizeEraMod = civSizeEraMod*(3.5 - pPlayer.getCurrentEra())
        capital = pPlayer.getCapitalCity()

        totalString = localText.getText("TXT_KEY_REV_WATCH_CITY_BY_CITY",())

        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s have civ size value %f, era mod %f"%(pPlayer.getCivilizationDescription(0),civSizeValue,civSizeEraMod))

        civSizeValue += civSizeEraMod

        for city in cityList :
            try:
                pCity = city.GetCy()
            except:
                # already a CyCity object
                pCity = city
            
            prevRevIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )

            posString = ''
            negString = ''

            # Sum up local factors for the city
            localRevIdx = 0

            # Happiness
            numUnhappy = RevUtils.getModNumUnhappy( pCity, self.warWearinessMod )
            happyIdx = 0
            if( numUnhappy > 0 ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_HAPPINESS",()) + ', '
                if( pCity.isOccupation() ) :
                    # Resistance is counted below ...
                    numUnhappy = min([numUnhappy/2.0,3.0])
                    
                happyIdx = int( 20*pow(numUnhappy, .8) )
            elif( numUnhappy < 0 ) :
                if( bIsRevWatch ) : posString += localText.getText("TXT_KEY_REV_WATCH_HAPPINESS",()) + ', '
                numHappy = abs(numUnhappy)
                happyIdx = -int( 2*pow(numHappy, .5) )

            localRevIdx += happyIdx

            ww = pCity.getWarWearinessPercentAnger()
            if( ww > 0 and self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Rev - %s has ww %d"%(pCity.getName(),ww))

            #if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s happiness index: %d"%(pCity.getName(),localRevIdx))

            # Connected to capital
            if( not pCity.isConnectedTo(capital) ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_NOT_CONNECTED",()) + ', '
                localRevIdx += 6

            # Distance to capital & Civ Size
            # Distance to captial divided by a little less than civ's "radius"
            cityDistModifier = plotDistance( pCity.getX(), pCity.getY(), capital.getX(), capital.getY() )/(iCivEffRadius)
            cityDistModifier = cityDistModifier*(1+0.08*gc.getNumEraInfos()-0.08*pPlayer.getCurrentEra())
            cityDistModifier = min([cityDistModifier, 3.5])
            
            cityDistModifier = self.civSizeModifier*cityDistModifier

            locationRevIdx = 0

            if( civSizeValue > 2.0 ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_HUGE_EMP",()) + ', '
                locationRevIdx += 4
                locationRevIdx += int(2.0*cityDistModifier + .5)
                if( bIsRevWatch and cityDistModifier > 1.5 ) :
                    negString += localText.getText("TXT_KEY_REV_WATCH_DISTANT",()) + ', '
            elif( civSizeValue > 1.6 ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_LARGE_EMP",()) + ', '
                locationRevIdx += 3
                locationRevIdx += int(1.6*cityDistModifier + .5)
                if( bIsRevWatch and cityDistModifier > 1.6 ) :
                    negString += localText.getText("TXT_KEY_REV_WATCH_DISTANT",()) + ', '
            elif( civSizeValue > 1.4 ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_LARGE_EMP",()) + ', '
                locationRevIdx += 2
                locationRevIdx += int(1.3*cityDistModifier + .5)
                if( bIsRevWatch and cityDistModifier > 1.7 ) :
                    negString += localText.getText("TXT_KEY_REV_WATCH_DISTANT",()) + ', '
            elif( civSizeValue > 1.2 ) :
                locationRevIdx += 1
                locationRevIdx += int(1.1*cityDistModifier + .5)
                if( bIsRevWatch and cityDistModifier > 1.8 ) :
                    negString += localText.getText("TXT_KEY_REV_WATCH_DISTANT",()) + ', '
            elif( civSizeValue > 1.0 ) :
                locationRevIdx += 0
                locationRevIdx += int(cityDistModifier + .5)
                if( bIsRevWatch and cityDistModifier > 1.9 ) :
                    negString += localText.getText("TXT_KEY_REV_WATCH_DISTANT",()) + ', '
            elif( civSizeValue > .7 ) :
                if( bIsRevWatch ) : posString += localText.getText("TXT_KEY_REV_WATCH_MED_EMP",()) + ', '
                locationRevIdx += int(.7*cityDistModifier + .5)
                if( bIsRevWatch and cityDistModifier > 2.2 ) :
                    negString += localText.getText("TXT_KEY_REV_WATCH_DISTANT",()) + ', '
            else :
                if( bIsRevWatch ) : posString += localText.getText("TXT_KEY_REV_WATCH_SMALL_EMP",()) + ', '
                locationRevIdx += int(.5*cityDistModifier + .5)
                if( bIsRevWatch and cityDistModifier > 3.0 ) :
                    negString += localText.getText("TXT_KEY_REV_WATCH_DISTANT",()) + ', '

            # Area
            if( not (pCity.area().getID() == capital.area().getID()) ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_COLONY",()) + ', '
                if( hasNationalism ) :
                    locationRevIdx += int( self.colonyModifier*5 )
                else :
                    locationRevIdx += int( self.colonyModifier*3 )

            if( locationRevIdx > 0 ) :

                distMod = RevUtils.getCivicsDistanceMod( iPlayer )
                locationRevIdx = int( locationRevIdx*(1.0+distMod) + .5)

            localRevIdx += int( locationRevIdx )

            # Civic choices

            #if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s location effects: %d  total: %d"%(pCity.getName(),locationRevIdx,localRevIdx))

            # Religion
            relIdx = 0
            if( pPlayer.isStateReligion() ) :
                stateRel = pPlayer.getStateReligion()
                relGoodIdx = 0
                relBadIdx = 0
                for relType in range(0,gc.getNumReligionInfos()) :
                    if( pCity.isHasReligion(relType) ) :
                        if( relType == stateRel ) :
                            relGoodIdx += 4
                        else :
                            if( relBadIdx > 4 ) :
                                relBadIdx += 1
                            else :
                                relBadIdx += 2.5

                # Holy city
                if( pCity.isHolyCity() ) :
                    if( pCity.isHolyCityByType(stateRel) ) :
                        relGoodIdx += 5
                    else :
                        relBadIdx += 4

                [holyCityGood,holyCityBad] = RevUtils.getCivicsHolyCityEffects(iPlayer)
                if( not hasLiberalism and pCity.isHasReligion(stateRel) ) :
                    stateHolyCity = game.getHolyCity( stateRel )
                    if( not stateHolyCity.isNone() ) :
                        holyCityOwnerID = stateHolyCity.getOwner()
                        if( holyCityOwnerID == iPlayer ) :
                            posString += localText.getText("TXT_KEY_REV_WATCH_HOLY_CITY",()) + ', '
                            relGoodIdx += holyCityGood
                        elif( not gc.getPlayer(holyCityOwnerID).getStateReligion() == stateRel ) :
                            negString += localText.getText("TXT_KEY_REV_WATCH_HEATHENS",()) + ', '
                            localRevIdx += holyCityBad

                [relGoodMod,relBadMod] = RevUtils.getCivicsReligionMods(iPlayer)

                relBadIdx = int(relBadIdx*(1.0 + relBadMod) + .5)
                relGoodIdx = int(relGoodIdx*(1.0 + relGoodMod) + .5)

                if( pTeam.getAtWarCount(True) > 1 ) :
                    # God is on your side =P
                    relGoodIdx = int(relGoodIdx*1.5 + .5)

                relIdx = int( self.religionModifier*(relBadIdx - relGoodIdx) + .5 )

                if( hasSciMethod ) :
                    relIdx = relIdx/3
                elif( hasLiberalism ) :
                    relIdx = relIdx/2

                if( bIsRevWatch ) :
                    if( relIdx > 0 ) :
                        negString += localText.getText("TXT_KEY_REV_WATCH_RELIGION",()) + ', '
                    elif( relIdx < 0 ) :
                        posString += localText.getText("TXT_KEY_REV_WATCH_RELIGION",()) + ', '
                        

                localRevIdx += relIdx
                #if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s religious index: %d, total: %d"%(pCity.getName(),relIdx,localRevIdx))
                
            # Culture
            cultIdx = -int( pow(abs(pCity.getCommerceRate(CommerceTypes.COMMERCE_CULTURE)/3.0), .5) + .5 )
            if( bIsRevWatch and cultIdx < -2 ) : posString += localText.getText("TXT_KEY_REV_WATCH_CULTURE_RATE",()) + ', '
            localRevIdx += cultIdx

            # Nationality
            natIdx = 0
            recentlyAcquired = (game.getGameTurn()-pCity.getGameTurnAcquired() < 20)
            culturePercent = 0
            maxCult = 0
            maxCultPlayer = -1
            for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( pPlayer.getTeam() == gc.getPlayer(idx).getTeam() ) :
                    culturePercent += pCity.plot().calculateCulturePercent(idx)
                if( pCity.plot().calculateCulturePercent(idx) > maxCult ) :
                    maxCult = pCity.plot().calculateCulturePercent(idx)
                    maxCultPlayer = idx

            if( maxCultPlayer < 0 ) :
                maxCultPlayer = iPlayer
                    
            if( culturePercent > 90 ) :
                natIdx = -4
            elif( culturePercent > 70 ) :
                natIdx = -2
            elif( culturePercent < 20 ) :
                natIdx = 26
                natIdx -= pCity.getCommerceRate(CommerceTypes.COMMERCE_CULTURE)
                natIdx = max([natIdx,9])
            elif( culturePercent < 40 ) :
                natIdx = 15
                natIdx -= pCity.getCommerceRate(CommerceTypes.COMMERCE_CULTURE)
                natIdx = max([natIdx,6])
            elif( culturePercent < 50 ) :
                natIdx = 8
                natIdx -= pCity.getCommerceRate(CommerceTypes.COMMERCE_CULTURE)/2
                natIdx = max([natIdx,3])
            elif( culturePercent < 60 ) :
                natIdx = 3
                natIdx -= pCity.getCommerceRate(CommerceTypes.COMMERCE_CULTURE)/2
                natIdx = max([natIdx,0])

            if( recentlyAcquired ) :
                natIdx = int((3*natIdx)/5.0)
                natIdx = min([natIdx,10])

            if( pTeam.isAtWar(gc.getPlayer(maxCultPlayer).getID()) and natIdx > 0 ) :
                natIdx = (3*natIdx)/2

            if( not hasNationalism ) :
                natIdx = int((1*natIdx)/2.0)

            if( natIdx > 0 ) :
                natMod = RevUtils.getCivicsNationalityMod( iPlayer )
                natIdx = int( natIdx*(1.0 + natMod) + .5 )
                
##                iGovernmentCivic = pPlayer.getCivics(self.iCivicOptionGovernment)
##                if( iGovernmentCivic == self.iUniversalSuffrageID ) :
##                    natIdx = (2*natIdx)/3
##                elif( iGovernmentCivic == self.iRepresentationID ) :
##                    natIdx = (3*natIdx)/4

            if( bIsRevWatch ) :
                if( natIdx > 0 ) :
                    negString += localText.getText("TXT_KEY_REV_WATCH_NATIONALITY",()) + ', '
                elif( natIdx < 0 ) :
                    posString += localText.getText("TXT_KEY_REV_WATCH_NATIONALITY",()) + ', '

            localRevIdx += natIdx

            # Size
            if( pCity.getPopulation() < 5 ) :
                if( pCity.isCapital() and pPlayer.getCurrentEra() - game.getStartEra() > 2 ) :
                    # To help remove late game tiny civs
                    localRevIdx += 4
                else :
                    if( bIsRevWatch ) : posString += localText.getText("TXT_KEY_REV_WATCH_SMALL_CITY",()) + ', '
                    localRevIdx -= 6 - pCity.getPopulation()

            # Health
            numUnhealthy = (pCity.badHealth(false) - pCity.goodHealth())
            if( numUnhealthy > 0 ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_UNHEALTHY",()) + ', '
                localRevIdx += int( 2*pow(numUnhealthy, .6) + .5 )

            # Garrison
            garIdx = -int( 2*pow(pCity.plot().getNumDefenders(iPlayer)/2, .7) - .5 )
            if( bIsRevWatch and garIdx < -2 ) :
                posString += 'Garrison, '
            if( natIdx > 0 ) :
                garIdx = max([garIdx,-5])
            else :
                garIdx = max([garIdx,-8])
            localRevIdx += garIdx

            # Spirit
            if( pCity.getNumRevolts(pPlayer.getID()) > 2 and prevRevIdx > int(self.warnFrac*self.revInstigatorThreshold) ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_SPIRIT",()) + ', '
                localRevIdx += pCity.getNumRevolts(pPlayer.getID())/2

            # Starving
            if( pCity.foodDifference(True) < 0 and abs(pCity.foodDifference(True)) > pCity.getFood() ) :
                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City %s is starving!"%(pCity.getName()))
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_STARVATION",()) + ', '
                localRevIdx += 100

            # Disorder
            if( pCity.isOccupation() ) :
                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City %s is in Disorder!"%(pCity.getName()))
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_DISORDER",()) + ', '
                if( game.getGameTurn() - pCity.getGameTurnAcquired() < 15 ) :
                    # Give recently acquired cities a break
                    localRevIdx += 10
                else :
                    localRevIdx += 50

            # Adjust for game speed
            localRevIdx = int( gameSpeedMod*self.revIdxModifier*localRevIdx + self.revIdxOffset + .5 )
            if( pPlayer.isHuman() ) :
                localRevIdx = int( self.humanIdxModifier*localRevIdx + self.humanIdxOffset + .5 )

            revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
            
            if( not bIsRevWatch ) :
                RevUtils.updateCityVal(pCity, 'LocalRevIndex', localRevIdx )
                if( localRevIdx < 0 and revIdx > self.alwaysViolentThreshold ) :
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', 2*localRevIdx )
                elif( localRevIdx < 0 and revIdx > self.revInstigatorThreshold ) :
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', int(1.2*localRevIdx) )

            if( RevUtils.getCityVal( pCity, 'RevolutionIndex' ) < 0 ) :
                RevUtils.updateCityVal(pCity, 'RevolutionIndex', 0 )

            revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )

            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s:   Hap %d,   Loc %d,   Rel %d,   Nat %d,   Cult %d,   Gar %d"%(pCity.getName(),happyIdx,locationRevIdx,relIdx,natIdx,cultIdx,garIdx))
            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt -         Local effects for %s (%s):  %d   with total  %d"%(pCity.getName(),pPlayer.getCivilizationDescription(0),localRevIdx,revIdx))

            cityString = '\n\n' + pCity.getName()
            if( revIdx >= self.revInstigatorThreshold ) :
                cityString += ' - ' + localText.getText("TXT_KEY_REV_WATCH_DANGER",())
            elif( revIdx >= int(self.revReadyFrac*self.revInstigatorThreshold) ) :
                cityString += ' - ' + localText.getText("TXT_KEY_REV_WATCH_WARNING",())
            else :
                cityString += ' - ' + localText.getText("TXT_KEY_REV_WATCH_SAFE",())
            if( localRevIdx < 0 ) :
                cityString += '  ' + localText.getText("TXT_KEY_REV_WATCH_IMPROVING",())
            else :
                cityString += '  ' + localText.getText("TXT_KEY_REV_WATCH_WORSENING",())
            cityString += '\n' + localText.getText("TXT_KEY_REV_WATCH_POSITIVE",()) + '  ' + posString.strip(', ') + '\n' + localText.getText("TXT_KEY_REV_WATCH_NEGATIVE",()) + '  ' + negString.strip(', ')
            totalString += cityString

        return totalString

    def updateCivRevIndices( self, iPlayer, bIsRevWatch = False ) :
        # Update the revolution effects for the entire empire

        posString = ''
        negString = ''

        # Adjust index accumulation for varying game speeds
        gameSpeedMod = (661.0)/(game.getEstimateEndTurn() + 1.0)
        if( gameSpeedMod == 661.0 ) : gameSpeedMod = 1.0
        #print "Revolt - Game speed mod is ", self.gameSpeedMod

        pPlayer = gc.getPlayer(iPlayer)
        if( pPlayer.getNumCities() == 0 ) :
            return localText.getText("TXT_KEY_REV_WATCH_NO_CITIES",())

        pTeam = gc.getTeam( pPlayer.getTeam() )

        civRevIdx = 0

        # Finances
        finIdx = 0
        if( pTeam.getAtWarCount(True) == 0 ) :
            if( pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) > 50 ) :

                if( pPlayer.getGold() < 40 + 20*pPlayer.getCurrentEra() and pPlayer.calculateGoldRate() < 1 + pPlayer.getCurrentEra() ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in serious financial trouble, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_SERIOUS_FINANCIAL",()) + ', '
                    finIdx += 2*(pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50) + 15
                elif( pPlayer.getGold() < 100 + 30*pPlayer.getCurrentEra() and pPlayer.calculateGoldRate() < 5 + pPlayer.getCurrentEra() ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in financial trouble, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_FINANCIAL",()) + ', '
                    finIdx += (pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50) + 5
                else :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are at peace and have very high taxes, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_TAXES",()) + ', '
                    finIdx += 2*(pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50)/3

        else :
            if( pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) > 70 ) :
                
                if( pPlayer.getGold() < 80 + 25*pPlayer.getCurrentEra() and pPlayer.calculateGoldRate() < 6 + pPlayer.getCurrentEra() ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in serious financial trouble, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_SERIOUS_FINANCIAL",()) + ', '
                    finIdx += (pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50) + 10
                elif( pPlayer.getGold() < 100 + 40*pPlayer.getCurrentEra() and pPlayer.calculateGoldRate() < 4 + pPlayer.getCurrentEra() ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in financial trouble, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_FINANCIAL",()) + ', '
                    finIdx += (pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50)/2 + 5
                else :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are at war and have very high taxes, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_TAXES",()) + ', '
                    finIdx += pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 70

        civRevIdx += finIdx

        [civicIdx,civicGoodStr,civicBadStr] = RevUtils.getCivicsRevIdxLocal( iPlayer )

        civRevIdx += civicIdx
        posString += civicGoodStr
        negString += civicBadStr

        # Helps catch tiny civs in late game
        if( pPlayer.getNumMilitaryUnits() > 2*pPlayer.getTotalPopulation() and pPlayer.getTotalPopulation()/pPlayer.getNumCities() < 4 ) :
            if( pPlayer.getCurrentEra() - game.getStartEra() > 2 ) :
                if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_MILITARY",()) + ', '
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt -    The %s have way more troops than citizens"%(pPlayer.getCivilizationDescription(0)))
                civRevIdx += int( 4*pPlayer.getNumMilitaryUnits()/pPlayer.getTotalPopulation() )
        
        # Any civic or religion changes cause anarchy for non-spiritual civs
        # TODO: single turn anarchy doesn't show up at beginning or end of players turn
        if( pPlayer.isAnarchy() ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in anarchy!"%(pPlayer.getCivilizationDescription(0)))
            if( bIsRevWatch ) : negString += localText.getText("TXT_KEY_REV_WATCH_ANARCHY",()) + ', '
            civRevIdx += 100

        if( pPlayer.isGoldenAge() ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are in their golden age!"%(pPlayer.getCivilizationDescription(0)))
            if( bIsRevWatch ) : posString += localText.getText("TXT_KEY_REV_WATCH_GOLDEN_AGE",()) + ', '
            civRevIdx -= 20

        civRevIdx = int( gameSpeedMod*self.revIdxModifier*civRevIdx + .5 )
        if( pPlayer.isHuman() ) :
            civRevIdx = int( self.humanIdxModifier*civRevIdx + .5 )

        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s civ wide effects: %d, civics: %d, finances: %d\n"%(pPlayer.getCivilizationDescription(0),civRevIdx,civicIdx,finIdx))

        if( not bIsRevWatch ) :
            for city in PyPlayer(iPlayer).getCityList() :
                pCity = city.GetCy()
                RevUtils.changeCityVal(pCity, 'RevolutionIndex', civRevIdx )

        civString = localText.getText("TXT_KEY_REV_WATCH_CIV_WIDE",()) + '\n' + localText.getText("TXT_KEY_REV_WATCH_POSITIVE",()) + '  ' + posString.strip(', ') + '\n' + localText.getText("TXT_KEY_REV_WATCH_NEGATIVE",()) + '  ' + negString.strip(', ')

        return civString


    def checkForRevolution( self, iPlayer ) :

        pPlayer = gc.getPlayer(iPlayer)
        if( pPlayer.getNumCities() == 0 ) :
            return

        #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Checking %s for revolutions"%(pPlayer.getCivilizationDescription(0)))
        playerPy = PyPlayer( iPlayer )
        cityList = playerPy.getCityList()

        revReadyCities = list()
        revInstigatorCities = list()
        warnCities = list()

        capRevIdx = 0

        for city in cityList :
            pCity = city.GetCy()

            revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
            prevRevIdx = RevUtils.getCityVal(pCity, 'PrevRevIndex')
            localRevIdx = RevUtils.getCityVal(pCity, 'LocalRevIndex')

            numUnhappy = RevUtils.getModNumUnhappy( pCity, self.warWearinessMod )
            if( numUnhappy > 0 ) :
                cityThreshold = max([int( self.revInstigatorThreshold - 2*self.revInstigatorThreshold*numUnhappy/pCity.getPopulation() ),self.revInstigatorThreshold/10])
            else :
                cityThreshold = self.revInstigatorThreshold

            if( pCity.isCapital() ) :
                capRevIdx = revIdx

            if( revIdx >= int( self.warnFrac*cityThreshold ) and RevUtils.getCityVal(pCity, 'RevolutionCounter') == 0 ) :
                if(  RevUtils.getCityVal(pCity, 'WarningCounter') == 0 ) :
                    # Warn human of impending revolution (note can't instigate on warning turn)
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  REVOLT - %s (%s) is over %d warning threshold in year %d!!!!"%(pCity.getName(),pPlayer.getCivilizationDescription(0),int( self.warnFrac*cityThreshold ),game.getGameTurnYear()))
                    warnCities.append(pCity)
                elif( revIdx > cityThreshold and prevRevIdx > cityThreshold ) :
                    # City meets instigator criteria
                    revInstigatorCities.append(pCity)
                
            if( revIdx > int(self.revReadyFrac*cityThreshold) and prevRevIdx > int(self.revReadyFrac*cityThreshold) and RevUtils.getCityVal(pCity, 'RevolutionCounter') == 0 ) :
                # City meets revolution ready criteria
                revReadyCities.append( pCity )

            RevUtils.updateCityVal( pCity, 'PrevRevIndex', revIdx )

        instigator = None

        if( len(revInstigatorCities) > 0 ) :
            for pCity in revInstigatorCities :
                revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                odds = revIdx

                if( RevUtils.getCityVal(pCity, 'LocalRevIndex') < -10 ) :
                    odds = odds/4
                elif( RevUtils.getCityVal(pCity, 'LocalRevIndex') < 0 ) :
                    odds = odds/2

                if( RevUtils.getCityVal(pCity, 'LocalRevIndex') > self.badLocalThreshold ) :
                    odds = 2*odds

                if( RevUtils.getCityVal(pCity, 'LocalRevIndex') > max([int(2.5*self.badLocalThreshold),10]) ) :
                    odds = 3*odds

                if( RevUtils.getModNumUnhappy( pCity, self.warWearinessMod, silent = True ) > 0 ) :
                    odds = max([ int(1.5*self.revInstigatorThreshold), odds, 2*revIdx ])

                odds = int( self.chanceModifier*odds )

                # Do revolution?
                if( game.getSorenRandNum( self.iRevRoll, 'Revolt - do revolution?' ) < odds ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  REVOLT - %s (%s) has decided to launch a revolution with index %d and odds %f in year %d!!!!"%(pCity.getName(),pPlayer.getCivilizationDescription(0),revIdx,odds/(1.0*self.iRevRoll),game.getGameTurnYear()))
                    instigator = pCity
                    break

        if( not instigator == None ) :
            self.pickRevolutionStyle( pPlayer, instigator, revReadyCities )
        else :
            for pCity in warnCities :
                RevUtils.updateCityVal( pCity, 'WarningCounter', self.warnTurns )
                
            if( self.showHumanPopup(pPlayer.getID()) and len(warnCities) > 0 ) :
                pTeam = gc.getTeam( pPlayer.getTeam() )
                # Additions by Caesium et al
                caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                caesiumtextResolution = caesiumtR.split('x')
                caesiumpasx = int(caesiumtextResolution[0])/10
                caesiumpasy = int(caesiumtextResolution[1])/10
                popup = PyPopup.PyPopup(bDynamic=False)
                if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                # End additions by Caesium et al
                popup.setHeaderString( localText.getText("TXT_KEY_REV_WARN_TITLE",()) )
                bodStr = localText.getText("TXT_KEY_REV_WARN_NEWS",()) + ' '
                if( len(warnCities) == 1 ) :
                    bodStr += '%s '%(warnCities[0].getName())
                else :
                    for pCity in warnCities[:len(warnCities)-1] :
                        bodStr += '%s, '%(pCity.getName())
                    bodStr += localText.getText("TXT_KEY_REV_AND",()) + ' %s '%(warnCities[-1].getName())
                bodStr += ' ' + localText.getText("TXT_KEY_REV_WARN_CONTEMPLATING",())                
                cityStrings = self.updateLocalRevIndices(pPlayer.getID(), subCityList = warnCities, bIsRevWatch = True )
                bodStr += cityStrings
                bodStr += '\n\n' + localText.getText("TXT_KEY_REV_WARN_CIV_WIDE",()) + '\n'
                civString = self.updateCivRevIndices( pPlayer.getID(), bIsRevWatch = True )
                bodStr += civString
                if( pTeam.getAtWarCount(True) > 0 ) :
                    bodStr += '\n\n' + localText.getText("TXT_KEY_REV_WARN_WARS",())
                else :
                    bodStr += '\n\n' + localText.getText("TXT_KEY_REV_WARN_GLORY",())
                    if( pPlayer.getCitiesLost() > 0 ) :
                        bodStr += "  " + localText.getText("TXT_KEY_REV_WARN_LOST",())
                    bodStr += '  ' + localText.getText("TXT_KEY_REV_WARN_TEMPORARY",())
                popup.setBodyString( bodStr )

                # Center camera on city
                CyCamera().JustLookAt( warnCities[0].plot().getPoint() )
                
                popup.launch()


############################  Game turn revolution functions  ###################################        

    def topCivAdjustments( self ) :
        # Penalty on top score/power to help keep game even
        # Benefit for highest culture

        maxCultPlayer = -1
        maxCult = 0
        maxPowerPlayer = -1
        maxPower = 0
        secPower = 0

        for iPlayer in range(0,gc.getMAX_CIV_PLAYERS()) :
            pPlayer = gc.getPlayer( iPlayer )
            if( pPlayer.isAlive() and not pPlayer.getNumCities() == 0 ) :            
                if( pPlayer.getPower() > maxPower ) :
                    maxPower = pPlayer.getPower()
                    maxPowerPlayer = iPlayer
                elif( pPlayer.getPower() > secPower ) :
                    secPower = pPlayer.getPower()
                if( pPlayer.countTotalCulture() > maxCult ) :
                    maxCult = pPlayer.countTotalCulture()
                    maxCultPlayer = iPlayer

        # Power - Increase revolution chance for top player
        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s have most power"%(gc.getPlayer(maxPowerPlayer).getCivilizationDescription(0)))
        for city in PyPlayer(maxPowerPlayer).getCityList() :
            pCity = city.GetCy()
            if( 1.0*maxPower > 1.5*secPower ) :
                powerEffect = 5
            elif( 1.0*maxPower > 1.2*secPower ) :
                powerEffect = 4
            else :
                powerEffect = 2
            RevUtils.changeCityVal(pCity, 'RevolutionIndex', powerEffect )

        # Culture - Cut the dominant culture player some slack
        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s have most culture"%(gc.getPlayer(maxCultPlayer).getCivilizationDescription(0)))
        for city in PyPlayer(maxCultPlayer).getCityList() :
            pCity = city.GetCy()
            RevUtils.changeCityVal(pCity, 'RevolutionIndex', -2 )

        # Score - Increase revolution chance for top player (don't pile on too much with top power)
        maxScorePlayer = game.getRankPlayer( 0 )
        secScorePlayer = game.getRankPlayer( 1 )
        maxScore = game.getPlayerScore( maxScorePlayer )
        if( secScorePlayer > 0 ) :
            secScore = game.getPlayerScore( secScorePlayer )
        else :
            secScore = 1
        if( maxScorePlayer == maxPowerPlayer ) :
            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s also have highest score"%(gc.getPlayer(maxScorePlayer).getCivilizationDescription(0)))
            for city in PyPlayer(maxScorePlayer).getCityList() :
                pCity = city.GetCy()
                if( maxScore > 1.5*secScore ) :
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', 3 )
                elif( maxScore > 1.2*secScore ) :
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', 2 )
                else :
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', 1 )
        else :
            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s have highest score"%(gc.getPlayer(maxScorePlayer).getCivilizationDescription(0)))
            for city in PyPlayer(maxScorePlayer).getCityList() :
                pCity = city.GetCy()
                if( maxScore > 1.5*secScore ) :
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', 6 )
                elif( maxScore > 1.2*secScore ) :
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', 4 )
                else :
                    RevUtils.changeCityVal(pCity, 'RevolutionIndex', 2 )
                

    def checkForAssimilation( self ) :

        iWorldCities = game.getNumCivCities()
        iPlayersAlive = game.countCivPlayersAlive()

        iWorldPlots = gc.getMap().getLandPlots()

        avgNumCities = iWorldCities/iPlayersAlive
        avgNumPlots = int(iWorldPlots/(1.0*iPlayersAlive) +.5)
        minNumPlots = min([avgNumPlots/3+1, 15])
        minNumPlots = max([minNumPlots,9])

        maxEra = 0
        for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
            iEra = gc.getPlayer( idx ).getCurrentEra()
            if( iEra > maxEra ) :
                maxEra = iEra

        for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
            playerI = gc.getPlayer( idx )

            if( not playerI.isAlive() or playerI.isBarbarian() ) :
                continue
            
            if( playerI.isHuman() ) :
                continue

            capital = playerI.getCapitalCity()

            if( game.getGameTurn() > 100 ) :
                if( (game.getGameTurn() - capital.getGameTurnAcquired()) > 15 and playerI.getNumCities() > 0 and playerI.getNumCities() < 3 ) :
                    if( playerI.getTotalLand() < minNumPlots or playerI.getTotalPopulation() < 5 ) :
                        #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is very small"%(playerI.getCivilizationDescription(0)))
                        
                        teamI = gc.getTeam( playerI.getTeam() )

                        if( capital.area().getNumCities() < playerI.getNumCities() + 2 ) :
                            # On own island
                            continue

                        if( teamI.getNumMembers() > 1 ) :
                            # In confederation or alliance
                            continue                 

                        iOdds = 0
                        assimPlayer = None
                        
                        if( capital.getPopulation() < 4 ) :
                            iOdds += 2 + maxEra/2
                        if(  playerI.getTotalLand() < minNumPlots/2 ) :
                            iOdds += 2

                        ### Special cases
                        if( teamI.isAVassal() ) :
                            ### Is a vassal
                            if( RevUtils.getCityVal(capital, 'RevolutionIndex') > self.alwaysViolentThreshold ) :
                                iOdds += 5
                                
                            iOdds = max([iOdds,4])

                            if( game.getSorenRandNum( 100, 'Revolution: Assimilate' ) < iOdds ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation!  Vassal %s considering assimilation to master"%(playerI.getCivilizationDescription(0)))                   

                                # If player is a Vassal, should only be allowed to assimilate with master
                                masterPlayer = None
                                for teamIdx in range(0,gc.getMAX_CIV_TEAMS()) :
                                    if( teamI.isVassal(teamIdx) ) :
                                        masterPlayer = gc.getPlayer( gc.getTeam(teamIdx).getLeaderID() )
                                        break

                                if( not masterPlayer == None ) :
                                    relations = playerI.AI_getAttitude( masterPlayer.getID() )
                                    if( capital.plot().getCulture(masterPlayer.getID())/(1.0*capital.plot().countTotalCulture()) > .3 ) :
                                        # Assimilate with master with large culture in city
                                        if( not relations == AttitudeTypes.ATTITUDE_FURIOUS ) :
                                            if( masterPlayer.isHuman() ) :
                                                if( not idx in self.noAssimilateList ) :
                                                    assimPlayer = masterPlayer
                                            else :
                                                assimPlayer = masterPlayer
                                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation to master based on culture")
                                    elif( relations == AttitudeTypes.ATTITUDE_PLEASED or relations == AttitudeTypes.ATTITUDE_FRIENDLY ) :
                                        # Assimilate with friendly, powerful master
                                        masterPower = masterPlayer.getPower()
                                        vassalPower = playerI.getPower()

                                        if( masterPower > 4*vassalPower and masterPlayer.getCurrentEra() > playerI.getCurrentEra() ) :
                                            if( masterPlayer.isHuman() ) :
                                                if( not idx in self.noAssimilateList ) :
                                                    assimPlayer = masterPlayer
                                            else :
                                                assimPlayer = masterPlayer
                                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation to friendly and powerful master")


                        elif( capital.plot().calculateCulturePercent(idx) < 90 ) :
                            ### Capital has foreign influence
                            iCultPlayer = capital.plot().calculateCulturalOwner()       # iCultPlayer guaranteed to be alive
                            if( not iCultPlayer == playerI.getID() ) :
                                iOdds += 3

                            if( RevUtils.getCityVal(capital, 'RevolutionIndex') > self.alwaysViolentThreshold ) :
                                iOdds += 10
                            elif( RevUtils.getCityVal(capital, 'RevolutionIndex') > self.revInstigatorThreshold ) :
                                iOdds += 3

                            if( game.getSorenRandNum( 100, 'Revolution: Assimilate' ) < iOdds ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation!  %s considering assimilation by culture"%(playerI.getCivilizationDescription(0)))                   

                                if( iCultPlayer > 0 and not iCultPlayer == idx and not playerI.AI_getAttitude(iCultPlayer) == AttitudeTypes.ATTITUDE_FURIOUS ) :
                                    ## Assimilate with cultural owner
                                    cultPlayer = gc.getPlayer(iCultPlayer)
                                    if( cultPlayer.isHuman() ) :
                                        if( not idx in self.noAssimilateList ) :
                                            assimPlayer = cultPlayer
                                    else :
                                        assimPlayer = cultPlayer
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation to culture owner")

                                if( assimPlayer == None ) :
                                    ## Check for good relations with second place culture
                                    maxSecondCult = 0
                                    for j in range(0,gc.getMAX_CIV_PLAYERS()) :
                                        if( not j == idx ) :
                                            if( capital.plot().getCulture(j) > maxSecondCult ) :
                                                secCultPlayer = gc.getPlayer(j)
                                                maxSecondCult = capital.plot().getCulture(j)

                                    if( maxSecondCult/(1.0*capital.plot().countTotalCulture()) > .2 ) :
                                        relations = playerI.AI_getAttitude( secCultPlayer.getID() )
                                        if( relations == AttitudeTypes.ATTITUDE_PLEASED or relations == AttitudeTypes.ATTITUDE_FRIENDLY ) :
                                            if( secCultPlayer.isHuman() ) :
                                                if( not idx in self.noAssimilateList ) :
                                                    assimPlayer = secCultPlayer
                                            else :
                                                assimPlayer = secCultPlayer
                                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation to friendly, 2nd culture player")
                                        else :
                                            if( maxSecondCult/(1.0*capital.plot().countTotalCulture()) > .35 ) :
                                                if( relations == AttitudeTypes.ATTITUDE_CAUTIOUS ) :
                                                    if( secCultPlayer.isHuman() ) :
                                                        if( not idx in self.noAssimilateList ) :
                                                            assimPlayer = secCultPlayer
                                                    else :
                                                        assimPlayer = secCultPlayer
                                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation to high 2nd culture player")


                        elif( playerI.getCurrentEra() + 2 < maxEra ) :
                            ### Far, far behind technologically
                            if( playerI.getCurrentEra() + 3 < maxEra ) :
                                iOdds += 5
                                
                            if( RevUtils.getCityVal(capital, 'RevolutionIndex') > self.alwaysViolentThreshold ) :
                                iOdds += 5
                            elif( RevUtils.getCityVal(capital, 'RevolutionIndex') > self.revInstigatorThreshold ) :
                                iOdds += 2

                            if( game.getSorenRandNum( 100, 'Revolution: Assimilate' ) < iOdds ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation!  %s considering assimilation due to tech"%(playerI.getCivilizationDescription(0)))                   

                                ## Check for good relations with second highest culture
                                maxSecondCult = 0
                                for j in range(0,gc.getMAX_CIV_PLAYERS()) :
                                    if( not j == idx ) :
                                        if( capital.plot().getCulture(j) > maxSecondCult ) :
                                            secCultPlayer = gc.getPlayer(j)
                                            maxSecondCult = capital.plot().getCulture(j)

                                if( maxSecondCult > 0 and secCultPlayer.isAlive() ) :
                                    if( secCultPlayer.getCurrentEra() > playerI.getCurrentEra() ) :
                                        relations = playerI.AI_getAttitude( secCultPlayer.getID() )
                                        if( relations == AttitudeTypes.ATTITUDE_PLEASED or relations == AttitudeTypes.ATTITUDE_FRIENDLY ) :
                                            if( secCultPlayer.isHuman() ) :
                                                if( not idx in self.noAssimilateList ) :
                                                    assimPlayer = secCultPlayer
                                            else :
                                                assimPlayer = secCultPlayer
                                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation to friendly, 2nd culture player")

            
                        else :
                            continue
                                    

                        if( not assimPlayer == None ) :
                            # Assimilate!
                            if( assimPlayer.isHuman() ) :
                                # Additions by Caesium et al
                                caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                                caesiumtextResolution = caesiumtR.split('x')
                                caesiumpasx = int(caesiumtextResolution[0])/10
                                caesiumpasy = int(caesiumtextResolution[1])/10
                                popup = PyPopup.PyPopup(self.assimilationPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                                if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                                # Additions by Caesium et al
                                #popup.setHeaderString( 'Assimilation' )
                                bodStr = 'The %s would like to give up their independence and join your empire!'%(playerI.getCivilizationDescription(0))
                                bodStr += '  They are simply to small to survive on their own, and hope that you will accept their request.'
                                bodStr += "\n\nIf you accept this request, you will receive all of the %s's cities and they will become a minority group in your empire."
                                popup.setBodyString( bodStr )
                                popup.addSeparator()
                                popup.addButton( 'Accept' )
                                popup.addButton( 'Reject' )
                                popup.setUserData( (playerI.getID(),) )
                                popup.launch(bCreateOkButton = False)

                            else :
                                self.doAssimilation( playerI.getID(), assimPlayer.getID() )
                                        

    def doAssimilation( self, iSmallPlayer, iBigPlayer ) :

        playerI = gc.getPlayer( iSmallPlayer )
        assimPlayer = gc.getPlayer( iBigPlayer )

        assimCapital = assimPlayer.getCapitalCity()
        if( assimCapital == None ) :
            return

        teamI = gc.getTeam( playerI.getTeam() )

        # Declare peace
        if( teamI.isAtWar( assimPlayer.getTeam() ) ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error: Assimilating with warring player ...")
            teamI.makePeace( assimPlayer.getTeam() )

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilating with %s"%(assimPlayer.getCivilizationDescription(0)))

        if( playerI.canContact(game.getActivePlayer()) ) :
            mess = localText.getText("TXT_KEY_REV_BIG_THE",()) + ' ' + playerI.getCivilizationDescription(0) + ' ' + localText.getText("TXT_KEY_REV_MESS_ASSIMILATE",()) + ' ' + assimPlayer.getCivilizationDescription(0) + '.'
            CyInterface().addImmediateMessage(mess,"")

        playerIPy = PyPlayer(playerI.getID())

        # Would like to be able to cancel this
        if( not teamI.isOpenBorders( assimPlayer.getTeam() ) ) :
            teamI.signOpenBorders( assimPlayer.getTeam() )

        unitList = list()
        for unit in playerIPy.getUnitList() :
            if( unit.canFight() ) :
                unitList.append( [unit, unit.getX(), unit.getY()] )
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - To convert: %s at %d, %d"%(unit.getName(),unit.getX(), unit.getY()))

        for city in playerIPy.getCityList() :
            assimPlayer.acquireCity( city.GetCy(), False, True )

        for sublist in unitList :
            unit = sublist[0]
            newUnit = assimPlayer.initUnit( unit.getUnitType(), sublist[1], sublist[2], UnitAITypes.NO_UNITAI )
            newUnit.convert( unit )
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assim unit %s at %d, %d"%(newUnit.getName(),newUnit.getX(),newUnit.getY()))

        # Must do this in case complete kills is on
        playerI.killUnits()
                

    def assimilateHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( popupReturn.getButtonClicked() == 0 ):
            self.doAssimilation( netUserData[0], iPlayerID )
        else :
            self.noAssimilateList.append( netUserData[0] )


##    def checkReturnHumanLeader( self ) :
##
##        if( not self.humanLeaderType == None ) :
##            if( game.getAIAutoPlay() == 1 ) :
##                humanPlayer = gc.getActivePlayer()
##                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Returning human chosen leader to control of %s"%(humanPlayer.getCivilizationDescription(0)))
##                RevUtils.changeCiv(humanPlayer.getID(),humanPlayer.getCivilizationType(),self.humanLeaderType)
##                self.humanLeaderType = None

                

###################################################################################

    def pickRevolutionStyle( self, pPlayer, instigator, revReadyCities ) :

        iPlayer = pPlayer.getID()
        pTeam = gc.getTeam( pPlayer.getTeam() )
        
        revCities = list()
        revCities.append(instigator)
        revInCapital = instigator.isCapital()
        
        # Who will join them?  City must be either in area with instigator, or close to instigator but not in homeland
        # City must also be able to revolt now (not recently revolted)
        for pCity in revReadyCities :
            if( RevUtils.getCityVal(pCity, 'RevolutionCounter') == 0 and not pCity.getID() == instigator.getID() ) :
                
                if( pCity.area().getID() == instigator.area().getID() ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is in the area, joining revolution"%(pCity.getName()))
                    revCities.append(pCity)
                    revInCapital = (revInCapital or pCity.isCapital())
                    
                elif( plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() ) <= self.closeRadius and not pCity.area().getID() == pPlayer.getCapitalCity().area().getID() ) :
                    # Catch cities on small island chains ... not in same area, but close and not in homeland
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is nearby, joining revolution"%(pCity.getName()))
                    revCities.append(pCity)
                    revInCapital = (revInCapital or pCity.isCapital())

        # Peaceful or violent?
        peaceful = True
        instRevIdx = RevUtils.getCityVal( instigator, 'RevolutionIndex')
        instLocalIdx = RevUtils.getCityVal( instigator, 'LocalRevIndex' )

        if( instRevIdx > self.alwaysViolentThreshold ) :
            # Situation really bad
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent, above always violent threshold")
            peaceful = False
        elif( instigator.getNumRevolts(iPlayer) == 0 ) :
            peaceful = True
        else :
            if( peaceful ) :
                modNumUnhappy = RevUtils.getModNumUnhappy( instigator, self.warWearinessMod )
                if( int(200*modNumUnhappy/instigator.getPopulation()) > game.getSorenRandNum( 100, 'Rev' ) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent due to Unhappiness")
                    peaceful = False
            if( peaceful and instLocalIdx > self.badLocalThreshold ) :
                # Situation deteriorating rapidly
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent due to rapidly deteriorating situation")
                peaceful = False
            if( peaceful and len(revCities) == 1 ) :
                peaceful = True
            elif( peaceful ) :
                lowerThresh = int( .8*self.alwaysViolentThreshold )

                civicsMod = RevUtils.getCivicsViolentRevMod( iPlayer )
                lowerThresh += int( civicsMod*self.alwaysViolentThreshold  + .5 )
                
##                if( pPlayer.getCivics(self.iCivicOptionGovernment) == self.iPoliceStateID ) :
##                    lowerThresh -= int( .15*self.alwaysViolentThreshold )
##                elif( pPlayer.getCivics(self.iCivicOptionLegal) == self.iFreeSpeechID ) :
##                    lowerThresh += int( .10*self.alwaysViolentThreshold )
                
                if( instLocalIdx > self.badLocalThreshold ) :
                    lowerThresh -= int( .05*self.alwaysViolentThreshold + .5 )
                elif( instLocalIdx < 0 ) :
                    lowerThresh += int( .10*self.alwaysViolentThreshold + .5 )
                
                if( instRevIdx > lowerThresh ) :
                    odds = (100*(instRevIdx - lowerThresh))/(self.alwaysViolentThreshold-lowerThresh)
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds for violence are %d"%(odds))
                    if( odds > game.getSorenRandNum( 100, 'Rev' ) ) :
                        peaceful = False
            

        if( peaceful ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful revolution")

#-------- Check for still existing violent revolution for instigator
        if( not peaceful ) :
            if( game.getGameTurn() - RevUtils.getCityVal(instigator, 'RevolutionTurn') < 3*self.turnsBetweenRevs ) :
                revCivType = RevUtils.getCityVal(instigator, 'RevolutionCiv')
                pRevPlayer = None
                if( revCivType >= 0 ) :
                    for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                        if( not i == pPlayer.getID() ) :
                            playerI = gc.getPlayer( i )
                            if( playerI.isAlive() and playerI.getCivilizationType() == revCivType ) :
                                if( pTeam.isAtWar(playerI.getTeam()) ) :
                                    pRevPlayer = playerI
                                    break

                if( not pRevPlayer == None and RevUtils.revObjectExists(pRevPlayer) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Cities revolt civ type %s is fighting owner"%(pRevPlayer.getCivilizationDescription(0)))
                    revTurn = RevUtils.revObjectGetVal(pRevPlayer, 'RevolutionTurn')
                    if( not revTurn == None and game.getGameTurn()-revTurn < 3*self.turnsBetweenRevs ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Joining existing revolution with %s"%(pRevPlayer.getCivilizationDescription(0))) 

                        joinRevCities = list()
                        for pCity in revCities :
                            cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
                            if( RevUtils.getCityVal(pCity, 'RevolutionCiv') == revCivType ) :
                                joinRevCities.append(pCity)
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has same rev type"%(pCity.getName()))
                            elif( RevUtils.getCityVal(pCity,'RevolutionIndex') > self.revInstigatorThreshold and cityDist <= 2*self.closeRadius ) :
                                joinRevCities.append(pCity)
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is over threshold, joining"%(pCity.getName()))


                        bodStr = pRevPlayer.getName() + ' ' + localText.getText("TXT_KEY_REV_LEADER",()) + ' ' + pRevPlayer.getCivilizationDescription(0)
                        bodStr += localText.getText("TXT_KEY_REV_JOINREV_OFFER",())

                        bodStr += ' ' + joinRevCities[0].getName()
                        if( len(joinRevCities) > 1 ) :
                            bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                            for pCity in joinRevCities[1:len(joinRevCities)-1] :
                                bodStr += ' %s,'%(pCity.getName())
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + joinRevCities[-1].getName()

                        bodStr += ' ' + localText.getText("TXT_KEY_REV_JOINREV_PEACE",())

                        # Determine strength of rebellion
                        isJoinWar = False
                        revArea = instigator.area()
                        revPower = revArea.getPower(pRevPlayer.getID())
                        pPower = revArea.getPower(pPlayer.getID())
                        if( revPower > 0 ) :
                            powerFrac = pPower/(1.0*revPower)
                        else :
                            powerFrac = 10.0

                        if( powerFrac < 1.5 ) :
                            # Rebels rival homeland power
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Rebels rival homeland power, limiting enlistment")
                            isJoinWar = True

                        self.makeRevolutionDecision( joinRevCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer, isJoinWar = isJoinWar )
                
                        return

        # All of these have violent and peaceful paths
#-------- Check if instigator influence by other culture -> try to join
        if( self.culturalRevolution and instigator.plot().calculateCulturePercent(iPlayer) <= self.maxNationalityThreshold ) :
            #cultOwnerID = instigator.plot().calculateCulturalOwner()
            # calculateCulturalOwner rules out dead civs ...
            maxCulture = 30
            cultOwnerID = -1
            for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( instigator.plot().getCulture( idx ) > maxCulture ) :
                    maxCulture = instigator.plot().getCulture( idx )
                    cultOwnerID = idx
                
            if( cultOwnerID >= 0 and cultOwnerID < gc.getBARBARIAN_PLAYER() and not gc.getPlayer(cultOwnerID).getTeam() == pPlayer.getTeam() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has majority culture from other player %d, asking to join"%(instigator.getName(),cultOwnerID))
                cultCities = list()
                for pCity in revCities :
                    maxCulture = 30
                    cityCultOwnerID = -1
                    for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                        if( pCity.plot().getCulture( idx ) > maxCulture ) :
                            maxCulture = pCity.plot().getCulture( idx )
                            cityCultOwnerID = idx
                            
                    if( cityCultOwnerID == cultOwnerID ) :
                        cultCities.append(pCity)
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s also wants to go to other civ"%(pCity.getName()))

                cultPlayer = gc.getPlayer( cultOwnerID )
                cultTeam = gc.getTeam( cultPlayer.getTeam() )

                bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + cultCities[0].getName() + ' '
                if( len(cultCities) > 1 ) :
                    bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                    for pCity in cultCities[1:len(cultCities)-1] :
                        bodStr += ' %s,'%(pCity.getName())
                    bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + cultCities[-1].getName()+ ' ' + localText.getText("TXT_KEY_REV_ARE",())
                else :
                    bodStr += localText.getText("TXT_KEY_REV_IS",())

                if( peaceful ) :

                    if( cultPlayer.isAlive() ) :
                        if( pTeam.isAtWar(cultTeam.getID()) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Owner at war with city's cultural civ")
                            if( pTeam.canChangeWarPeace(cultTeam.getID()) ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Ask for end to hostilities")

                        if( pPlayer.getCurrentEra() - cultPlayer.getCurrentEra() > 1 ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Tech divide with cultural player, ask for charity")
                    
                    # ask to join other civ, if denied get angrier
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, asking to join the %s"%(cultPlayer.getCivilizationDescription(0)))

                    pRevPlayer = None
                    isJoinWar  = False
                    joinPlayer = None

                    if( cultPlayer.isAlive() ) :
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_CULT_PEACE_JOIN",()) + ' ' + cultPlayer.getCivilizationDescription(0) + '.'
                        # Will only form pRevPlayer if human join player rebuffs revolutionaries
                        if( cultPlayer.isStateReligion() ) :
                            giveRelType = cultPlayer.getStateReligion()
                        else :
                            if( 50 > game.getSorenRandNum(100,'Rev') ) :
                                giveRelType = None
                            else :
                                giveRelType = -1
                        [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( cultCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = False, giveRelType = giveRelType )
                        joinPlayer = cultPlayer

                    else :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Cult player is dead, trying to reform")
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_CULT_PEACE_REFORM",()) + ' ' + cultPlayer.getCivilizationDescription(0) + '.'
                        pRevPlayer = cultPlayer
                        

                    bodStr += '  ' + localText.getText("TXT_KEY_REV_CULT_PEACE",())
                    
                else :
                    # demand to join other civ, if denied, fight!
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent, demanding to join the %s"%(cultPlayer.getCivilizationDescription(0)))

                    if( cultPlayer.isAlive() ) :
                        joinPlayer = cultPlayer
                        if( joinPlayer.isStateReligion() ) :
                            giveRelType = joinPlayer.getStateReligion()
                        else :
                            if( 50 > game.getSorenRandNum(100,'Rev') ) :
                                giveRelType = None
                            else :
                                giveRelType = -1
                        [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( cultCities, bJoinCultureWar = True, bReincarnate = True, bJoinRebels = True, giveRelType = giveRelType )
                        if( joinPlayer.getID() == pRevPlayer.getID() ) :
                            joinPlayer = None
                        else :
                            if( self.allowSmallBarbRevs and len(cultCities) == 1 ) :
                                if( instRevIdx < int(1.2*self.revInstigatorThreshold) ) :
                                    pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                                    isJoinWar = False
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Small, disorganized Revolution")
                    else :
                        pRevPlayer = cultPlayer
                        isJoinWar = False
                        joinPlayer = None
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Cult player is dead, trying to reform")

                    if( not joinPlayer == None ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent, demanding to join the %s"%(joinPlayer.getCivilizationDescription(0)))
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - If denied, will form/join the %s, alive: %d"%(pRevPlayer.getCivilizationDescription(0),pRevPlayer.isAlive()))
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_JOIN",()) + ' ' + joinPlayer.getCivilizationDescription(0) + '.'
                        if( isJoinWar ) :
                            bodStr += '  ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_JOIN_DENY",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + '.'
                        else :
                            if( pRevPlayer.isBarbarian() ) :
                                bodStr += '  ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_JOIN_DENY_BARB",())
                            else :
                                bodStr += '  ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_JOIN_DENY",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + '.'
                            if( gc.getTeam(pPlayer.getTeam()).canDeclareWar(joinPlayer.getTeam()) ) :
                                bodStr += '  ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_DECLARE_WAR",()) + ' ' +  joinPlayer.getCivilizationDescription(0) + '.'
                    else :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent, demanding to join the %s"%(pRevPlayer.getCivilizationDescription(0)))
                        if( pRevPlayer.isBarbarian() ) :
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_REFORM_BARB",())
                        elif( not pRevPlayer.isAlive() ) :
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_REFORM",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + '.'
                        else :
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_JOIN",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + '.'

                    bodStr += '  ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_1",())
                    if( not joinPlayer == None ) :
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_THE",()) + ' ' + joinPlayer.getCivilizationDescription(0) + ' ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_2",())
                        if( pRevPlayer.isBarbarian() ) :
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_BARB",())
                        else :
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_FIGHT",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + '.'
                    else :
                        if( pRevPlayer.isBarbarian() ) :
                            pass
                        elif( pRevPlayer.isAlive() ) :
                            bodStr += '  ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_ENEMY",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + '.'

                    bodStr += '  ' + localText.getText("TXT_KEY_REV_CULT_VIOLENT_FINAL",())

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution"%(len(cultCities)))
                assert( len(cultCities) > 0 )
                self.makeRevolutionDecision( cultCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer, isJoinWar = isJoinWar, joinPlayer = joinPlayer )
                
                return

#-------- Check for religious revolution
        stateRel = pPlayer.getStateReligion()
        if( self.religiousRevolution and pPlayer.isStateReligion() and stateRel >= 0 ) :
            revRel = None
            
            # Check holy city instigating
            if( instigator.isHolyCity() ) :
            
                if( not instigator.isHolyCityByType(stateRel) ) :

                    for relType in range(0,gc.getNumReligionInfos()) :
                        if( instigator.isHolyCityByType(relType) and not stateRel == relType ) :
                            if( self.allowStateReligionToJoin or not instigator.isHasReligion(stateRel) ) :
                                revRel = relType
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Instigator is rival religion (%s) holy city"%(gc.getReligionInfo( revRel ).getDescription()))
                                break

            # Check for significant minority religion
            if( revRel == None  ) :
                if( self.allowStateReligionToJoin or not instigator.isHasReligion(stateRel) ) :
                    if( len(revCities) >= 3 ) :
                        # Must be large movement of like minded cities
                        maxCount = 0
                        maxCountRel = -1
                        for relType in range(0,gc.getNumReligionInfos()) :
                            if( instigator.isHasReligion(relType) and not stateRel == relType ) :
                                relCount = 0
                                for pCity in revCities :
                                    if( pCity.isHasReligion(relType) ) :
                                        if( pCity.isHasReligion(stateRel) ) :
                                            relCount += .5
                                        else :
                                            relCount += 1
                                if( relCount > maxCount ) :
                                    maxCount = relCount
                                    maxCountRel = relType

                        # Is the best good enough?
                        if( maxCount >= 3 ) :
                            revRel = maxCountRel
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Instigator and enough others have %s"%(gc.getReligionInfo( revRel ).getDescription()))

            # Decide how to revolt
            if( not revRel == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Religious revolution")

                relCities = list()
                for pCity in revCities :
                    if( pCity.isHasReligion(revRel) ) :
                        if( self.allowStateReligionToJoin or not pCity.isHasReligion(stateRel) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has %s"%(pCity.getName(),gc.getReligionInfo( revRel ).getDescription()))
                            relCities.append(pCity)

                if( peaceful ) :
                    # Check if there are civics they would like changed
                    newRelCivic = None

                    [level,optionType] = RevUtils.getReligiousFreedom( iPlayer )
                    [newLevel,newRelCivic] = RevUtils.getBestReligiousFreedom( iPlayer, optionType )
                    
##                    iReligionCivic = pPlayer.getCivics(self.iCivicOptionReligion)
##                    newRelCivic = None
##                    if( iReligionCivic == self.iTheocracyID ) :
##                        
##                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change from Theocracy")
##                        if( pPlayer.canDoCivics(self.iFreeReligionID) ) :
##                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to Free Religion")
##                            newRelCivic = self.iFreeReligionID
##                        elif( pPlayer.canDoCivics(self.iPacifismID) ) :
##                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to Pacifism")
##                            newRelCivic = self.iPacifismID
##                        elif( pPlayer.canDoCivics(self.iOrganizedReligionID) ) :
##                             # Should always be able to at least switch to organized
##                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No better alternative than to ask for Organized Religion")
##                            newRelCivic = self.iOrganizedReligionID
##                        
##                    elif( not iReligionCivic == self.iFreeReligionID and pPlayer.canDoCivics(self.iFreeReligionID) ) :
##                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to Free Religion")
##                        newRelCivic = self.iFreeReligionID

                    if( not newRelCivic == None and newLevel > level ) :
                        if( level < -5 or (newLevel > 5 and level < 0) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking for change from %s to %s"%(gc.getCivicInfo(pPlayer.getCivics(optionType)).getDescription(),gc.getCivicInfo(newRelCivic).getDescription()))

                            bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
                            if( len(relCities) > 1 ) :
                                bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                                for pCity in relCities[1:len(relCities)-1] :
                                    bodStr += ' %s,'%(pCity.getName())
                                bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + relCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
                            else :
                                bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())

                            # Can't pay them enough so they don't feel oppressed
                                
                            if( level < -5 ) :
                                bodStr += ' ' + localText.getText("TXT_KEY_REV_REL_THEOCRACY1",()) + gc.getCivicInfo(pPlayer.getCivics(optionType)).getDescription() + ".  " + localText.getText("TXT_KEY_REV_REL_THEOCRACY2",()) + ' ' + gc.getCivicInfo(newRelCivic).getDescription() + '.'
                            else :
                                bodStr += ' ' + localText.getText("TXT_KEY_REV_REL_FREE_REL1",()) + gc.getCivicInfo(newRelCivic).getDescription() + '.  ' + localText.getText("TXT_KEY_REV_REL_FREE_REL2",()) + ' ' + gc.getReligionInfo(stateRel).getDescription()
                                bodStr += ' ' + localText.getText("TXT_KEY_REV_REL_PRACTICE",()) + ' ' + gc.getReligionInfo(revRel).getDescription() + '.'
                            bodStr += '\n\n' + localText.getText("TXT_KEY_REV_HONORING",())
                            
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution"%(len(relCities)))
                            assert( len(relCities) > 0 )
                            
                            self.makeRevolutionDecision( relCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newRelCivic )
                            
                            return

                    # Ask for switch of state religions?
                    revRelCount = pPlayer.getHasReligionCount(revRel)
                    stateRelCount = pPlayer.getHasReligionCount(stateRel)
                    if( revRelCount > stateRelCount/4 ) :
                        
                        bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
                        if( len(relCities) > 1 ) :
                            bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                            for pCity in relCities[1:len(relCities)-1] :
                                bodStr += ' %s,'%(pCity.getName())
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + relCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
                        else :
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())
                            
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change in state religion, %d practice new, %d practice state"%(revRelCount,stateRelCount))
                        totalRevIdx = 0
                        for pCity in relCities :
                            totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                        buyOffCost = totalRevIdx/(17-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        #buyOffCost = (60 + 12*pPlayer.getCurrentEra())*len(relCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_REL_CHANGE",()) + ' ' + gc.getReligionInfo( revRel ).getDescription() + '.'
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_CURRENTLY",()) + ' %d '%(revRelCount) + localText.getText("TXT_KEY_REV_REL_NEW_REL",()) + ' %d '%(stateRelCount) + localText.getText("TXT_KEY_REV_REL_STATE_REL",())
                        bodStr += '\n\n' + localText.getText("TXT_KEY_REV_REL_HONORING",())
                        
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(relCities),buyOffCost))
                        assert( len(relCities) > 0 )
                        self.makeRevolutionDecision( relCities, pPlayer, 'religion', peaceful, bodStr, newReligion = revRel, buyOffCost = buyOffCost )

                        return

                    # Ask for independence
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking for independence for religious reasons")

                    # Prune for only close cities?  Cities in area may be quite far away
                    indCities = list()
                    for pCity in relCities :
                        cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
                        if( cityDist <= 2*self.closeRadius ) :
                            for iCity in indCities :
                                cityDist = min([cityDist, plotDistance( pCity.getX(), pCity.getY(), iCity.getX(), iCity.getY() )])

                            if( cityDist <= self.closeRadius ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is close enough to join in independence quest"%(pCity.getName()))
                                indCities.append(pCity)   

                    bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
                    if( len(indCities) > 1 ) :
                        bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                        for pCity in indCities[1:len(indCities)-1] :
                            bodStr += ' %s,'%(pCity.getName())
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + indCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
                    else :
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())

                    #buyOffCost = (60 + 12*pPlayer.getCurrentEra())*len(indCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                    totalRevIdx = 0
                    for pCity in indCities :
                        totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                    buyOffCost = totalRevIdx/(17-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                    if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                    buyOffCost = max( [buyOffCost,pPlayer.getGold()/12 + game.getSorenRandNum(50,'Rev')] )

                    [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( indCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = False, giveRelType = revRel, bMatchCivics = True )

                    vassalStyle = None
                    if( not pTeam.isAVassal() and pTeam.isVassalStateTrading() ) :
                        if( totalRevIdx/len(indCities) < self.revInstigatorThreshold ) :
                            if( pRevPlayer.AI_getAttitude(pPlayer.getID()) == AttitudeTypes.ATTITUDE_FRIENDLY or pRevPlayer.AI_getAttitude(pPlayer.getID()) == AttitudeTypes.ATTITUDE_PLEASED ) :
                                vassalStyle = 'free'
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Vassal style %s chosen"%(vassalStyle))

                    if( not vassalStyle == None ) :
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_REL_PEACE_VASSAL_1",()) + ' ' + gc.getReligionInfo( revRel ).getDescription() + '.'
                        bodStr += '   ' + localText.getText("TXT_KEY_REV_REL_PEACE_VASSAL_2",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + ' ' + localText.getText("TXT_KEY_REV_REL_PEACE_VASSAL_3",())
                    else :
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_REL_PEACE_IND_1",()) + ' ' + gc.getReligionInfo( revRel ).getDescription() + '.'
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_REL_PEACE_IND_2",()) + ' ' + gc.getReligionInfo(pPlayer.getStateReligion()).getDescription() + ' ' + localText.getText("TXT_KEY_REV_REL_PEACE_IND_3",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + '.'
                        bodStr += '\n\n' + localText.getText("TXT_KEY_REV_REL_PEACE",())

                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(indCities),buyOffCost))
                    assert( len(indCities) > 0 )
                    self.makeRevolutionDecision( indCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer = pRevPlayer, buyOffCost = buyOffCost, vassalStyle = vassalStyle )

                    return
                    
                else :
                    # Demand independence for religious reasons
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Demanding independence for religious reasons")

                    # Prune for only close cities?  Cities in area may be quite far away
                    indCities = list()
                    for pCity in relCities :
                        cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
                        if( cityDist <= 2*self.closeRadius ) :
                            for iCity in indCities :
                                cityDist = min([cityDist, plotDistance( pCity.getX(), pCity.getY(), iCity.getX(), iCity.getY() )])

                            if( cityDist <= self.closeRadius ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is close enough to join in independence quest"%(pCity.getName()))
                                indCities.append(pCity)   

                    bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
                    if( len(indCities) > 1 ) :
                        bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                        for pCity in indCities[1:len(indCities)-1] :
                            bodStr += ' %s,'%(pCity.getName())
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + indCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
                    else :
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())

                    bodStr += ' ' + localText.getText("TXT_KEY_REV_REL_VIOLENT_IND_1",()) + ' %s.'%(gc.getReligionInfo( revRel ).getDescription())
                    bodStr += '  ' + localText.getText("TXT_KEY_REV_REL_VIOLENT_IND_2",()) + ' %s '%(gc.getReligionInfo( pPlayer.getStateReligion() ).getDescription()) + localText.getText("TXT_KEY_REV_REL_VIOLENT_IND_3",())
                    bodStr += '\n\n' + localText.getText("TXT_KEY_REV_REL_VIOLENT_IND",())

                    [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( indCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = False, giveRelType = revRel )

                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution"%(len(indCities)))
                    assert( len(indCities) > 0 )
                    self.makeRevolutionDecision( indCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer = pRevPlayer)

                    return

# --------------- Special options for homeland revolutions
        if( instigator.area().getID() == pPlayer.getCapitalCity().area().getID() ) :
            # Revolution in homeland
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revolution in homeland")

            if( peaceful and not gc.getTeam(pPlayer.getTeam()).isHasTech(self.iNationalismTech) ) :
                [goodEffect,badEffect] = RevUtils.getCivicsHolyCityEffects( iPlayer )
                if( badEffect > 0 ) :
                    stateRel = pPlayer.getStateReligion()
                    if( pPlayer.isStateReligion() and stateRel >= 0 ) :
                        # Check for ask for crusade
                        stateHolyCity = game.getHolyCity( stateRel )                        
                        stateHolyCityOwner = gc.getPlayer( stateHolyCity.getOwner() )
                        if( instigator.isHasReligion(stateRel) and not stateHolyCityOwner.getID() == iPlayer ) :
                            if( pTeam.canDeclareWar( stateHolyCityOwner.getTeam() ) and not pTeam.isAVassal() ) :

                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Holy city for %s (%d) is %s, owner %s practices %d"%(gc.getReligionInfo(stateRel).getDescription(),stateRel,stateHolyCity.getName(),stateHolyCityOwner.getCivilizationDescription(0),stateHolyCityOwner.getStateReligion()))

                                relCities = list()
                                for city in revCities :
                                    if( city.isHasReligion(stateRel) ) :
                                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has state religion"%(city.getName()))
                                        relCities.append(city)

                                bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
                                if( len(revCities) > 1 ) :
                                    bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                                    for pCity in relCities[1:len(relCities)-1] :
                                        bodStr += ' %s,'%(pCity.getName())
                                    bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + relCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
                                else :
                                    bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())
                                        
                                if( not stateHolyCityOwner.getStateReligion() == stateRel ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Ask for Crusade against %s!"%(stateHolyCityOwner.getCivilizationDescription(0)))

                                    bodStr += " " + localText.getText("TXT_KEY_REV_HL_HOLY_WAR",()) + " %s."%(stateHolyCityOwner.getCivilizationDescription(0))
                                    bodStr += "   " + localText.getText("TXT_KEY_REV_HL_HOLY_RECLAIM",()) + " %s, "%(stateHolyCity.getName()) + localText.getText("TXT_KEY_REV_HL_HOLY_HEATHENS",())
                                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_HL_HOLY_REQUEST",())

                                    assert( len(relCities) > 0 )
                                    self.makeRevolutionDecision( relCities, pPlayer, 'war', peaceful, bodStr, pRevPlayer = stateHolyCityOwner )

                                    return

                                if( pPlayer.AI_getAttitude(stateHolyCityOwner.getID()) == AttitudeTypes.ATTITUDE_FURIOUS or pPlayer.AI_getAttitude(stateHolyCityOwner.getID()) == AttitudeTypes.ATTITUDE_ANNOYED ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Ask for crusade against fellow believer, %s!"%(stateHolyCityOwner.getCivilizationDescription(0)))

                                    bodStr += " " + localText.getText("TXT_KEY_REV_HL_HOLY_WAR",()) + " %s."%(stateHolyCityOwner.getCivilizationDescription(0))
                                    bodStr += "   " + localText.getText("TXT_KEY_REV_HL_HOLY_WHILE",()) + ' %s '%(stateHolyCityOwner.getCivilizationDescription(0)) + localText.getText("TXT_KEY_REV_HL_HOLY_CLAIMS",()) + ' %s, '%(gc.getReligionInfo(stateRel).getDescription()) + localText.getText("TXT_KEY_REV_HL_HOLY_WORTHY",()) + " %s!"%(stateHolyCity.getName())
                                    bodStr += "   " + localText.getText("TXT_KEY_REV_HL_HOLY_DEVOTION",()) + " %s "%(stateHolyCity.getName()) + localText.getText("TXT_KEY_REV_HL_HOLY_UNWORTHY",())
                                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_HL_HOLY_REQUEST",())
                                    
                                    assert( len(relCities) > 0 )
                                    self.makeRevolutionDecision( relCities, pPlayer, 'war', peaceful, bodStr, pRevPlayer = stateHolyCityOwner )

                                    return


            if( self.civicRevolution ) :
                [laborLevel,optionType] = RevUtils.getLaborFreedom( iPlayer )
                [newLaborLevel,newCivic] = RevUtils.getBestLaborFreedom( iPlayer, optionType )
                if( peaceful and laborLevel < 0 and newLaborLevel > 5 and not newCivic == None ) :
                    if( (10*abs(laborLevel) > game.getSorenRandNum(100, 'Revolt - emancipation request')) ):
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to emancipation, %d"%(newCivic))

                        bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
                        if( len(revCities) > 1 ) :
                            bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                            for pCity in revCities[1:len(revCities)-1] :
                                bodStr += ' %s,'%(pCity.getName())
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + revCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
                        else :
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())

                        #buyOffCost = (50 + 10*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in revCities :
                            totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                        buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_HL_EMAN_REJECT",()) + ' %s '%(gc.getCivicInfo(newCivic).getDescription()) + localText.getText("TXT_KEY_REV_HL_EMAN_CIVIC",())
                        bodStr += '\n\n' + localText.getText("TXT_KEY_REV_PEACEFUL_CONCLUSION",())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_HL_EMAN_SLAVE",())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_BRIBE",())

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                        assert( len(revCities) > 0 )
                        self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                        return

                if( laborLevel < -5 and newLaborLevel > (laborLevel + 2) and not newCivic == None ) :
                    if( not peaceful and 50 > game.getSorenRandNum( 100, 'Revolt - do slave rebellion' ) ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Slave rebellion!!!, %d"%(newCivic))

                        slaveCities = list()
                        for pCity in revCities :
                            cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
                            if( cityDist <= 2*self.closeRadius ) :
                                for iCity in slaveCities :
                                    cityDist = min([cityDist, plotDistance( pCity.getX(), pCity.getY(), iCity.getX(), iCity.getY() )])

                                if( cityDist <= self.closeRadius ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is close enough to join in slave rebellion"%(pCity.getName()))
                                    slaveCities.append(pCity)  

                        bodStr = localText.getText("TXT_KEY_REV_HL_SLAVE_REBELLION",())
                        if( len(slaveCities) == 1 ) :
                            bodStr += ' %s!'%(slaveCities[0].getName())
                        else :
                            for pCity in slaveCities[:len(slaveCities)-1] :
                                bodStr += ' %s, '%(pCity.getName())
                            bodStr += localText.getText("TXT_KEY_REV_AND",()) + ' %s!'%(slaveCities[-1].getName())

                        bodStr += '  ' + localText.getText("TXT_KEY_REV_HL_SLAVE_DEMAND",())
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to %s"%(gc.getCivicInfo(newCivic).getDescription()))
                        bodStr += ' %s '%(gc.getCivicInfo(newCivic).getDescription())  
##                        if( pPlayer.canDoCivics(self.iEmancipationID) ) :
##                            newCivic = self.iEmancipationID
##                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to emancipation")
##                            bodStr += ' %s '%(gc.getCivicInfo(self.iEmancipationID).getDescription())                            
##
##                        elif( pPlayer.canDoCivics(self.iSerfdomID) ) :
##                            newCivic = self.iSerfdomID
##                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to serfdom")
##                            bodStr += ' %s '%(gc.getCivicInfo(self.iSerfdomID).getDescription())
##
##                        else :
##                            newCivic = self.iTribalismID
##                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to tribalism")
##                            bodStr += ' %s '%(gc.getCivicInfo(self.iTribalismID).getDescription())

                        bodStr += localText.getText("TXT_KEY_REV_HL_SLAVE_DENY",())

                        # Slaves always rise up as barbarians
                        pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                        assert(len(slaveCities) > 0)
                        self.makeRevolutionDecision( slaveCities, pPlayer, 'civics', peaceful, bodStr, pRevPlayer = pRevPlayer, newCivic = newCivic )

                        return

                [enviroLevel,optionType] = RevUtils.getEnvironmentalProtection( iPlayer )
                [newEnviroLevel,newCivic] = RevUtils.getBestEnvironmentalProtection( iPlayer, optionType )
                if( peaceful and newEnviroLevel > enviroLevel + 2 and not newCivic == None ) :
                    if( 30 > game.getSorenRandNum(100, 'Revolt - environmentalism request') ):
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to %s, %d (environment)"%(gc.getCivicInfo(newCivic).getDescription(),newCivic))

                        bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
                        if( len(revCities) > 1 ) :
                            bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                            for pCity in revCities[1:len(revCities)-1] :
                                bodStr += ' %s,'%(pCity.getName())
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + revCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
                        else :
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())

                        #buyOffCost = (50 + 10*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in revCities :
                            totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                        buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_HL_ENV_REQUEST",()) + ' %s.'%(gc.getCivicInfo(newCivic).getDescription())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_HL_ENV_GREEN",())
                        bodStr += '\n\n' + localText.getText("TXT_KEY_REV_PEACEFUL_CONCLUSION",())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_HL_ENV_SMOG",())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_BRIBE",())
                        
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                        assert( len(revCities) > 0 )
                        self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                        return


# -------------------- Special options for colonial revolutions
        else :
            # Revolution based in other area
            # These are special requests peaceful colonists may make
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revolution in colony")

            foreignCities = list()
            capitalArea = pPlayer.getCapitalCity().area().getID()
            for pCity in revCities :
                if( not pCity.area().getID() == capitalArea ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is colony"%(pCity.getName()))
                    foreignCities.append( pCity )

            bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
            if( len(foreignCities) > 1 ) :
                bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                for pCity in foreignCities[1:len(foreignCities)-1] :
                    bodStr += ' %s,'%(pCity.getName())
                bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + foreignCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
            else :
                bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())
                        
            if( peaceful and self.civicRevolution ) :
                # Sufferage or representation
                [demoLevel,optionType] = RevUtils.getDemocracyLevel( iPlayer )
                [newDemoLevel,newCivic] = RevUtils.getBestDemocracyLevel( iPlayer, optionType )
                if( demoLevel < 0 and newDemoLevel > 0 and not newCivic == None ) :
                        
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_COL_GOVT_REQUEST",())
                        if( newDemoLevel > 9 ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to universal sufferage")
                            bodStr += "  " + localText.getText("TXT_KEY_REV_COL_GOVT_PROTESTING",()) + " %s!"%(gc.getCivicInfo(newCivic).getDescription())
                        else :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to representation")
                            bodStr += "  " + localText.getText("TXT_KEY_REV_COL_GOVT_CRIES",()) + " %s!' "%(gc.getCivicInfo(newCivic).getDescription()) +localText.getText("TXT_KEY_REV_COL_GOVT_MARCH",())

                        #buyOffCost = (75 + 12*pPlayer.getCurrentEra())*len(foreignCities) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in foreignCities :
                            totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                        buyOffCost = totalRevIdx/(15-pPlayer.getCurrentEra()) + game.getSorenRandNum(80+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/8 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += '\n\n' + localText.getText("TXT_KEY_REV_PEACEFUL_CONCLUSION",())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_COL_GOVT_PRACTICES",())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_BRIBE",())
                        
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                        assert( len(revCities) > 0 )
                        self.makeRevolutionDecision( foreignCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                        return

            #if( peaceful and self.vassalRevolution ) :
                # Ask to become vassal
                # Trim down cities to only those close to instigator
            
                             
#-------- If capital or majority of cities,
        if( revInCapital or len(revCities) >= (pPlayer.getNumCities()+1)/3 ) :

            bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
            if( len(revCities) > 1 ) :
                bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                for pCity in revCities[1:len(revCities)-1] :
                    bodStr += ' %s,'%(pCity.getName())
                bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + revCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
            else :
                bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())
            
            if( peaceful ) :
                # If peaceful, ask change to civics ... if no civics, ask for change of leader
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Capital or large number of cities in peaceful revolution!")

                if( self.civicRevolution ) :
                    # Emancipation
                    [laborLevel,optionType] = RevUtils.getLaborFreedom( iPlayer )
                    [newLaborLevel,newCivic] = RevUtils.getBestLaborFreedom( iPlayer, optionType )
                    if( laborLevel < 0 and newLaborLevel > 5 and not newCivic == None ):
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to %s, %d"%(gc.getCivicInfo(newCivic).getDescription(),newCivic))

                        #buyOffCost = (50 + 12*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in revCities :
                            totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                        buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_HL_EMAN_REJECT",()) + ' %s '%(gc.getCivicInfo(newCivic).getDescription()) + localText.getText("TXT_KEY_REV_HL_EMAN_CIVIC",())
                        bodStr += '\n\n' + localText.getText("TXT_KEY_REV_PEACEFUL_CONCLUSION",())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_HL_EMAN_SLAVE",())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_BRIBE",())
                        
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                        assert( len(revCities) > 0 )
                        self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                        return

                    [isCanDoCommunism,newCivic] = RevUtils.canDoCommunism( iPlayer )
                    if( isCanDoCommunism and not newCivic == None ) :
                        if( 35 > game.getSorenRandNum(100, 'Rev - Communist revolution') ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Communist revolution, ask change to %s, %d"%(gc.getCivicInfo(newCivic).getDescription(), newCivic))
                            
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_CAP_COM_REQUEST",()) + ' %s.  '%(gc.getCivicInfo(newCivic).getDescription()) + localText.getText("TXT_KEY_REV_CAP_COM_BROTHER",())
                            bodStr += ' %s '%(gc.getCivicInfo(newCivic).getDescription()) + localText.getText("TXT_KEY_REV_CAP_COM_ECON",())

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution"%(len(revCities)))
                            assert( len(revCities) > 0 )
                            self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic )

                            return

                    # Sufferage or representation
                    [demoLevel,optionType] = RevUtils.getDemocracyLevel( iPlayer )
                    [newDemoLevel,newCivic] = RevUtils.getBestDemocracyLevel( iPlayer, optionType )
                    if( demoLevel < 0 and newDemoLevel > 0 and not newCivic == None ) :

                            bodStr += ' '
                            if( newDemoLevel > 9 ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to universal sufferage, %d"%(newCivic))
                                bodStr += "  " + localText.getText("TXT_KEY_REV_CAP_VOTE_US",()) + " %s!"%(gc.getCivicInfo(newCivic).getDescription())
                            else :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to representation, %d"%(newCivic))
                                bodStr += "  " + localText.getText("TXT_KEY_REV_CAP_VOTE_CRIES",()) + " %s!' "%(gc.getCivicInfo(newCivic).getDescription()) +localText.getText("TXT_KEY_REV_CAP_VOTE_MARCH",())

                            #buyOffCost = (50 + 15*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(150+15*pPlayer.getCurrentEra(),'Rev')
                            totalRevIdx = 0
                            for pCity in revCities :
                                totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                            buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                            if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                            buyOffCost = max( [buyOffCost,pPlayer.getGold()/8 + game.getSorenRandNum(50,'Rev')] )
                            bodStr += '\n\n' + localText.getText("TXT_KEY_REV_PEACEFUL_CONCLUSION",())
                            bodStr += '  ' + localText.getText("TXT_KEY_REV_CAP_VOTE_POWER",())
                            bodStr += "  " + localText.getText("TXT_KEY_REV_BRIBE",()) + "  " + localText.getText("TXT_KEY_REV_CAP_VOTE_STATUES",())

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                            assert( len(revCities) > 0 )
                            self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                            return

                    [bCanDoFreeSpeech,newCivic] = RevUtils.canDoFreeSpeech( iPlayer )
                    if( bCanDoFreeSpeech and not newCivic == None ) :
                        if( 50 > game.getSorenRandNum(100, 'Revolt - free speech request') ):
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to Free Speech, %d"%(newCivic))

                            bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
                            if( len(revCities) > 1 ) :
                                bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                                for pCity in revCities[1:len(revCities)-1] :
                                    bodStr += ' %s,'%(pCity.getName())
                                bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + revCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
                            else :
                                bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())

                            #buyOffCost = (50 + 10*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                            totalRevIdx = 0
                            for pCity in revCities :
                                totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                            buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+12*pPlayer.getCurrentEra(),'Rev')
                            if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                            buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_CAP_SPEECH_REQUEST",()) + ' %s.'%(gc.getCivicInfo(newCivic).getDescription())
                            bodStr += '  ' + localText.getText("TXT_KEY_REV_CAP_SPEECH_DENY",())

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                            assert( len(revCities) > 0 )
                            self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                            return

                if( self.leaderRevolution and len(revCities) > (pPlayer.getNumCities()+2)/3 and len(revCities) > 2 ) :
                    if( not pPlayer.isHuman() or self.humanLeaderRevolution ) :
                        # Ask for change of leader
   
                        bodStr += ' ' + localText.getText("TXT_KEY_REV_CAP_LEAD_REQUEST",())
                        bCanDoElection = RevUtils.canDoElection( iPlayer )
                        if( not bCanDoElection ) :
                            if( 50 > game.getSorenRandNum(100, 'Revolt - cede power request') ):

                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking for change in leader from %s"%(pPlayer.getName()))
                                [newLeaderType,newLeaderName] = self.chooseRevolutionLeader( revCities )
                                
                                isElection = False
                                #buyOffCost = (50 + 15*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                                totalRevIdx = 0
                                for pCity in revCities :
                                    totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                                buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(80+10*pPlayer.getCurrentEra(),'Rev')
                                if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                                buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                                bodStr += '  ' + localText.getText("TXT_KEY_REV_CAP_LEAD_CEDE",()) + ' %s.'%(newLeaderName)
                                bodStr += '\n\n' + localText.getText("TXT_KEY_REV_CAP_LEAD_CONCLUSION",())
                                bodStr += '  %s '%(newLeaderName) + localText.getText("TXT_KEY_REV_CAP_LEAD_RULER",())

                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                                assert( len(revCities) > 0 )
                                self.makeRevolutionDecision( revCities, pPlayer, 'leader', peaceful, bodStr, newLeaderType = newLeaderType, newLeaderName = newLeaderName, isElection = isElection, buyOffCost = buyOffCost )

                                return

                        else :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking for %s to face election"%(pPlayer.getName()))
                            [newLeaderType,newLeaderName] = self.chooseRevolutionLeader( revCities )
                            
                            isElection = True
                            #buyOffCost = (30 + 10*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                            totalRevIdx = 0
                            for pCity in revCities :
                                totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                            buyOffCost = totalRevIdx/(25-pPlayer.getCurrentEra()) + game.getSorenRandNum(90+10*pPlayer.getCurrentEra(),'Rev')
                            if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                            buyOffCost = max( [buyOffCost,pPlayer.getGold()/12 + game.getSorenRandNum(50,'Rev')] )
                            bodStr += '  ' + localText.getText("TXT_KEY_REV_CAP_LEAD_ELECTION",()) + ' %s!'%(newLeaderName)
                            bodStr += '\n\n' + localText.getText("TXT_KEY_REV_PEACEFUL_CONCLUSION",())
                            bodStr += '\n\n' + localText.getText("TXT_KEY_REV_CAP_LEAD_BUYOFF",())

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                            assert( len(revCities) > 0 )
                            self.makeRevolutionDecision( revCities, pPlayer, 'leader', peaceful, bodStr, newLeaderType = newLeaderType, newLeaderName = newLeaderName, isElection = isElection, buyOffCost = buyOffCost )

                            return
                
            else :
                # If violent, demand change of leader!
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Capital or majority of cities in violent revolution!")

                if( self.leaderRevolution and len(revCities) > 2 ) :
                    if( not pPlayer.isHuman() or self.humanLeaderRevolution ) :

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Demanding change in leader from %s"%(pPlayer.getName()))

                        [newLeaderType,newLeaderName] = self.chooseRevolutionLeader( revCities )
                        
                        if( pPlayer.isStateReligion() ) :
                            giveRelType = pPlayer.getStateReligion()
                        else :
                            giveRelType = None

                        [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( revCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = True, giveRelType = giveRelType, bMatchCivics = True )

                        bodStr += ' ' + localText.getText("TXT_KEY_REV_CAP_RULE_DEMAND",())
                        if( not RevUtils.canDoElection( iPlayer ) ) :
                            isElection = False
                            buyOffCost = -1
                            bodStr += '  ' + localText.getText("TXT_KEY_REV_CAP_RULE_CEDE",()) + ' %s.'%(newLeaderName)
                            bodStr += '\n\n' + localText.getText("TXT_KEY_REV_CAP_RULE_TERRIBLE",())
                            bodStr += ' %s '%(pRevPlayer.getCivilizationDescription(0)) + localText.getText("TXT_KEY_REV_CAP_RULE_FIGHT",())
                            bodStr += '  %s '%(newLeaderName) + localText.getText("TXT_KEY_REV_CAP_LEAD_RULER",())

                        else :
                            isElection = True
                            #buyOffCost = (50 + 12*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                            totalRevIdx = 0
                            for pCity in revCities :
                                totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                            buyOffCost = totalRevIdx/(22-pPlayer.getCurrentEra()) + game.getSorenRandNum(80+10*pPlayer.getCurrentEra(),'Rev')
                            if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                            buyOffCost = max( [buyOffCost,pPlayer.getGold()/8 + game.getSorenRandNum(50,'Rev')] )
                            bodStr += '  ' + localText.getText("TXT_KEY_REV_CAP_RULE_ELECTION",()) + ' %s!'%(newLeaderName)
                            bodStr += '\n\n' + localText.getText("TXT_KEY_REV_CAP_RULE_RISE",()) + ' %s '%(pRevPlayer.getCivilizationDescription(0))
                            bodStr += localText.getText("TXT_KEY_REV_CAP_RULE_BUYOFF",())

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                        assert( len(revCities) > 0 )
                        self.makeRevolutionDecision( revCities, pPlayer, 'leader', peaceful, bodStr, pRevPlayer = pRevPlayer, isJoinWar = isJoinWar, newLeaderType = newLeaderType, newLeaderName = newLeaderName, isElection = isElection, buyOffCost = buyOffCost )
                        
                        return

#-------- Default to ask/demand independence
        if( not self.independenceRevolution ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - WARNING: default ask for independence has been disabled!")
            return
        
        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Default: ask/demand independence!")

        # Prune for only close cities?  Cities in area may be quite far away
        indCities = list()
        for pCity in revCities :
            cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
            # Must be within 2 dists of instigator
            if( cityDist <= 2*self.closeRadius ) :
                for iCity in indCities :
                    cityDist = min([cityDist, plotDistance( pCity.getX(), pCity.getY(), iCity.getX(), iCity.getY() )])
                # Must be close to another revolutionary
                if( cityDist <= self.closeRadius ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is close enough to join in independence quest"%(pCity.getName()))
                    indCities.append(pCity)                

        bodStr = localText.getText("TXT_KEY_REV_CITY_OF",()) + ' ' + instigator.getName()
        if( len(indCities) > 1 ) :
            bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
            for pCity in indCities[1:len(indCities)-1] :
                bodStr += ' %s,'%(pCity.getName())
            bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + indCities[-1].getName() + ' ' + localText.getText("TXT_KEY_REV_ARE",()) + ' '
        else :
            bodStr += ' ' + localText.getText("TXT_KEY_REV_IS",())

        buyOffCost = -1

        if( peaceful ) :
            if( pPlayer.isStateReligion() ) :
                if( 70 > game.getSorenRandNum(100,'Rev') ) :
                    giveRelType = pPlayer.getStateReligion()
                else :
                    giveRelType = -1
            else :
                if( 50 > game.getSorenRandNum(100,'Rev') ) :
                    giveRelType = None
                else :
                    giveRelType = -1
            [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( indCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = True, giveRelType = giveRelType, bMatchCivics = True )
            #buyOffCost = (100 + 20*pPlayer.getCurrentEra())*len(indCities) + game.getSorenRandNum(100+20*pPlayer.getCurrentEra(),'Rev')
            totalRevIdx = 0
            totalPop = 0
            for pCity in indCities :
                totalRevIdx += RevUtils.getCityVal(pCity, 'RevolutionIndex')
                totalPop += pCity.getPopulation()
            buyOffCost = totalRevIdx/(17-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
            if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
            buyOffCost = max( [buyOffCost,pPlayer.getGold()/8 + game.getSorenRandNum(50,'Rev')] )

            # Determine Vassal or no
            # if annoyed or worse, ask for independence
            # if tiny or really like, ask capitulation
            # if
            vassalStyle = None
            if( not pTeam.isAVassal() and pTeam.isVassalStateTrading() ) :
                if( pRevPlayer.AI_getAttitude(pPlayer.getID()) == AttitudeTypes.ATTITUDE_FURIOUS or pRevPlayer.AI_getAttitude(pPlayer.getID()) == AttitudeTypes.ATTITUDE_ANNOYED ) :
                    vassalStyle = None
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Vassal style: No due to attitude")
                elif( totalPop < pPlayer.getTotalPopulation()/3 ) :
                    if( pRevPlayer.AI_getAttitude(pPlayer.getID()) == AttitudeTypes.ATTITUDE_FRIENDLY or pRevPlayer.AI_getAttitude(pPlayer.getID()) == AttitudeTypes.ATTITUDE_PLEASED ) :
                        vassalStyle = 'capitulated'
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Vassal style %s chosen from size and attitude"%(vassalStyle))
                    elif( totalPop < pPlayer.getTotalPopulation()/6 ) :
                        vassalStyle = 'capitulated'
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Vassal style %s chosen from size"%(vassalStyle))
                    else :
                        vassalStyle = 'free'
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Vassal style %s chosen from size"%(vassalStyle))
                elif( pRevPlayer.AI_getAttitude(pPlayer.getID()) == AttitudeTypes.ATTITUDE_FURIOUS or pRevPlayer.AI_getAttitude(pPlayer.getID()) == AttitudeTypes.ATTITUDE_FURIOUS ) :
                    vassalStyle = 'free'
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Vassal style %s chosen from attitude"%(vassalStyle))
                           
            if( not vassalStyle == None ) :
                bodStr += ' ' + localText.getText("TXT_KEY_REV_IND_PEACE_VASSAL",())
                bodStr += ' %s '%(pRevPlayer.getCivilizationDescription(0)) + localText.getText("TXT_KEY_REV_IND_PEACE_VASSAL",())
                if( vassalStyle == 'free' ) :
                    bodStr += ' %s '%(localText.getText("TXT_KEY_REV_VASSAL_FREE",()))
                else :
                    bodStr += ' %s '%(localText.getText("TXT_KEY_REV_VASSAL_CAP",()))
                bodStr += localText.getText("TXT_KEY_REV_IND_PEACE_GRANT",())
            else :
                bodStr += ' ' + localText.getText("TXT_KEY_REV_IND_PEACE_REQUEST",()) + ' %s.'%(pRevPlayer.getCivilizationDescription(0))
            bodStr += '\n\n' + localText.getText("TXT_KEY_REV_IND_PEACE_ACTION",())
            
        else :
            [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( indCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = True )
            vassalStyle = None
            if( self.allowSmallBarbRevs and len(indCities) == 1 ) :
                if( instRevIdx < int(1.4*self.revInstigatorThreshold) ) :
                    pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                    isJoinWar = False
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Small, disorganized Revolution")
            bodStr += ' ' + localText.getText("TXT_KEY_REV_IND_VIOLENT_DEMAND",())
            if( pRevPlayer.isBarbarian() ) :
                bodStr += " " + localText.getText("TXT_KEY_REV_IND_VIOLENT_BARB",())
            else :
                bodStr += ' ' + localText.getText("TXT_KEY_REV_IND_VIOLENT_FORM",()) + ' %s.'%(pRevPlayer.getCivilizationDescription(0))
            bodStr += '\n\n' + localText.getText("TXT_KEY_REV_IND_VIOLENT_FIGHT",())

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(indCities),buyOffCost))
        assert( len(indCities) > 0 )
        self.makeRevolutionDecision( indCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer = pRevPlayer, isJoinWar = isJoinWar, buyOffCost = buyOffCost, vassalStyle = vassalStyle )

        return


######################################################################

    def chooseRevolutionCiv( self, cityList, bJoinCultureWar = True, bReincarnate = True, bJoinRebels = True, pNotThisCiv = None, giveTechs = True, giveRelType = -1, bMatchCivics = False ) :
        # All cities should have same owner

        if( self.bRebelTypes ) :
            RebelTypes.setup()
        
        pRevPlayer = None
        isJoinWar = False

        owner = gc.getPlayer( cityList[0].getOwner() )
        ownerTeam = gc.getTeam( owner.getTeam() )

        if( game.countCivPlayersAlive() >= self.maxCivs ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No available slots, spawning as Barbarians")
            pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
            return [pRevPlayer, isJoinWar]

        # Attempt to find a worthy civ to reincarnate in any of these cities
        for pCity in cityList :

            if( pRevPlayer == None ) :

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Looking for revolution worthy civ in %s"%(pCity.getName()))

                cultPlayer = None
                if( pCity.countTotalCultureTimes100() > 50*100 ) :
                    cultPlayer = gc.getPlayer( pCity.findHighestCulture() )
                    if( cultPlayer.getID() == owner.getID() ) :
                        cultPlayer = None

                if( bJoinCultureWar and not cultPlayer == None ) :
                    # If at war with significant culture, join them
                    if( ownerTeam.getAtWarCount(True) > 0 ) :
                        if( ownerTeam.isAtWar(cultPlayer.getID()) and cultPlayer.isAlive() ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Owner at war with dominant culture player %s"%(cultPlayer.getCivilizationDescription(0)))
                            pRevPlayer = cultPlayer
                            if( cultPlayer.getNumCities() > 1 ) :
                                isJoinWar = True

                if( bJoinRebels and pRevPlayer == None ) :
                    # If at war with this cities rebel civ type, join them
                    revCivType = RevUtils.getCityVal(pCity, 'RevolutionCiv')
                    if( revCivType >= 0 ) :
                        for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                            if( not i == owner.getID() ) :
                                playerI = gc.getPlayer( i )
                                if( playerI.isAlive() and playerI.getCivilizationType() == revCivType ) :
                                    if( ownerTeam.isAtWar(playerI.getTeam()) ) :
                                        pRevPlayer = playerI
                                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Owner at war with cities revolt civ type, player %s"%(pRevPlayer.getCivilizationDescription(0)))

                if( bReincarnate and pRevPlayer == None ) :
                    # Check for civ that can rise from the ashes
                    for i in range(0,gc.getMAX_CIV_PLAYERS()) :       
                        if( not i == owner.getID() ) :
                            playerI = gc.getPlayer( i )
                            if( (not playerI.isAlive()) and (pCity.getCulture( i ) > 50) ) :
                                pRevPlayer = playerI
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Reincarnating player %s"%(pRevPlayer.getCivilizationDescription(0)))
                                break
                            
        # Search around all cities for a rebellion that wants to spill over into this territory
        if( bJoinRebels and pRevPlayer == None ) :
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Checking for rebellions that could spill over")

            rebelIDList = list()
            for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( not i == owner.getID() and pCity.area().getUnitsPerPlayer(i) > 0 ) :
                    playerI = gc.getPlayer(i)
                    teamI = gc.getTeam( playerI.getTeam() )
                    relations = playerI.AI_getAttitude(owner.getID())

                    if( teamI.canDeclareWar(ownerTeam.getID()) ) :   
                        if( not playerI.isFoundedFirstCity() ) :  # Is a homeless rebel
                            if( playerI.getCivilizationType() == RevUtils.getCityVal(pCity, 'RevolutionCiv') ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Homeless rebel (type) %s in area"%(playerI.getCivilizationDescription(0)))
                                rebelIDList.append(i)
                            elif( teamI.isAtWar(ownerTeam.getID()) ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Homeless rebel (at war) %s in area"%(playerI.getCivilizationDescription(0)))
                                rebelIDList.append(i)
                            elif( relations == AttitudeTypes.ATTITUDE_FURIOUS or relations == AttitudeTypes.ATTITUDE_ANNOYED ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Homeless rebel (attitude) %s in area"%(playerI.getCivilizationDescription(0)))
                                rebelIDList.append(i)

                        if( playerI.getCitiesLost() < 3 and playerI.getNumCities() < 4 ) :
                            if( game.getGameTurn() - playerI.getCapitalCity().getGameTurnAcquired() < 30 and not playerI.getCapitalCity().getPreviousOwner() == gc.getBARBARIAN_PLAYER() ) :
                                if( playerI.getCivilizationType() == RevUtils.getCityVal(pCity, 'RevolutionCiv') ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Young rebel (type) %s in area"%(playerI.getCivilizationDescription(0)))
                                    rebelIDList.append(i)
                                elif( teamI.isAtWar(ownerTeam.getID()) ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Young rebel (at war) %s in area"%(playerI.getCivilizationDescription(0)))
                                    rebelIDList.append(i)
                                elif( relations == AttitudeTypes.ATTITUDE_FURIOUS or relations == AttitudeTypes.ATTITUDE_ANNOYED ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Young rebel (attitude) %s in area"%(playerI.getCivilizationDescription(0)))
                                    rebelIDList.append(i)
                                
            if( len(rebelIDList) > 0 ) :
                for pCity in cityList :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Checking around %s for other rebellion"%(pCity.getName()))
                
                    for [radius,plotI] in RevUtils.plotGenerator( pCity.plot(), 6 ) :
                        for rebelID in rebelIDList :
                            if( plotI.getNumDefenders(rebelID) > 0 ) :
                                pRevPlayer = gc.getPlayer(rebelID)
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Found %s within %d, expanding their rebellion!"%(pRevPlayer.getCivilizationDescription(0),radius))
                                break

                        if( not pRevPlayer == None ) :
                            break

                    if( not pRevPlayer == None ) :
                            break
                            
        # Search around all cities in list for a dead player to reincarnate
        if( pRevPlayer == None and bReincarnate ) :
            if( game.countCivPlayersAlive() < game.countCivPlayersEverAlive() ) :

                deadCivs = list()
                for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                    if( not gc.getPlayer(idx).isAlive() and gc.getPlayer(idx).isEverAlive() ) :
                        deadCivs.append(idx)

                for pCity in cityList :
                    if( pRevPlayer == None ) :
                        for civIdx in deadCivs :
                            playerI = gc.getPlayer(civIdx)
                            if( playerI.getCivilizationType() == RevUtils.getCityVal(pCity, 'RevolutionCiv') ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Reincarnation %s's rev civ, the %s"%(pCity.getName(),playerI.getCivilizationDescription(0)))
                                pRevPlayer = playerI
                                break
                        
                for pCity in cityList :
                    if( pRevPlayer == None ) :
                        
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Checking around %s for dead player"%(pCity.getName()))
                        maxCult = 0
                        maxCultRad = 5
                        for [radius,plotI] in RevUtils.plotGenerator( pCity.plot(), 4 ) :

                            if( not pRevPlayer == None and radius > maxCultRad ) :
                                break
                                                            
                            for civIdx in deadCivs :
                                if( plotI.getCulture(civIdx) > maxCult ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("Revolt: Found plot culture from dead player %d"%(civIdx))
                                    maxCult = plotI.getCulture(civIdx)
                                    maxCultRad = radius
                                    pRevPlayer = gc.getPlayer(civIdx)

        # Create new civ based on culture/owner of first city in list
        if( pRevPlayer == None ) :

            pCity = cityList[0]
            owner = gc.getPlayer( pCity.getOwner() )
            ownerTeam = gc.getTeam( owner.getTeam() )
            
            # Search for empty slot
            newPlayerIdx = -1
            # Overwrite a civ that never managed to found a city and hasn't lost a city (probably a previous rebel)
            for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( (not gc.getPlayer(i).isAlive()) and (not gc.getPlayer(i).isFoundedFirstCity()) and gc.getPlayer(i).getCitiesLost() == 0 and (gc.getPlayer(i).getNumUnits() == 0) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Dead player %d has founded city? %d Num cities lost: %d"%(i,gc.getPlayer(i).isFoundedFirstCity(),gc.getPlayer(i).getCitiesLost()))
                    newPlayerIdx = i
                    break

            if( newPlayerIdx < 0 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No available slots, spawning as Barbarians")
                pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                return [pRevPlayer, isJoinWar]

            # Create list of available civs and similar civ types
            cultPlayer = None
            if( pCity.countTotalCultureTimes100() > 50*100 ) :
                cultPlayer = gc.getPlayer( pCity.findHighestCulture() )
                if( cultPlayer.getID() == owner.getID() ) :
                    cultPlayer = None
            
            # Don't incarnate as either of these
            iMinor = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_MINOR')
            iBarbarian = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_BARBARIAN')
            # Civs not currently in the game
            availableCivs = list()
            # Civs with similar style to cultOwner, if they exist
            similarStyleCivs = list()
            similarOwnerStyleCivs = list()
            for civType in range(0,gc.getNumCivilizationInfos()) :
                if( not civType == iBarbarian ) :
                    if( not civType == iMinor ) :
                        taken = False
                        for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                            if( civType == gc.getPlayer(i).getCivilizationType() ) :
                                if( gc.getPlayer(i).isAlive() or gc.getPlayer(i).isFoundedFirstCity() or gc.getPlayer(i).getCitiesLost() > 0 ) :
                                    taken = True
                                    break
                        if( not taken ) :
                            availableCivs.append(civType)
                            if( not cultPlayer == None ) :
                                if( gc.getCivilizationInfo( cultPlayer.getCivilizationType() ).getArtStyleType() == gc.getCivilizationInfo(civType).getArtStyleType() ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Potential similar style civ from culture: %s (%d)"%(gc.getCivilizationInfo(civType).getShortDescription(0),civType))
                                    similarStyleCivs.append(civType)
                            if( gc.getCivilizationInfo( owner.getCivilizationType() ).getArtStyleType() == gc.getCivilizationInfo(civType).getArtStyleType() ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Potential similar style civ from owner: %s (%d)"%(gc.getCivilizationInfo(civType).getShortDescription(0),civType))
                                similarOwnerStyleCivs.append(civType)

            if( len(availableCivs) < 1 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No available civs, spawning as Barbarians")
                pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                return [pRevPlayer, isJoinWar]

            newCivIdx = None
            
            # If city has a revolutionary civ type, if that type is available choose it
            if( RevUtils.getCityVal(pCity, 'RevolutionCiv') in availableCivs ) :
                # City previously rebelled as available civ type
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Respawning previous rebel civ type for this city")
                newCivIdx = RevUtils.getCityVal(pCity, 'RevolutionCiv')
            else :
                chosenCivs = list()
                try :
                    if( not cultPlayer == None ) :
                        shortListType = cultPlayer.getCivilizationType()
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Using cultural owner %s for short list"%(gc.getCivilizationInfo(shortListType).getShortDescription(0)))
                    else :
                        shortListType = owner.getCivilizationType()
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - looking up %d in list of length %d"%(shortListType,len(RebelTypes.RebelTypeList)))
                    rebList = RebelTypes.RebelTypeList[shortListType]
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d civs on short list for this type"%(len(rebList)))
                    for civType in rebList :
                        if( civType in availableCivs ) :
                            chosenCivs.append(civType)

                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Found %d civs available from short list"%(len(chosenCivs)))
                except :
                    pass

                if( self.bRebelTypes and len(chosenCivs) > 0 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Creating a civ from short list")
                    availableCivs = chosenCivs
                else :
                    if( self.bArtStyleTypes ) :
                        if( len(similarStyleCivs) > 0 ) :
                            # Create a civ using style of cultural owner of city
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Creating a similar style civ to cultural owner %s"%(cultPlayer.getCivilizationDescription(0)))
                            availableCivs = similarStyleCivs
                        elif( len(similarOwnerStyleCivs) > 0 ) :
                            # Create a civ using style of owner of city
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Creating a similar style civ to owner %s"%(owner.getCivilizationDescription(0)))
                            availableCivs = similarOwnerStyleCivs
                        
                newCivIdx = availableCivs[game.getSorenRandNum(len(availableCivs),'Revolution: pick unused civ type')]

            leaderList = list()
            for leaderType in range(0,gc.getNumLeaderHeadInfos()) :
                if( gc.getCivilizationInfo(newCivIdx).isLeaders(leaderType) ) :
                    leaderList.append(leaderType)

            if( len(leaderList) < 1 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Unexpected lack of possible leaders, spawning as Barbarians")
                pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                return [pRevPlayer, isJoinWar]
            
            newLeaderIdx = leaderList[game.getSorenRandNum(len(leaderList),'Revolution: pick leader')]

            success = game.createNewPlayer( newPlayerIdx, newCivIdx, newLeaderIdx, -1, False, 1, 1 )

            if( not success ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - New civ creation failed, spawning as Barbarians")
                pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                return [pRevPlayer, isJoinWar]

            pRevPlayer = gc.getPlayer(newPlayerIdx)

            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Created the %s"%(pRevPlayer.getCivilizationDescription(0)))
            
        # Do special setup for non-living revolutionaries ...
        
        if( giveTechs and not pRevPlayer.isAlive() and not pRevPlayer.isBarbarian() ) :
            RevUtils.giveTechs( pRevPlayer, owner )

        if( not giveRelType == None and not pRevPlayer.isAlive() and not pRevPlayer.isBarbarian() ) :
            if( giveRelType >= 0 ) :
                # Give specified religion
                pRevPlayer.setLastStateReligion( giveRelType )
                if( self.LOG_DEBUG ) :
                    CvUtil.pyPrint("  Revolt - Setting revolutionary's religion to %d"%(giveRelType))
                    religionDesc = gc.getReligionInfo(giveRelType).getDescription()
                    CvUtil.pyPrint("  Revolt - Revolutionary's religion set to %s"%(religionDesc))
            else :
                # Give minority religion in city
                availRels = list()
                for relType in range(0,gc.getNumReligionInfos()) :
                    if( not relType == owner.getStateReligion() ) :
                        if( pCity.isHolyCityByType(relType) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is holy city for non-state religion %s"%(pCity.getName(), gc.getReligionInfo( relType ).getDescription()))
                            giveRelType = relType
                        elif( pCity.isHasReligion(relType) ) :
                            availRels.append( relType )

                if( giveRelType < 0 ) :
                    if( len(availRels) > 0 ) :
                        giveRelType = availRels[game.getSorenRandNum(len(availRels),'Revolution: pick religion')]

                if( giveRelType >= 0 ) :
                    pRevPlayer.setLastStateReligion( giveRelType )
                    if( self.LOG_DEBUG ) :
                        CvUtil.pyPrint("  Revolt - Setting revolutionary's religion to %d"%(giveRelType))
                        religionDesc = gc.getReligionInfo(giveRelType).getDescription()
                        CvUtil.pyPrint("  Revolt - Revolutionary's religion set to %s"%(religionDesc))

        if( bMatchCivics ) :
            pPlayer = gc.getPlayer( pCity.getOwner() )
            for civicOptionID in range(0,gc.getNumCivicOptionInfos()) :
                #civicOption = gc.getCivicOptionInfo(civicOptionID)
                civicType = pPlayer.getCivics(civicOptionID)
                if( pRevPlayer.canDoCivics( civicType ) ) :
                    pRevPlayer.setCivics( civicOptionID, civicType )

        return [pRevPlayer, isJoinWar]


    def chooseRevolutionLeader( self, cityList ) :

        newLeaderType = None
        newLeaderName = None

        owner = gc.getPlayer( cityList[0].getOwner() )
        ownerCivType = owner.getCivilizationType()
        ownerLeaderType = owner.getLeaderType()
        ownerCivInfo = gc.getCivilizationInfo( ownerCivType )

        # Use new leader type
        count = 0
        availLeader = list()
        for i in range(0,gc.getNumLeaderHeadInfos()) :
            if( ownerCivInfo.isLeaders(i) ) :
                count += 1
                if( not i == ownerLeaderType ) :
                    availLeader.append(i)

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Found %d available from %d for civ"%(len(availLeader),count))

        if( len(availLeader) > 0 ) :
            newLeaderType = availLeader[game.getSorenRandNum(len(availLeader),'Revolution: pick leader')]
            newLeaderName = gc.getLeaderHeadInfo( newLeaderType ).getDescription()
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Switching from leader type %d to %d (%s)"%(ownerLeaderType,newLeaderType,newLeaderName))

        if( newLeaderType == None ) :
            # Use same leader type, but with new name
            # Hack for right now
            newLeaderType = ownerLeaderType
            newLeaderName = gc.getLeaderHeadInfo( newLeaderType ).getDescription()
            if( newLeaderName == owner.getName() ) :
                newLeaderName = newLeaderName.encode('latin-1') + ' II'
            else :
                newLeaderName = newLeaderName.encode('latin-1')
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No new leader available, using current type %d (%s)"%(newLeaderType,newLeaderName))

        return [newLeaderType,newLeaderName]            

    def makeRevolutionDecision( self, cityList, pPlayer, revType, peaceful, bodStr, pRevPlayer = None, isJoinWar = False, joinPlayer = None, newLeaderType = None, newLeaderName = None, isElection = True, newCivic = None, newReligion = None, buyOffCost = -1, vassalStyle = None ) :

        pTeam = gc.getTeam( pPlayer.getTeam() )

        iAggressive = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGGRESSIVE')
        iSpiritual = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_SPIRITUAL')
        numRevCities = len(cityList)

        if( revType == 'civics' ) :
            if( newCivic == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no civic specified")
                return

            if( peaceful ) :
                if( numRevCities > pPlayer.getNumCities()/2 ) :
                    odds = 70
                elif( numRevCities < pPlayer.getNumCities()/4 ) :
                    odds = 25
                else :
                    odds = 40
            else :
                if( numRevCities > pPlayer.getNumCities()/2 ) :
                    odds = 80
                elif( numRevCities < pPlayer.getNumCities()/4 ) :
                    odds = 40
                else :
                    odds = 60

            odds += self.config.getint("Revolution","CivicsOdds",0)

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on civics")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.odds = odds
                self.buyOffCost = buyOffCost
                self.vassalStyle = vassalStyle

                # Additions by Caesium et al
                caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                caesiumtextResolution = caesiumtR.split('x')
                caesiumpasx = int(caesiumtextResolution[0])/10
                caesiumpasy = int(caesiumtextResolution[1])/10
                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                # End additions by Caesium et al
                if( peaceful ) :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_PEACEFUL",()) )
                else :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_VIOLENT",()) )
                if( odds >= 70 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_ACCEPT",())
                elif( odds <= 30 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_REJECT",())
                else :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_NEUTRAL",())
                popup.setBodyString( bodStr )
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_ACCEPT",()) )
                buttons = ('accept',)
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_REJECT",()) )
                buttons += ('reject',)
                if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_BUYOFF",()) + ' %d'%(buyOffCost) )
                    buttons += ('buyoff',)

                popup.setUserData( buttons )

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on civics")

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))

                if( game.getSorenRandNum(100,'Revolt: switch civics') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)
                    
                else :
                    if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                        if( peaceful ) :
                            base = .8
                        else :
                            base = .7
                        buyOdds = int( odds/2 + 75*(1.0 - pow(base,pPlayer.getGold()/buyOffCost)) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyoff odds are %d, with base %f, from cost %d and gold %d"%(buyOdds,base,buyOffCost,pPlayer.getGold()))
                        if( buyOdds > game.getSorenRandNum( 100, 'Rev - AI buyoff decision' ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyingoff rebels")
                            self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, buyOffCost = buyOffCost, didBuyOff = True)
                            return
                    
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

        elif( revType == 'religion' ) :
            if( newReligion == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no religion specified")
                return

            if( not pPlayer.isStateReligion() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI has no state religion ... this shouldn't happen")
                return

            newCount = pPlayer.getHasReligionCount( newReligion )
            newHoly = pPlayer.hasHolyCity( newReligion )

            stateCount = pPlayer.getHasReligionCount( pPlayer.getStateReligion() )
            stateHoly = pPlayer.hasHolyCity( pPlayer.getStateReligion() )

            if( stateHoly and not newHoly ) :
                odds = 0
            elif( newHoly and not stateHoly ) :
                if( pPlayer.hasTrait( iSpiritual ) ) :
                    # Spiritual player will spread religion pretty quickly
                    if( stateCount > 2*newCount ) :
                        odds = 60
                    else :
                        odds = 100
                else :
                    if( stateCount > 2*newCount ) :
                        odds = 30
                    elif( newCount > stateCount ) :
                        odds = 100
                    else :
                        odds = 70
            else :
                if( stateCount > 2*newCount ) :
                    odds = 0
                elif( newCount > stateCount ) :
                    odds = 70
                else :
                    odds = 30

            odds += self.config.getint("Revolution","ReligionOdds",0)

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on religion change")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.odds = odds
                self.buyOffCost = buyOffCost
                self.vassalStyle = vassalStyle

                # Additions by Caesium et al
                caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                caesiumtextResolution = caesiumtR.split('x')
                caesiumpasx = int(caesiumtextResolution[0])/10
                caesiumpasy = int(caesiumtextResolution[1])/10
                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                # End additions by Caesium et al
                if( peaceful ) :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_PEACEFUL",()) )
                else :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_VIOLENT",()) )
                if( odds >= 70 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_ACCEPT",())
                elif( odds <= 30 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_REJECT",())
                else :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_NEUTRAL",())
                popup.setBodyString( bodStr )
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_ACCEPT",()) )
                buttons = ('accept',)
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_REJECT",()) )
                buttons += ('reject',)
                if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_BUYOFF",()) + ' %d'%(buyOffCost) )
                    buttons += ('buyoff',)

                popup.setUserData( buttons )

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on religion change")

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))

                if( game.getSorenRandNum(100,'Revolt: switch religion') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

                else :
                    if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                        if( peaceful ) :
                            base = .8
                        else :
                            base = .7
                        buyOdds = int( odds/2 + 75*(1.0 - pow(base,pPlayer.getGold()/buyOffCost)) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyoff odds are %d, with base %f, from cost %d and gold %d"%(buyOdds,base,buyOffCost,pPlayer.getGold()))
                        if( buyOdds > game.getSorenRandNum( 100, 'Rev - AI buyoff decision' ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyingoff rebels")
                            self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, buyOffCost = buyOffCost, didBuyOff = True)
                            return
                        
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

        elif( revType == 'leader' ) :
            if( newLeaderType == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no leader specified")
                return

            if( not newLeaderName == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Potential new leader: %s"%(newLeaderName))

            # Compute approval rating (formula from demographics info screen)
            if (pPlayer.calculateTotalCityHappiness() > 0) :
                iHappiness = int((1.0 * pPlayer.calculateTotalCityHappiness()) / (pPlayer.calculateTotalCityHappiness() + \
                                    pPlayer.calculateTotalCityUnhappiness()) * 100)
            else :
                iHappiness = 100

            # Adjusted approval rating based on num cities in revolt
            if( numRevCities > pPlayer.getNumCities()/2 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Approval rating (initially %d) adjusted due to %d of %d cities revolting"%(iHappiness,numRevCities,pPlayer.getNumCities()))
                iHappiness = int( iHappiness - 100*(numRevCities - pPlayer.getNumCities()/2)/(2.0*pPlayer.getNumCities()) )
                iHappiness = max([iHappiness,25])
                
            # Other potential factors:
            # Player rank

            if( numRevCities > pPlayer.getNumCities()/2 ) :
                odds = 50
                if( peaceful ) :
                    odds -= 20
                else :
                    if( pTeam.getAtWarCount(True) > 1 ) :
                        odds += 10
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 30
            elif( numRevCities < pPlayer.getNumCities()/4 ) :
                odds = 25
                if( peaceful ) :
                    odds -= 20
                else :
                    if( pTeam.getAtWarCount(True) > 1 ) :
                        odds += 10
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 20
            else :
                odds = 35
                if( peaceful ) :
                    odds -= 30
                else :
                    if( pTeam.getAtWarCount(True) > 1 ) :
                        odds += 10
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 20

            if( isElection ) :
                if( iHappiness > 55 ) :
                    odds += 40
                elif( iHappiness < 40 ) :
                    odds -= 30
                elif( iHappiness < 50 ) :
                    odds -= 15
                else :
                    odds += 10

            odds += self.config.getint("Revolution","LeaderOdds",0)

            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Adjusted approval rating is %d"%(iHappiness))

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on leader change")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.iHappiness = iHappiness
                self.odds = odds
                self.buyOffCost = buyOffCost
                self.vassalStyle = vassalStyle

                # Additions by Caesium et al
                caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                caesiumtextResolution = caesiumtR.split('x')
                caesiumpasx = int(caesiumtextResolution[0])/10
                caesiumpasy = int(caesiumtextResolution[1])/10
                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                # End additions by Caesium et al
                if( peaceful ) :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_PEACEFUL",()) )
                else :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_VIOLENT",()) )
                if( odds >= 70 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_ACCEPT",())
                elif( odds <= 30 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_REJECT",())
                else :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_NEUTRAL",())
                if( isElection ) :
                    bodStr += "  " + localText.getText("TXT_KEY_REV_ADVISOR_APPROVAL",()) + " %d."%(iHappiness)
                    if( iHappiness < 40 ) :
                        bodStr += "  " + localText.getText("TXT_KEY_REV_ADVISOR_ELEC_LOSE",())
                popup.setBodyString( bodStr )
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_ACCEPT",()) )
                buttons = ('accept',)
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_REJECT",()) )
                buttons += ('reject',)
                if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                    if( isElection ) :
                        popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_BUY_ELECTION",()) + ' %d'%(buyOffCost) )
                        buttons += ('buyelection',)
                    else :
                        popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_BUYOFF",()) + ' %d'%(buyOffCost) )
                        buttons += ('buyoff',)

                popup.setUserData(buttons)

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on leader change")

                if( game.getSorenRandNum(100,'Revolt: switch leader') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, iHappiness = iHappiness, buyOffCost = buyOffCost)

                else:
                    if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                        if( peaceful ) :
                            base = .8
                        else :
                            base = .7
                        buyOdds = int( odds/2 + 75*(1.0 - pow(base,pPlayer.getGold()/buyOffCost)) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyoff odds are %d, with base %f, from cost %d and gold %d"%(buyOdds,base,buyOffCost,pPlayer.getGold()))
                        if( buyOdds > game.getSorenRandNum( 100, 'Rev - AI buyoff decision' ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyingoff rebels")
                            self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, buyOffCost = buyOffCost, didBuyOff = True)
                            return
                        
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

        elif( revType == 'war' ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making decision on crusade")

            victim = pRevPlayer

            [odds,attackerTeam,victimTeam] = RevUtils.computeWarOdds( pPlayer, victim, cityList[0].area(), allowAttackerVassal = False, allowBreakVassal = self.bAllowBreakVassal )

            if( numRevCities > pPlayer.getNumCities()/2 ) :
                odds += 50

            elif( numRevCities < pPlayer.getNumCities()/4 ) :
                odds += 20

            else :
                odds += 30

            odds += self.config.getint("Revolution","CrusadeOdds",0)

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on crusade")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.odds = odds
                self.buyOffCost = buyOffCost
                self.vassalStyle = vassalStyle

                # Additions by Caesium et al
                caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                caesiumtextResolution = caesiumtR.split('x')
                caesiumpasx = int(caesiumtextResolution[0])/10
                caesiumpasy = int(caesiumtextResolution[1])/10
                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                # End additions by Caesium et al
                if( peaceful ) :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_PEACEFUL",()) )
                else :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_VIOLENT",()) )
                if( odds >= 70 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_ACCEPT",())
                elif( odds <= 30 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_REJECT",())
                else :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_NEUTRAL",())
                
                popup.setBodyString( bodStr )
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_ACCEPT",()) )
                buttons = ('accept',)
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_REJECT",()) )
                buttons += ('reject',)

                popup.setUserData( buttons )

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on crusade")
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))

                if( game.getSorenRandNum(100,'Revolt: crusade') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

                else:
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)


        elif( revType == 'independence' ) :

            if( numRevCities > pPlayer.getNumCities()/2 ) :
                odds = 30
                if( peaceful ) :
                    odds -= 30
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 10
                if( pTeam.getAtWarCount(True) > 1 ) :
                    odds += 10
                if( not joinPlayer == None ) :
                    if( pTeam.isAtWar(joinPlayer.getTeam()) ) :
                        if( peaceful ) :
                            odds -= 40
                        else :
                            odds -= 20
            elif( numRevCities < pPlayer.getNumCities()/4 ) :
                if( numRevCities == 1 ) :
                    odds = 20
                else :
                    odds = 40
                if( peaceful ) :
                    odds -= 20
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 10
                if( not cityList[0].area().getID() == pPlayer.getCapitalCity().area().getID() ) :
                    odds += 20
                if( pTeam.getAtWarCount(True) > 1 ) :
                    odds += 20
                if( not joinPlayer == None ) :
                    if( pTeam.isAtWar(joinPlayer.getTeam()) ) :
                        if( peaceful ) :
                            odds -= 40
                        else :
                            odds -= 30
                if( cityList[0].getNumRevolts(pPlayer.getID()) > 2 ) :
                    odds += 10
                    if( vassalStyle == 'capitulated' ) :
                        odds += 15

                if( not vassalStyle == None ) :
                    odds += 10
                
            else :
                odds = 30
                if( peaceful ) :
                    odds -= 20
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 10
                if( not cityList[0].area().getID() == pPlayer.getCapitalCity().area().getID() ) :
                    odds += 20
                if( pTeam.getAtWarCount(True) > 1 ) :
                    odds += 10
                if( not joinPlayer == None ) :
                    if( pTeam.isAtWar(joinPlayer.getTeam()) ) :
                        if( peaceful ) :
                            odds -= 30
                        else :
                            odds -= 20
                if( cityList[0].getNumRevolts(pPlayer.getID()) > 2 ) :
                    odds += 10
                    if( vassalStyle == 'capitulated' ) :
                        odds += 20
                    elif( not vassalStyle == None ) :
                        odds += 10

            odds += self.config.getint("Revolution","IndependenceOdds",0)

            if( joinPlayer == None and pRevPlayer.isBarbarian() ) :
                odds = 0

            if( numRevCities == pPlayer.getNumCities() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Will not surrender all cities")
                odds = 0

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on independence")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.odds = odds
                self.buyOffCost = buyOffCost
                self.vassalStyle = vassalStyle

                # Additions by Caesium et al
                caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                caesiumtextResolution = caesiumtR.split('x')
                caesiumpasx = int(caesiumtextResolution[0])/10
                caesiumpasy = int(caesiumtextResolution[1])/10
                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                # End additions by Caesium et al
                if( peaceful ) :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_PEACEFUL",()) )
                else :
                    popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_VIOLENT",()) )
                if( odds >= 70 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_ACCEPT",())
                elif( odds <= 30 ) :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_REJECT",())
                else :
                    bodStr += "\n\n" + localText.getText("TXT_KEY_REV_ADVISOR_NEUTRAL",())
                
                popup.setBodyString( bodStr )
                if( not vassalStyle == None ) :
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_VASSAL",()) )
                    buttons = ('vassal',)
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_INDEPENDENCE",()) )
                    buttons = ('accept',)
                else :
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_ACCEPT",()) )
                    buttons = ('accept',)
                popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_REJECT",()) )
                buttons += ('reject',)
                if( not joinPlayer == None and not isJoinWar and gc.getTeam(pPlayer.getTeam()).canDeclareWar(joinPlayer.getTeam()) ) :
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_WAR",()) )
                    buttons += ('war',)
                if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_BUYOFF",()) + ' %d'%(buyOffCost) )
                    buttons += ('buyoff',)
                if( peaceful and joinPlayer == None and (pRevPlayer.getNumCities() == 0) ) :
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_CONTROL",()) )
                    buttons += ('control',)
                elif( not peaceful and joinPlayer == None and (pRevPlayer.getNumCities() == 0) ) :
                    popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_CONTROL",()) )
                    buttons += ('control',)

                popup.setUserData( buttons )

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on independence")
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))

                if( game.getSorenRandNum(100,'Revolt: give independence') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, vassalStyle = vassalStyle)

                else:
                    if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                        if( peaceful ) :
                            base = .8
                        else :
                            base = .7
                        buyOdds = int( odds/2 + 75*(1.0 - pow(base,pPlayer.getGold()/buyOffCost)) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyoff odds are %d, with base %f, from cost %d and gold %d"%(buyOdds,base,buyOffCost,pPlayer.getGold()))
                        if( buyOdds > game.getSorenRandNum( 100, 'Rev - AI buyoff decision' ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyingoff rebels")
                            self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, buyOffCost = buyOffCost, didBuyOff = True)
                            return
                        
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

        else :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Unknown revolution type %s"%(revType))

    def revolutionPopupHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Handling revolution popup")

        newHumanPlayer = None
        termsAccepted = False
        didBuyOff = False
        vassalStyle = None
        buttonLabel = netUserData[popupReturn.getButtonClicked()]
        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Button %d pressed, label: %s"%(popupReturn.getButtonClicked(),buttonLabel))

        if( buttonLabel == 'accept' ) :
            termsAccepted = True
        elif( buttonLabel == 'reject' ) :
            termsAccepted = False
        elif( buttonLabel == 'buyoff' ) :
            # Pay the cities not to revolt
            termsAccepted = False
            didBuyOff = True
        elif( buttonLabel == 'vassal' ) :
            # pRevPlayer becomes vassal of pPlayer
            termsAccepted = True
            vassalStyle = self.vassalStyle
        elif( buttonLabel == 'control' ) :
            # Give control of new civ
            termsAccepted = True
            newHumanPlayer = self.pRevPlayer.getID()
        elif( buttonLabel == 'buyelection' ) :
            # Attempt buyoff of election results
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Feature not fully implemented %s"%(buttonLabel))
            didBuyOff = True
            termsAccepted = False
        elif( buttonLabel == 'war' ) :
            # Declare war on cultural owner of rebelling cities
            termsAccepted = False
            self.pRevPlayer = self.joinPlayer
            self.joinPlayer = None
            gc.getTeam(self.pPlayer.getTeam()).declareWar( self.pRevPlayer.getTeam(), True )
        else :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error! Unrecognized button label %s"%(buttonLabel))
            termsAccepted = False

        self.processRevolution( self.cityList, self.pPlayer, self.revType, self.peaceful, termsAccepted, self.pRevPlayer, self.isJoinWar, self.joinPlayer, self.newLeaderType, self.newLeaderName, self.isElection, self.newCivic, self.newReligion, self.iHappiness, self.buyOffCost, didBuyOff, vassalStyle )

        if( not newHumanPlayer == None ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Human being given control of new player %d"%(newHumanPlayer))
            RevUtils.changeHuman( newHumanPlayer, iPlayerID )

        self.cityList = None
        self.pPlayer = None
        self.revType = None
        self.peaceful = None
        self.pRevPlayer = None
        self.isJoinWar = None
        self.joinPlayer = None
        self.newLeaderType = None
        self.newLeaderName = None
        self.isElection = None
        self.newCivic = None
        self.newReligion = None
        self.iHappiness = None
        self.odds = None
        self.buyOffCost = None
        self.vassalStyle = None

    def processRevolution( self, cityList, pPlayer, revType, peaceful, termsAccepted = False, pRevPlayer = None, isJoinWar = False, joinPlayer = None, newLeaderType = None, newLeaderName = None, isElection = True, newCivic = None, newReligion = None, iHappiness = None, buyOffCost = -1, didBuyOff = False, vassalStyle = None ) :

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Processing revolution")
        if( self.DEBUG_MESS ) : CyInterface().addImmediateMessage('Processing revolution!!!',"")

        pTeam = gc.getTeam( pPlayer.getTeam() )

        iAggressive = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGGRESSIVE')
        iSpiritual = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_SPIRITUAL')
        numRevCities = len(cityList)
        capital = pPlayer.getCapitalCity()
        capitalArea = capital.area().getID()

        for pCity in cityList :
            pCity.changeNumRevolts( pPlayer.getID(), 1 )
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has now revolted %d times"%(pCity.getName(),pCity.getNumRevolts(pPlayer.getID())))

        if( didBuyOff ) :
            if( revType == 'leader' and isElection ) :
                # TODO something special for this
                pass
            pPlayer.changeGold( -buyOffCost )
            for pCity in cityList :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Buying off revolutionaries in %s"%(pCity.getName()))
                revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                RevUtils.updateCityVal( pCity, 'RevolutionIndex', min([int(self.revReadyFrac*self.revInstigatorThreshold),revIdx-200]) )
                RevUtils.updateCityVal( pCity, 'RevolutionCounter', self.buyoffTurns )
                RevUtils.updateCityVal( pCity, 'ReinforcementCounter', 0 )
                pCity.changeNumRevolts( pPlayer.getID(), -1 )
            return

        if( revType == 'civics' ) :
            if( newCivic == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no civic specified")
                return

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts to switch, lowering rev indices")
                # Do switch
                newCivicOption = gc.getCivicInfo( newCivic ).getCivicOptionType()
                pPlayer.setCivics( newCivicOption, newCivic )
                iSpiritual = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_SPIRITUAL')
                pPlayer.changeRevolutionTimer(5)
                if( not pPlayer.hasTrait(iSpiritual) ) :
                    if( pPlayer.getCurrentEra() > gc.getNumEraInfos()/2 ) :
                        pPlayer.changeAnarchyTurns(2)
                    else :
                        pPlayer.changeAnarchyTurns(1)
                # TODO: figure out way to not allow immeadiate switch back

                # Make those cities happier
                for pCity in cityList :
                    revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                    RevUtils.updateCityVal( pCity, 'RevolutionIndex', min([revIdx/2,600]) )
                    RevUtils.updateCityVal( pCity, 'RevolutionCounter', self.acceptedTurns )
                    RevUtils.updateCityVal( pCity, 'ReinforcementCounter', 0 )
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player declines to switch")
                if( peaceful ) :
                                    
                    revIdxInc = 250
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")
                    # Make those cities more unhappy
                    for pCity in cityList :
                        if( not pCity.area().getID() == capitalArea ) :
                            RevUtils.changeCityVal( pCity, 'RevolutionIndex', revIdxInc + 50 )
                        else :
                            RevUtils.changeCityVal( pCity, 'RevolutionIndex', revIdxInc )
                        RevUtils.updateCityVal( pCity, 'RevolutionCounter', self.deniedTurns )

                else :
                    if( isJoinWar and not joinPlayer == None ) :
                        self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, isJoinWar )
                    else :
                        self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

        elif( revType == 'religion' ) :
            if( newReligion == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no religion specified")
                return

            if( not pPlayer.isStateReligion() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player has no state religion ... this shouldn't happen")
                return

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts to switch, lowering rev indices")
                # Do switch
                pPlayer.convert(newReligion)
                # Make those cities happier
                for pCity in cityList :
                    revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                    RevUtils.updateCityVal(pCity, 'RevolutionIndex', min([revIdx/2,600]) )
                    RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.acceptedTurns )
                    RevUtils.updateCityVal(pCity, 'ReinforcementCounter', 0 )
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player declines to switch")
                if( not peaceful ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error ... shouldn't have violent request for religion switch")
                    
                revIdxInc = 300
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")
                # Make those cities more unhappy
                for pCity in cityList :
                    if( not pCity.area().getID() == capitalArea ) :
                        RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc + 50 )
                    else :
                        RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc )
                    RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.deniedTurns )                        

        elif( revType == 'leader' ) :
            if( newLeaderType == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no leader specified")
                return

            if( not newLeaderName == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Potential new leader: %s"%(newLeaderName))

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts to cede power, lowering rev indices")

                # Lower indices
                for pCity in cityList :
                    revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                    RevUtils.updateCityVal(pCity, 'RevolutionIndex', min([revIdx/2,600]) )
                    RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.acceptedTurns )
                    RevUtils.updateCityVal(pCity, 'ReinforcementCounter', 0 )
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

                # Do switch leader
                if( isElection ) :
                    if( iHappiness == None ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - ERROR!  Happiness not set")
                        iHappiness = 50
                    
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Running an election for a new leader!")
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Adjusted approval rating is %d"%(iHappiness))

                    if( iHappiness < 40 ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Election lost due to low approval rating")
                        # Continue to change leader code below
                    elif( game.getSorenRandNum(25,'Revolution: election results') + 30 > iHappiness ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Election lost by probability")
                    else :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Election won!  Leader stays in power!")
                        if( self.showHumanPopup(pPlayer.getID()) ) :
                            # Report election victory
                            # Additions by Caesium et al
                            caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                            caesiumtextResolution = caesiumtR.split('x')
                            caesiumpasx = int(caesiumtextResolution[0])/10
                            caesiumpasy = int(caesiumtextResolution[1])/10
                            popup = PyPopup.PyPopup()
                            if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                            # End additions by Caesium et al
                            bodStr = localText.getText("TXT_KEY_REV_HUMAN_ELEC_VICTORY",())
                            popup.setBodyString(bodStr)
                            popup.launch()
                        return

                if( self.showHumanPopup(pPlayer.getID()) ) :
                    # Additions by Caesium et al
                    caesiumtR = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
                    caesiumtextResolution = caesiumtR.split('x')
                    caesiumpasx = int(caesiumtextResolution[0])/10
                    caesiumpasy = int(caesiumtextResolution[1])/10
                    popup = PyPopup.PyPopup(self.controlLostPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL)
                    if( self.centerPopups ) : popup.setPosition(3*caesiumpasx,3*caesiumpasy)
                    # End additions by Caesium et al
                    gameSpeedMod = (661.0)/(game.getEstimateEndTurn() + 1.0)
                    if( gameSpeedMod == 661.0 ) : gameSpeedMod = 1.0
                    if( isElection ) :
                        iNumTurns = int( gameSpeedMod*15 )
                        bodStr = localText.getText("TXT_KEY_REV_HUMAN_ELEC_LOSS",())
                    else :
                        iNumTurns = int( gameSpeedMod*24 )
                        bodStr = localText.getText("TXT_KEY_REV_HUMAN_CEDE",())

                    bodStr += '\n\n' + localText.getText("TXT_KEY_REV_HUMAN_CONTROL_RETURNED",()) + ' %d '%(iNumTurns) + localText.getText("TXT_KEY_REV_TURNS",()) + '.'
                    popup.setBodyString( bodStr )
                    popup.setUserData( (iNumTurns,) )
                    popup.launch()

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Changing leader to %s"%(newLeaderName))

                if( newLeaderType == pPlayer.getLeaderType() ) :
                    # Just change the name of leader
                    pPlayer.setName( newLeaderName.encode('latin-1') )
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No leader change, leader's of same type")
                    if( pPlayer.canContact(game.getActivePlayer()) ) :
                        CyInterface().addImmediateMessage( '%s '%(newLeaderName) + localText.getText("TXT_KEY_REV_MESS_NEW_LEADER",()) + ' %s!'%(pPlayer.getCivilizationDescription(0)), "" )

                else :
                    if( pPlayer.getID() == game.getActivePlayer() ) :
                        # Don't need to save this as player can't exit while in automation
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Skipping leader change for human")
                        #self.humanLeaderType = pPlayer.getLeaderType()
                    else :
                        if( pPlayer.canContact(game.getActivePlayer()) ) :
                            CyInterface().addImmediateMessage( '%s '%(newLeaderName) + localText.getText("TXT_KEY_REV_MESS_NEW_LEADER",()) + ' %s!'%(pPlayer.getCivilizationDescription(0)), "" )
                        RevUtils.changeCiv(pPlayer.getID(),pPlayer.getCivilizationType(),newLeaderType)

                if( isElection ) :
                    pPlayer.changeAnarchyTurns(1)
                elif( pPlayer.getCurrentEra() > gc.getNumEraInfos()/2 ) :
                    pPlayer.changeAnarchyTurns(3)
                else :
                    pPlayer.changeAnarchyTurns(2)

            else:
                if( isElection ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player refuses election")
                else :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player clings to power")
                
                if( peaceful ) :
                    revIdxInc = 300
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")
                    for pCity in cityList :
                        if( not pCity.area().getID() == capitalArea ) :
                            RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc + 50 )
                        else :
                            RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc )
                        RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.deniedTurns ) 

                else :
                    if( isJoinWar and not joinPlayer == None ) :
                        if( not pRevPlayer.isBarbarian() ) :
                            joinPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -2 )
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                            pPlayer.AI_changeAttitudeExtra( joinPlayer.getID(), -2 )
                        self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, isJoinWar )
                        
                    else :
                        if( not pRevPlayer.isBarbarian() ) :
                            pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -5 )
                            if( not joinPlayer == None ) : pRevPlayer.AI_changeAttitudeExtra( joinPlayer.getID(), 5 )
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                            pPlayer.AI_changeAttitudeExtra( pRevPlayer.getID(), -3 )
                        self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

        elif( revType == 'war' ) :

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts for war!!!")
                
                for pCity in cityList :
                    revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                    RevUtils.updateCityVal(pCity, 'RevolutionIndex', min([revIdx/2,500]) )
                    RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.acceptedTurns )
                    RevUtils.updateCityVal(pCity, 'ReinforcementCounter', 0 )
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

                pTeam.declareWar( pRevPlayer.getTeam(), True )

                # spawn fanatic units or something for holy war?

            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player declines cries for war.")

                if( peaceful ) :
                    revIdxInc = 300
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")
                    # Make those cities more unhappy
                    for pCity in cityList :
                        if( not pCity.area().getID() == capitalArea ) :
                            RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc + 50 )
                        else :
                            RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc )
                        RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.deniedTurns )

                else :
                    # Violent uprising!!!
                    if( isJoinWar and not joinPlayer == None ) :
                        self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, isJoinWar )
                    else :
                        if( not joinPlayer == None ) :
                            joinPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -3 )
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(joinPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),joinPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                            
                            # Check if joinPlayer would like to declare war on player
                            [warOdds,attackerTeam,victimTeam] = RevUtils.computeWarOdds( joinPlayer, pPlayer, cityList[0].area(), False, allowBreakVassal = self.bAllowBreakVassal )
                            # TODO: Find way to support human selecting war in this case
                            if( attackerTeam.isHuman() ) :
                                warOdds = 0

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - War odds are %d"%(warOdds))

                            if( warOdds > 25 and warOdds > game.getSorenRandNum(100,'Revolution: war') ) :
                                # have joinPlayer declare war on player
                                #joinTeam = gc.getTeam( joinPlayer.getTeam() )
                                #joinTeam.declareWar(pPlayer.getID(),True)
                                attackerTeam.declareWar(pPlayer.getID(),True)
                                isJoinWar = True

                                self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, True )

                            else :
                                if( not pRevPlayer.isBarbarian() ) :
                                    pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -5 )
                                    pRevPlayer.AI_changeAttitudeExtra( joinPlayer.getID(), 5 )
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                                    pPlayer.AI_changeAttitudeExtra( pRevPlayer.getID(), -3 )

                                self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

                        else :
                            if( not pRevPlayer.isBarbarian() ) :
                                pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -5 )
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                                pPlayer.AI_changeAttitudeExtra( pRevPlayer.getID(), -3 )
                            
                            self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )
                
        elif( revType == 'independence' ) :

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts to give independence, lowering rev indices")
                
                # Lower indices (makes life a little easier for new civ ...)
                for pCity in cityList :
                    revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
                    RevUtils.updateCityVal(pCity, 'RevolutionIndex', min([revIdx/2,600]) )
                    # Other changes handled by acquired city logic
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

                if( joinPlayer == None ) :
                    # Grant independence
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are taking over the revolutionary cities"%(pRevPlayer.getCivilizationDescription(0)))
                    bIsBarbRev = pRevPlayer.isBarbarian()

                    pRevTeam = gc.getTeam( pRevPlayer.getTeam() )

                    if( not bIsBarbRev ) :
                        if( not pRevTeam.isAlive() ) :
                            if( pRevTeam.isAtWar(pPlayer.getTeam()) ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Reincarnated %s ending war with %s"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0)))
                                pRevTeam.makePeace( pPlayer.getTeam() )

                        pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), 3 )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))

                    # Since instigator is first in list, it will become capital
                    for pCity in cityList :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Acquiring %s, pop %d"%(pCity.getName(),pCity.getPopulation()))
                        # Move units out of city?  Handled by trade style

                        if( pRevTeam.isAtWar(pPlayer.getTeam()) and pPlayer.getNumCities() > len(cityList) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Moving owner's units")
                            RevUtils.clearOutCity( pCity, pPlayer, pRevPlayer )
                            
                        [iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, pRevPlayer, pPlayer )

                        RevUtils.updateCityVal(pCity, 'RevolutionCiv', -1 )

                        # Store building types in city
                        buildingClassList = list()
                        for buildingType in range(0,gc.getNumBuildingInfos()) :
                            if( pCity.isHasRealBuilding(buildingType) ) :
                                buildingInfo = gc.getBuildingInfo(buildingType)
                                buildingClassList.append(buildingInfo.getBuildingClassType())
                            
                        cityPlot = pCity.plot()
                        try :
                            pCity.plot().setOwner( pRevPlayer.getID() )
                        except :
                            CvUtil.pyPrint("  Revolt - Error!  Attempted to give %s to %s, failed. **************"%(pCity.getName(),pRevPlayer.getCivilizationDescription(0)))
                            continue
                        pCity = cityPlot.getPlotCity()

                        # Should buildings stay or some destroyed?
                        for buildingClass in buildingClassList :
                            buildingType = gc.getCivilizationInfo(pRevPlayer.getCivilizationType()).getCivilizationBuildings(buildingClass)
                            if( not pCity.isHasRealBuilding(buildingType) ) :
                                buildingInfo = gc.getBuildingInfo(buildingType)
                                if( not buildingInfo.isGovernmentCenter() ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Building %s saved"%(buildingInfo.getDescription()))
                                    pCity.setHasRealBuilding( buildingType, True )

                        #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s at %d, %d"%(pCity.getName(),pCity.getX(),pCity.getY()))

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Culture in %s: %d, plot %d"%(pCity.getName(),pCity.getCulture(pPlayer.getID()),pCity.plot().getCulture(pPlayer.getID())))

                        newCulVal = int( self.revCultureModifier*max([pCity.getCulture(pPlayer.getID()),pCity.countTotalCultureTimes100()/200]) )
                        newPlotVal = int( self.revCultureModifier*max([pCity.plot().getCulture(pPlayer.getID()),pCity.plot().countTotalCulture()/2]) )
                        RevUtils.giveCityCulture( pCity, pRevPlayer.getID(), newCulVal, newPlotVal, overwriteHigher = False )

                        # Give a few defenders
                        ix = pCity.getX()
                        iy = pCity.getY()
                        pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                        #pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                        pRevPlayer.initUnit( iCounter, ix, iy, UnitAITypes.NO_UNITAI )
                        pRevPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )                            
                        if( pRevPlayer.getNumCities() <= 2 ) :
                            # Extra units for first two cities
                            pRevPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                            pRevPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                        elif( gc.getTeam(pPlayer.getTeam()).isAtWar(pRevPlayer.getTeam()) ) :
                            pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                            #pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )

                        if( not bIsBarbRev ) : RevUtils.updateCityVal(pCity, 'RevolutionCiv', pRevPlayer.getCivilizationType() )

                    if( not pRevPlayer.isAlive() ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Setting new player alive")
                        pRevPlayer.setNewPlayerAlive(True)

                    if( not bIsBarbRev and not vassalStyle == None ) :
                        if( vassalStyle == 'capitulated' ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is forming as capitulated vassal to %s"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0)))
                            pTeam.assignVassal( pRevPlayer.getTeam(), True )
                        else :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is forming as vassal to %s"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0)))
                            pTeam.assignVassal( pRevPlayer.getTeam(), False )

                    if( peaceful ) :
                        pTeam.signOpenBorders( pRevPlayer.getTeam() )

                else :
                    # Join cultural owner
                    if( self.showHumanPopup(pPlayer.getID()) ) :
                        # Both tests needed for AutoPlay and multiplayer

                        self.cityList = cityList
                        self.pPlayer = pPlayer
                        self.revType = revType
                        self.peaceful = peaceful
                        self.pRevPlayer = pRevPlayer
                        self.isJoinWar = isJoinWar
                        self.joinPlayer = joinPlayer
                        self.newLeaderType = newLeaderType
                        self.newLeaderName = newLeaderName
                        self.isElection = isElection
                        self.newCivic = newCivic
                        self.newReligion = newReligion
                        self.buyOffCost = buyOffCost

                        # Offer human the option of accepting the cities
                        PyPopup.PyPopup( self.joinHumanPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                        
                        popup.setHeaderString( localText.getText("TXT_KEY_REV_TITLE_GOOD_NEWS",()) )
                        
                        bodStr = localText.getText("TXT_KEY_REV_CITIZENS_OF",()) + ' %s'%(cityList[0].getName())
                        if( len(cityList) > 1 ) :
                            bodStr += localText.getText("TXT_KEY_REV_ALONG_WITH",())
                            for city in cityList[1:len(cityList)-1]:
                                bodStr += ', %s'%(city.getName())
                            bodStr += ' ' + localText.getText("TXT_KEY_REV_AND",()) + ' ' + cityList[-1].getName()

                        bodStr += ' ' + localText.getText("TXT_KEY_REV_HUMAN_GRANTED",()) + ' %s!'%(pPlayer.getName())
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_HUMAN_REGAIN",()) + ' %s.'%(pRevPlayer.getCivilizationDescription(0))
                        bodStr += '  ' + localText.getText("TXT_KEY_REV_HUMAN_JOIN_CHOICE",())
                        
                        popup.setBodyString( bodStr )
                        popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_WELCOME",()) )
                        buttons = ('welcome')
                        popup.addButton( localText.getText("TXT_KEY_REV_BUTTON_NOT_WANTED",()) )
                        buttons += ('goaway',)

                        # Center camera on city
                        CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                        
                        popup.launch(bCreateOkButton = False)

                    else :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are claiming their cultural cities"%(joinPlayer.getCivilizationDescription(0)))

                        bIsBarbRev = joinPlayer.isBarbarian()

                        if( not bIsBarbRev ) :
                            # Improve relations
                            joinPlayer.AI_changeAttitudeExtra( pPlayer.getID(), 2 )
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(joinPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),joinPlayer.AI_getAttitudeExtra(pPlayer.getID())))

                        pRevTeam = gc.getTeam( pRevPlayer.getTeam() )
                        
                        for pCity in cityList :
                            
                            # Move units out of city
                            if( pRevTeam.isAtWar(pPlayer.getTeam()) and pPlayer.getNumCities() > len(cityList) ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Moving owner's units")
                                RevUtils.clearOutCity( pCity, pPlayer, pRevPlayer )
                                
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Acquiring %s, pop %d"%(pCity.getName(),pCity.getPopulation()))

                            [iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, joinPlayer, joinPlayer )

                            RevUtils.updateCityVal(pCity, 'RevolutionCiv', -1 )

                            # Acquire city
                            #joinPlayer.acquireCity( pCity, False, True )
                            cityPlot = pCity.plot()
                            pCity.plot().setOwner( joinPlayer.getID() )
                            pCity = cityPlot.getPlotCity()

                            # Give a few defenders
                            ix = pCity.getX()
                            iy = pCity.getY()
                            joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                            #joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                            joinPlayer.initUnit( iCounter, ix, iy, UnitAITypes.NO_UNITAI )
                            if( joinPlayer.getNumCities() <= 2 ) :
                                # Extra units for first two cities (unlikely in this case)
                                joinPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )
                                joinPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                                joinPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )

                            elif( gc.getTeam(pPlayer.getTeam()).isAtWar(joinPlayer.getTeam()) ) :
                                joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                                #joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )

                            if( not bIsBarbRev ) : RevUtils.updateCityVal(pCity, 'RevolutionCiv', joinPlayer.getCivilizationType() )

            else:
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player refuses to let civ walk")
                
                if( peaceful ) :
                    # Cities get more pissed off
                    revIdxInc = 300
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")

                    # Save civ type for instigator only
                    RevUtils.updateCityVal( cityList[0], 'RevolutionCiv', pRevPlayer.getCivilizationType() )
                    
                    for pCity in cityList :
                        if( not pCity.area().getID() == capitalArea ) :
                            RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc + 50 )
                        else :
                            RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc )
                        RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.deniedTurns )

                    if( not joinPlayer == None ) :
                        joinPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -2 )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(joinPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),joinPlayer.AI_getAttitudeExtra(pPlayer.getID())))

                else :
                    # Violent uprising!!!
                    if( isJoinWar and not joinPlayer == None ) :
                        self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, isJoinWar )
                    else :
                        if( not joinPlayer == None ) :
                            joinPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -3 )
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(joinPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),joinPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                            
                            # Check if joinPlayer would like to declare war on player
                            [warOdds,attackerTeam,victimTeam] = RevUtils.computeWarOdds( joinPlayer, pPlayer, cityList[0].area(), allowBreakVassal = self.bAllowBreakVassal )
                            # TODO: provide support for human war selection
                            if( attackerTeam.isHuman() ) :
                                warOdds = 0

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - War odds are %d"%(warOdds))

                            if( warOdds > 25 and warOdds > game.getSorenRandNum(100,'Revolution: war') ) :
                                # have joinPlayer declare war on player
                                #joinTeam = gc.getTeam( joinPlayer.getTeam() )
                                #joinTeam.declareWar(pPlayer.getTeam(),True)
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - joinPlayer's team decides to declare war on pPlayer's team!")
                                attackerTeam.declareWar(pPlayer.getTeam(),True)
                                isJoinWar = True

                                self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, True )

                            else :
                                # Check if pPlayer wants to declare war on joinPlayer
                                [warOdds,attackerTeam,victimTeam] = RevUtils.computeWarOdds( pPlayer, joinPlayer, cityList[0].area(), allowBreakVassal = self.bAllowBreakVassal )
                                if( attackerTeam.isHuman() ) :
                                    warOdds = 0

                                if( warOdds > 25 and warOdds > game.getSorenRandNum(100,'Revolution: war') ) :
                                    # pPlayer declares war on joinPlayer!!!
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - pPlayer's team decides to declare war on joinPlayer's team!")
                                    attackerTeam.declareWar(joinPlayer.getTeam(),True)

                                    self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, False )

                                else :
                                    # pRevPlayer revolts against pPlayer
                                    if( not pRevPlayer.isBarbarian() ) :
                                        pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -5 )
                                        pRevPlayer.AI_changeAttitudeExtra( joinPlayer.getID(), 5 )
                                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                                        pPlayer.AI_changeAttitudeExtra( pRevPlayer.getID(), -3 )

                                    self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

                        else :
                            if( not pRevPlayer.isBarbarian() ) :
                                pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -5 )
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                                pPlayer.AI_changeAttitudeExtra( pRevPlayer.getID(), -3 )
                            
                            self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

        else :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Unknown revolution type %s"%(revType))

        return

    def controlLostHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Handling human lost control of civ popup")

        iNumTurns = netUserData[0]
        game.setAIAutoPlay( iNumTurns )
                            

    def joinHumanHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Handling join human popup")

        buttonLabel = netUserData[popupReturn.getButtonClicked()]

        if( buttonLabel == 'welcome' ) :
            # Welcome back!
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revolutionaries welcomed with open arms")

            self.joinPlayer.AI_changeAttitudeExtra( self.pPlayer.getID(), 2 )
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(self.joinPlayer.getCivilizationDescription(0),self.pPlayer.getCivilizationDescription(0),self.joinPlayer.AI_getAttitudeExtra(self.pPlayer.getID())))

            pRevTeam = gc.getTeam( pRevPlayer.getTeam() )
            
            for pCity in self.cityList :
                # Move units out of city
                if( pRevTeam.isAtWar(pPlayer.getTeam()) and pPlayer.getNumCities() > len(cityList) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Moving owner's units")
                    RevUtils.clearOutCity( pCity, pPlayer, pRevPlayer )
                    
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Acquiring %s"%(pCity.getName()))

                [iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, self.joinPlayer, self.joinPlayer )

                RevUtils.updateCityVal(pCity, 'RevolutionCiv', -1 )

                # Acquire city
                #joinPlayer.acquireCity( pCity, False, True )
                cityPlot = pCity.plot()
                pCity.plot().setOwner( self.joinPlayer.getID() )
                pCity = cityPlot.getPlotCity()

                # Give a few defenders
                ix = pCity.getX()
                iy = pCity.getY()
                self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                #self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                self.joinPlayer.initUnit( iCounter, ix, iy, UnitAITypes.NO_UNITAI )
                if( self.joinPlayer.getNumCities() <= 2 ) :
                    # Extra units for first two cities (unlikely in this case)
                    self.joinPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )
                    self.joinPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                    self.joinPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                elif( gc.getTeam(self.pPlayer.getTeam()).isAtWar(self.joinPlayer.getTeam()) ) :
                    self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                    #self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )

                RevUtils.updateCityVal(pCity, 'RevolutionCiv', self.joinPlayer.getCivilizationType() )

        elif( buttonLabel == 'goaway' ) :
            # Go away!
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revolutionaries rebuffed, striking out on their own")

            self.processRevolution( self.cityList, self.pPlayer, self.revType, self.peaceful, True, self.pRevPlayer, False, None, self.newLeaderType, self.newLeaderName, self.isElection, self.newCivic, self.newReligion, self.iHappiness, self.buyOffCost )

        else :
            # WTF?
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Unexpected button label %s"%(buttonLabel))

        self.cityList = None
        self.pPlayer = None
        self.revType = None
        self.peaceful = None
        self.pRevPlayer = None
        self.isJoinWar = None
        self.joinPlayer = None
        self.newLeaderType = None
        self.newLeaderName = None
        self.isElection = None
        self.newCivic = None
        self.newReligion = None
        self.iHappiness = None
        self.odds = None
        self.buyOffCost = None


    def spawnRevolutionaries( self, cityList, pPlayer, pRevPlayer, isJoinWar = False ) :

        if( self.LOG_DEBUG ) :
            if( pPlayer.isBarbarian() ) :
                print "Error: spawning revs for barb player"

        bIsBarbRev = pRevPlayer.isBarbarian()

        if( gc.getPlayer(game.getActivePlayer()).canContact(pPlayer.getID()) ) :
            if( bIsBarbRev ) :
                if( game.getActivePlayer() == pPlayer.getID() ) :
                    mess = localText.getText("TXT_KEY_REV_MESS_YOU_BARB",())
                else :
                    mess = localText.getText("TXT_KEY_REV_MESS_VIOLENT",()) + ' ' + PyPlayer(pPlayer.getID()).getCivilizationName() + '!!!'
                    mess += "  " + localText.getText("TXT_KEY_REV_MESS_BARB",())
            else :
                if( game.getActivePlayer() == pPlayer.getID() ) :
                    if( pRevPlayer.isAlive() ) :
                        mess = localText.getText("TXT_KEY_REV_MESS_JOIN",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + ' ' + localText.getText("TXT_KEY_REV_MESS_YOU_JOIN",())
                    else :
                        mess = localText.getText("TXT_KEY_REV_BIG_THE",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + ' ' + localText.getText("TXT_KEY_REV_MESS_YOU_RISEN",())
                else :
                    mess = localText.getText("TXT_KEY_REV_MESS_VIOLENT",()) + ' ' + PyPlayer(pPlayer.getID()).getCivilizationName() + '!!!'
                    if( pRevPlayer.isAlive() ) :
                        mess += "  " + localText.getText("TXT_KEY_REV_MESS_JOIN",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + ' ' + localText.getText("TXT_KEY_REV_MESS_JOIN2",())
                    else :
                        mess += "  " + localText.getText("TXT_KEY_REV_BIG_THE",()) + ' ' + pRevPlayer.getCivilizationDescription(0) + ' ' + localText.getText("TXT_KEY_REV_MESS_RISEN",())
            CyInterface().addImmediateMessage(mess,"")

        if( not bIsBarbRev ) :
            pRevTeam = gc.getTeam( pRevPlayer.getTeam() )
            if( not pRevTeam.isAtWar(pPlayer.getTeam()) ) :
                pRevTeam.declareWar( pPlayer.getTeam(), True )
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s revolutionaries declare war on the %s!"%(pRevPlayer.getCivilizationAdjective(0),pPlayer.getCivilizationDescription(0)))
            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s revolutionaries join in the war against the %s!"%(pRevPlayer.getCivilizationAdjective(0),pPlayer.getCivilizationDescription(0)))

            if( pRevPlayer.getGold() < 50 ) :
                pRevPlayer.changeGold( 30 + game.getSorenRandNum(25*len(cityList),'Revolt: give gold') )
            else :
                pRevPlayer.changeGold( 10 + game.getSorenRandNum(15*len(cityList),'Revolt: give gold') )

            if( not isJoinWar and pRevPlayer.getNumCities() < 4 ) :
                RevUtils.revObjectInit( pRevPlayer, dict() )
                RevUtils.revObjectSetVal( pRevPlayer, 'RevolutionTurn', game.getGameTurn() )

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spawning %s revolutionaries!!!"%(pRevPlayer.getCivilizationAdjective(0)))

        # Additional ideas:
        # Who should the revs be able to contact?

        iGoodyMap = CvUtil.findInfoTypeNum(gc.getGoodyInfo,gc.getNumGoodyInfos(),'GOODY_MAP')
        iGeneral = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_GREAT_GENERAL')
        iSpy = CvUtil.findInfoTypeNum(gc.getUnitInfo,gc.getNumUnitInfos(),'UNIT_SPY')
        revIdxInc = 300
        
        for [cityIdx,pCity] in enumerate(cityList) :

            revIdx = RevUtils.getCityVal( pCity, 'RevolutionIndex' )
            localRevIdx = RevUtils.getCityVal(pCity, 'LocalRevIndex')
            ix = pCity.getX()
            iy = pCity.getY()

            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - In %s, with rev idx %d (%d local)"%(pCity.getName(),revIdx,localRevIdx))

            spawnableUnits = RevUtils.getUprisingUnitTypes( pCity, pRevPlayer, isJoinWar )
            [iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, pRevPlayer, pPlayer )

            if( len(spawnableUnits) < 1 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - ERROR!!! No rev units possible in %s"%(pCity.getName()))
                continue

            spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 2, iSpawnPlotOwner = pRevPlayer.getID(), bCheckForEnemy = True )
            if( len(spawnablePlots) == 0 ) :
                spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 3, iSpawnPlotOwner = pRevPlayer.getID(), bCheckForEnemy = True )
            if( len(spawnablePlots) == 0 ) :
                spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 3, iSpawnPlotOwner = -1, bCheckForEnemy = False )

            revSpawnLoc = None
            if( len(spawnablePlots) > 0 ) :
                revSpawnLoc = spawnablePlots[game.getSorenRandNum(len(spawnablePlots),'Revolution: Pick rev plot')]

            iNumUnits = int( self.strengthModifier*(1.7*revIdx/(1.0*self.revReadyFrac*self.revInstigatorThreshold) + 1) + .5 )
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Initial enlistment: %d"%(iNumUnits))
            if( cityIdx == 0 ) :
                # More for instigator
                iNumUnits += 1
            if( pRevPlayer.getCivilizationType() == RevUtils.getCityVal(pCity, 'RevolutionCiv') ) :
                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Repeat revolution, increasing enlistment")
                iNumUnits += 1 + game.getSorenRandNum( 3, 'Revolution: more units' )
            if( localRevIdx < 0 and iNumUnits > 4 ) :
                iNumUnits = 2 + iNumUnits/2
            elif( localRevIdx > self.badLocalThreshold ) :
                iNumUnits += 1

            if( pPlayer.getCurrentEra() < 4 ) :
                iNumUnits -= 4 - (pPlayer.getCurrentEra())


            localFactor = min([localRevIdx+2,1.5*self.badLocalThreshold])
            if( pRevPlayer.getCivilizationType() == RevUtils.getCityVal(pCity, 'RevolutionCiv') ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Repeat revolution, increasing enlistment")
                effPop = int(1.2*pCity.getPopulation()) + 1
            else :
                effPop = pCity.getPopulation() + 1
            popMod = (revIdx/(1.0*self.alwaysViolentThreshold))
            popMod = min([ popMod, pow(popMod-.1,.6), 2.0])
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Population modifier: %f on pop %d"%(popMod,effPop))
            iNumUnits2a = (popMod*(effPop))/2.0
            
            if( bIsBarbRev ) :
                iNumUnits2b = (localFactor/(2.0*self.badLocalThreshold))*pow(RevUtils.getNumDefendersNearPlot(ix,iy,pPlayer.getID()),.5)/2.0
            else :
                iNumUnits2b = (localFactor/(1.0*self.badLocalThreshold))*pow(RevUtils.getNumDefendersNearPlot(ix,iy,pPlayer.getID()),.5)/2.0
            if( iNumUnits2b < 0 ) :
                iNumUnits2b = max([iNumUnits2b,-2])
            iNumUnits2 = int( self.strengthModifier*(iNumUnits2a + iNumUnits2b) + .5 )
##            if( pRevPlayer.getCivilizationType() == RevUtils.getCityVal(pCity, 'RevolutionCiv') ) :
##                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Repeat revolution, increasing enlistment")
##                iNumUnits2 = int( iNumUnits2*(11 + game.getSorenRandNum( 3, 'Revolution: more units' ))/10.0 + .7 )

            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Old method: %d"%(iNumUnits))
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - New method from pop: %f, from troops: %f, total: %d"%(iNumUnits2a,iNumUnits2b,iNumUnits2))

            iNumUnits = iNumUnits2
            
            unitAdjust = max([3, pCity.getPopulation()/4, RevUtils.getNumDefendersNearPlot(ix,iy,pPlayer.getID())])
            if( isJoinWar ) :
                # Joining established player in war
                iNumUnits = min( [iNumUnits, pCity.getPopulation()/3, unitAdjust] )
                # Really small cities shouldn't be revolting, but just in case
                iNumUnits = max( [iNumUnits, 2] )
            elif( bIsBarbRev ) :
                # Lower max number of barb units
                iNumUnits = min( [iNumUnits - 2, pCity.getPopulation()/2, unitAdjust] )
                iNumUnits = max( [iNumUnits, 2] )
            else :
                #iNumUnits = min( [iNumUnits, pCity.getPopulation(), 2*unitAdjust] )
                iNumUnits = max( [iNumUnits, 2] )
                
            iNumDefenders = RevUtils.getNumDefendersNearPlot(ix,iy,pPlayer.getID())
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revised enlistment: %d, num defenders nearby: %d"%(iNumUnits,iNumDefenders))
            
            revControl = False
            # Determine who controls city, revs or player
            
            if( revSpawnLoc == None ) :
                # If no plot on which to spawn revs, they get city and owners units flee
                # What about isJoinWar case?
                revControl = True
            else :
                # Compare strength of revolution and garrison
                iRevStrength = iNumUnits
                if( (pCity.unhappyLevel(0) - pCity.happyLevel()) > 0 ) :
                    iRevStrength += 2
                if( isJoinWar ) :
                    iRevStrength -= 2
                if( bIsBarbRev ) :
                    iRevStrength -= 4
                if( pCity.isCapital() ) :
                    iRevStrength -= 1

                iGarrisonStrength = pCity.plot().getNumDefenders(pPlayer.getID()) + 1
                iGarrisonStrength = int( iGarrisonStrength*(110 + pCity.getBuildingDefense())/100.0 )

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Rev strength: %d,  Garrison strength: %d"%(iRevStrength,iGarrisonStrength))

                if( iRevStrength > iGarrisonStrength ) :
                    # Revolutionaries out muscle the city garrison and take control
                    revControl = True
                else :
                    # Spawn in countryside
                    revControl = False    
            
            if( revControl ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revs take control of %s (%d,%d)"%(pCity.getName(),pCity.getX(),pCity.getY()))
            
                # Turn off rebellious city capture logic, all components handled here
                RevUtils.updateCityVal(pCity, 'RevolutionCiv', -1 )
                
                # Run wounded soldiers out of Town
                iInjureMax = 50
                retreatPlots = RevUtils.getSpawnablePlots( ix, iy, pPlayer, bLand = True, bIncludePlot = False, bIncludeCities = True, bSameArea = True, iRange = 3, iSpawnPlotOwner = pPlayer.getID(), bCheckForEnemy = True )
                if( len(retreatPlots) == 0 ) :
                    iInjureMax = 60
                    retreatPlots = RevUtils.getSpawnablePlots( ix, iy, pPlayer, bLand = True, bIncludePlot = False, bIncludeCities = True, bSameArea = False, iRange = 4, iSpawnPlotOwner = -1, bCheckForEnemy = True )
                if( len(retreatPlots) == 0 ) :
                    iInjureMax = 70
                    # Try to move to another of players cities
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No nearby plots, trying move to another of players cities")
                    for otherCity in PyPlayer( pPlayer.getID() ).getCityList() :
                        pOtherCity = otherCity.GetCy()
                        if( not pOtherCity.getID() == pCity.getID() ) :
                            retreatPlots.append([pOtherCity.getX(),pOtherCity.getY()])
                            
                if( len(retreatPlots) > 0 ) :
                    retreatLoc = retreatPlots[game.getSorenRandNum(len(retreatPlots),'Revolution: Pick rev plot')]
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City garrison retreating to %d,%d"%(retreatLoc[0], retreatLoc[1]))
                else :
                    retreatLoc = None
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City garrison massacred (had nowhere to go)")
                    # pPlayer is about to die anyway

                # Defections?

                toRebelList = list()
                if( not retreatLoc == None ) :
                    RevUtils.moveEnemyUnits( ix, iy, pRevPlayer.getID(), retreatLoc[0], retreatLoc[1], iInjureMax = iInjureMax, bDestroyNonLand = False, bLeaveSiege = self.rebelCaptureSiege )
                    if( self.rebelEquipFrac > 0 ) :
                        unitList = RevUtils.getEnemyUnits( ix, iy, pRevPlayer.getID() )
                        for unit in unitList :
                            if( self.rebelEquipFrac > game.getSorenRandNum(100,'Revolution: give rebels equipment') ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Will be giving rebels %s"%(unit.getName()))
                                toRebelList.append( unit.getUnitType() )
                            else :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Destroying %s"%(unit.getName()))

                # Store building types in city
                buildingClassList = list()
                for buildingType in range(0,gc.getNumBuildingInfos()) :
                    if( pCity.isHasRealBuilding(buildingType) ) :
                        buildingInfo = gc.getBuildingInfo(buildingType)
                        buildingClassList.append(buildingInfo.getBuildingClassType())

                # Acquire city by cultural conversion
                #pRevPlayer.acquireCity( pCity, False, False )
                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Population of %s before is %d"%(pCity.getName(),pCity.getPopulation()))
                cityPlot = pCity.plot()
                pCity.plot().setOwner( pRevPlayer.getID() )
                pCity = cityPlot.getPlotCity()
                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Population of %s after is %d"%(pCity.getName(),pCity.getPopulation()))

                if( pCity.getPopulation() < 1 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error!  City %s is empty"%(pCity.getName()))

                if( pCity.getPopulation() > 3 + pPlayer.getCurrentEra() ) :
                    pCity.setOccupationTimer(3)
                else :
                    pCity.setOccupationTimer(2)

                newCulVal = int( self.revCultureModifier*max([pCity.getCulture(pPlayer.getID()),pCity.countTotalCultureTimes100()/200]) )
                newPlotVal = int( self.revCultureModifier*max([pCity.plot().getCulture(pPlayer.getID()),pCity.plot().countTotalCulture()/2]) )
                RevUtils.giveCityCulture( pCity, pRevPlayer.getID(), newCulVal, newPlotVal, overwriteHigher = False )

                newUnitList = list()

                # Remove the default given defenders
                for i in range(0,cityPlot.getNumUnits()) :
                    pUnit = cityPlot.getUnit(i)
                    if( pUnit.getOwner() == pRevPlayer.getID() ) :
                        pUnit.kill(False,-1)

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s, pop. %d has %d defenders after removing defaults"%(pCity.getName(),pCity.getPopulation(),pCity.plot().getNumDefenders(pCity.getOwner())))

                # Create stolen equipment
                for unitType in toRebelList :
                    newUnit = pRevPlayer.initUnit( unitType, ix, iy, UnitAITypes.NO_UNITAI )
                    newUnitList.append( newUnit )

                # Spawn revs in the city, plus an extra defender or two
                # Couple revs die in taking control of city:
                if( iNumUnits > 9 ) :
                    iNumUnits -= 2
                elif( iNumUnits > 5 ) :
                    iNumUnits -= 1
                    
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spawning %d units for city of size %d"%(iNumUnits,pCity.getPopulation()))

                for i in range(0,iNumUnits) :
                    newUnitID = spawnableUnits[game.getSorenRandNum( len(spawnableUnits), 'Revolution: pick unit' )]
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - In %s, spawning %s"%(pCity.getName(),PyInfo.UnitInfo(newUnitID).getDescription()))
                    newUnit = pRevPlayer.initUnit( newUnitID, ix, iy, UnitAITypes.NO_UNITAI )
                    newUnitList.append( newUnit )
                
                # Give a few extra defenders and a worker
                if( cityIdx == 0 ) :
                    newUnitList.append( pRevPlayer.initUnit(iBestDefender, ix, iy, UnitAITypes.NO_UNITAI) )
                pRevPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )
                if( pCity.getPopulation() > 5 ) :
                    pRevPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )

                # Injure units to simulate the lack of training in rebel troops
                for newUnit in newUnitList :
                    iDamage = 15 + game.getSorenRandNum(15,'Rev - Injure unit')
                    newUnit.setDamage( iDamage, pPlayer.getID() )

                if( pCity.getPopulation() > 4 and len(newUnitList) > 4 ) :
                    deltaPop = (len(newUnitList)/5)
                    pCity.changePopulation( -deltaPop )
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City population decreased by %d for %d rebel units spawned"%(deltaPop,len(newUnitList)))

                # Extra stuff for instigator city
                if( cityIdx == 0 and len(cityList) > 1 and not bIsBarbRev ) :
                    if( 10 + 3*len(cityList) + 5*pCity.getNumRevolts(pCity.getOwner()) < game.getSorenRandNum(100,'Rev') ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Great General (%d) spawned in %s"%(iGeneral,pCity.getName()))
                        pRevPlayer.initUnit( iGeneral, ix, iy, UnitAITypes.NO_UNITAI )

                    if( pRevPlayer.canTrain(iSpy,False,False) and pCity.getNumRevolts(pCity.getOwner()) > 2 ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spy spawned in %s"%(pCity.getName()))
                        pRevPlayer.initUnit( iSpy, ix, iy, UnitAITypes.NO_UNITAI )


                # Should buildings stay or some destroyed?
                for buildingClass in buildingClassList :
                    buildingType = gc.getCivilizationInfo(pRevPlayer.getCivilizationType()).getCivilizationBuildings(buildingClass)
                    if( not pCity.isHasRealBuilding(buildingType) ) :
                        buildingInfo = gc.getBuildingInfo(buildingType)
                        if( not buildingInfo.isGovernmentCenter() ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Building %s saved"%(buildingInfo.getDescription()))
                            pCity.setHasRealBuilding( buildingType, True )

                # Reveal surrounding countryside
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Giving map")
                pRevPlayer.receiveGoody( gc.getMap().plot(ix,iy), iGoodyMap, newUnitList[0] )
                pRevPlayer.receiveGoody( gc.getMap().plot(ix,iy), iGoodyMap, newUnitList[0] )

                # No more revolutions for a while
                RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.turnsBetweenRevs )
                RevUtils.updateCityVal(pCity, 'ReinforcementCounter', 0 )  # Should be handled by city acquired logic anyway ...
                RevUtils.updateCityVal(pCity, 'RevolutionTurn', game.getGameTurn() )
                if( not bIsBarbRev ) :
                    RevUtils.updateCityVal(pCity, 'RevolutionCiv', pRevPlayer.getCivilizationType() )
                
            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Owner keeps control of %s (%d,%d), revs spawning at %d,%d"%(pCity.getName(),ix,iy,revSpawnLoc[0],revSpawnLoc[1]))

                # City in disorder
                iTurns = revIdx/int(self.revReadyFrac*self.revInstigatorThreshold)
                if( localRevIdx > 4 ) :
                    iTurns = int(iTurns*1.5)
                pCity.setOccupationTimer( min([iTurns,12]) )
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City occupation timer set to %d"%(pCity.getOccupationTimer()))

                # Wound player's units?
                unitList = RevUtils.getEnemyUnits( ix, iy, pRevPlayer.getID() )
                for pUnit in unitList :
                    if( revIdx > self.revInstigatorThreshold ) :
                        pUnit.setDamage( 35 + game.getSorenRandNum(20,'Revolution: Wound units'), pRevPlayer.getID() )
                    else :
                        pUnit.setDamage( 20 + game.getSorenRandNum(20,'Revolution: Wound units'), pRevPlayer.getID() )

                # Defections?
                    
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spawning %d units for city of size %d"%(iNumUnits,pCity.getPopulation()))

                # Spawn rev units outside city
                newUnitList = list()
                for i in range(0,iNumUnits) :
                    newUnitID = spawnableUnits[game.getSorenRandNum( len(spawnableUnits), 'Revolution: pick unit' )]
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Outside of %s, spawning %s"%(pCity.getName(),PyInfo.UnitInfo(newUnitID).getDescription()))
                    newUnit = pRevPlayer.initUnit( newUnitID, revSpawnLoc[0], revSpawnLoc[1], UnitAITypes.NO_UNITAI )
                    newUnitList.append( newUnit )

                for newUnit in newUnitList :
                    # Injure units to simulate the lack of training in rebel troops
                    iDamage = 10 + game.getSorenRandNum(10,'Rev - Injure unit')
                    newUnit.setDamage( iDamage, pPlayer.getID() )

                    # Check AI settings
                    iniAI = newUnit.getUnitAIType()
                    if( iniAI == UnitAITypes.UNITAI_CITY_DEFENSE ) :
                        newUnit.setUnitAIType( UnitAITypes.UNITAI_ATTACK )
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s starting with AI type: %d (ini %d)"%(newUnit.getName(),newUnit.getUnitAIType(),iniAI))
                        

                if( pCity.getPopulation() > 4 and len(newUnitList) > 4 ) :
                    deltaPop = (len(newUnitList)/5)
                    pCity.changePopulation( -deltaPop )
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City population decreased by %d for %d rebel units spawned"%(deltaPop,len(newUnitList)))

                # Extra stuff for instigator city
                if( cityIdx == 0 and len(cityList) > 1 and not bIsBarbRev ) :
                    if( 3*len(cityList) + 5*pCity.getNumRevolts(pCity.getOwner()) < game.getSorenRandNum(100,'Rev') ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Great General spawned outside %s"%(pCity.getName()))
                        pRevPlayer.initUnit( iGeneral, revSpawnLoc[0], revSpawnLoc[1], UnitAITypes.NO_UNITAI )

                    if( pRevPlayer.canTrain(iSpy,False,False) and pCity.getNumRevolts(pCity.getOwner()) > 2 ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spy spawned outside %s"%(pCity.getName()))
                        pRevPlayer.initUnit( iSpy, revSpawnLoc[0], revSpawnLoc[1], UnitAITypes.NO_UNITAI )

                # Reveal surrounding countryside
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Giving map")
                pRevPlayer.receiveGoody( gc.getMap().plot(revSpawnLoc[0], revSpawnLoc[1]), iGoodyMap, newUnitList[0] )
                pRevPlayer.receiveGoody( gc.getMap().plot(revSpawnLoc[0], revSpawnLoc[1]), iGoodyMap, newUnitList[0] )
                
                if( not bIsBarbRev and revIdx > self.revInstigatorThreshold ) :
                    RevUtils.updateCityVal(pCity, 'RevolutionCiv', pRevPlayer.getCivilizationType() )
                    iReinforceTurns = self.baseReinforcementTurns - revIdx/self.revInstigatorThreshold - min([localRevIdx,12])/4
                    if( pCity.getPopulation() < 7 ) :
                        iReinforceTurns += 7 - pCity.getPopulation()
                        if( pCity.getPopulation() < 5 ) :
                            iReinforceTurns = max([iReinforceTurns,4 - (pCity.getPopulation()/2)])

                    minReinfTurns = self.minReinforcementTurns
                    if( pPlayer.getCurrentEra() < 4 ) :
                        minReinfTurns += 2 - pPlayer.getCurrentEra()/2
                    iReinforceTurns = max([iReinforceTurns,self.minReinforcementTurns])
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s reinforcement counter set to %d"%(pCity.getName(),iReinforceTurns))
                    RevUtils.updateCityVal(pCity, 'ReinforcementCounter', iReinforceTurns + 1 )

                RevUtils.changeCityVal(pCity, 'RevolutionIndex', revIdxInc )
                RevUtils.updateCityVal(pCity, 'RevolutionCounter', self.turnsBetweenRevs )
                RevUtils.updateCityVal(pCity, 'RevolutionTurn', game.getGameTurn() )

        if( not pRevPlayer.isAlive() ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Setting new player alive")
            pRevPlayer.setNewPlayerAlive(True)

        if( pPlayer.getNumCities() == 0 ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Homeland has no more cities!  Setting founded city to false to keep civil war alive")
            pPlayer.setFoundedFirstCity( False )

        return
